/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: CloudAdapter.java,v 1.1.16.3 2005/06/14 20:38:07 christianfoltin Exp $*/

package freemind.modes;

import java.awt.Color;

import freemind.controller.Controller;
import freemind.main.FreeMind;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.main.XMLElement;

public abstract class CloudAdapter extends LineAdapter implements MindMapCloud {

    private static Color standardColor = null;
    private static String standardStyle = null;
    private static LineAdapterListener listener = null;
    //
    // Constructors
    //
    public CloudAdapter(MindMapNode target,FreeMindMain frame) {
        this(target, frame, FreeMind.RESOURCES_CLOUD_COLOR, "standardcloudstyle");/*_I*/_prof.prof.cnt[10787]++;/*I_*//*_I*/_prof.prof.cnt[10788]++;/*I_*/
    }

    /** For derived classes.*/
    protected  CloudAdapter(MindMapNode target,FreeMindMain frame, String standardColorPropertyString, String standardStylePropertyString)  {
        super(target, frame, standardColorPropertyString, standardStylePropertyString);/*_I*/_prof.prof.cnt[10789]++;/*I_*//*_I*/_prof.prof.cnt[10790]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10791]++;/*I_*/NORMAL_WIDTH = 3;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10792]++;/*I_*/iterativeLevel = -1;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10793]++;/*I_*/if(listener == null) {
            /*_I*/{_prof.prof.cnt[10794]++;/*I_*/listener = new LineAdapterListener();/*_I*/}/*I_*/ 
            /*_I*/{_prof.prof.cnt[10795]++;/*I_*/Controller.addPropertyChangeListener(listener);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        
    }
    /**
    *  calculates the cloud iterative level which 
    *  is importent for the cloud size
    */
    
    private void calcIterativeLevel(MindMapNode target) {/*_I*/_prof.prof.cnt[10796]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10797]++;/*I_*/iterativeLevel = 0;/*_I*/}/*I_*/	
        /*_I*/{_prof.prof.cnt[10798]++;/*I_*/if (target != null) {	
        	/*_I*/{_prof.prof.cnt[10799]++;/*I_*/for(MindMapNode parentNode = target.getParentNode(); 
        	    parentNode != null; 
        	    parentNode = parentNode.getParentNode()) {
        	    	/*_I*/_prof.prof.cnt[10800]++;/*I_*/MindMapCloud cloud = parentNode.getCloud();
        	    	/*_I*/{_prof.prof.cnt[10801]++;/*I_*/if (cloud != null) {
        				/*_I*/{_prof.prof.cnt[10802]++;/*I_*/iterativeLevel = cloud.getIterativeLevel() + 1;/*_I*/}/*I_*/
        	    		/*_I*/{_prof.prof.cnt[10803]++;/*I_*/break;/*_I*/}/*I_*/
        	    	}/*_I*/}/*I_*/ 
        	}/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

	public void setTarget(MindMapNode target) {/*_I*/_prof.prof.cnt[10804]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10805]++;/*I_*/super.setTarget(target);/*_I*/}/*I_*/ 
    }

    public Color getExteriorColor() {/*_I*/_prof.prof.cnt[10806]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10807]++;/*I_*/return getColor().darker();/*_I*/}/*I_*/
    }

    /**  gets iterative level which is required for painting and layout. */
	public int getIterativeLevel() {/*_I*/_prof.prof.cnt[10808]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10809]++;/*I_*/if (iterativeLevel == -1) {
			/*_I*/{_prof.prof.cnt[10810]++;/*I_*/calcIterativeLevel(target);/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10811]++;/*I_*/return iterativeLevel;/*_I*/}/*I_*/
	}

	/**  changes the iterative level.*/
	public void changeIterativeLevel(int deltaLevel) {/*_I*/_prof.prof.cnt[10812]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10813]++;/*I_*/if (iterativeLevel != -1) {
			/*_I*/{_prof.prof.cnt[10814]++;/*I_*/iterativeLevel = iterativeLevel + deltaLevel;/*_I*/}/*I_*/
		}/*_I*/}/*I_*/	
	}
	
	private int iterativeLevel;
	
    public XMLElement save() {/*_I*/_prof.prof.cnt[10815]++;/*I_*/
        /*_I*/_prof.prof.cnt[10816]++;/*I_*/XMLElement cloud = new XMLElement();
        /*_I*/{_prof.prof.cnt[10817]++;/*I_*/cloud.setName("cloud");/*_I*/}/*I_*/
    
        /*_I*/{_prof.prof.cnt[10818]++;/*I_*/if (style != null) {
            /*_I*/{_prof.prof.cnt[10819]++;/*I_*/cloud.setAttribute("STYLE",style);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10820]++;/*I_*/if (color != null) {
            /*_I*/{_prof.prof.cnt[10821]++;/*I_*/cloud.setAttribute("COLOR",Tools.colorToXml(color));/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10822]++;/*I_*/if(width != DEFAULT_WIDTH) {
            /*_I*/{_prof.prof.cnt[10823]++;/*I_*/cloud.setAttribute("WIDTH",Integer.toString(width));/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10824]++;/*I_*/return cloud;/*_I*/}/*I_*/
    }

    protected Color getStandardColor() {/*_I*/_prof.prof.cnt[10825]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10826]++;/*I_*/return standardColor;/*_I*/}/*I_*/
    }
    protected void setStandardColor(Color standardColor) {/*_I*/_prof.prof.cnt[10827]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10828]++;/*I_*/CloudAdapter.standardColor = standardColor;/*_I*/}/*I_*/
    }
    protected String getStandardStyle() {/*_I*/_prof.prof.cnt[10829]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10830]++;/*I_*/return standardStyle;/*_I*/}/*I_*/
    }
    protected void setStandardStyle(String standardStyle) {/*_I*/_prof.prof.cnt[10831]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10832]++;/*I_*/CloudAdapter.standardStyle = standardStyle;/*_I*/}/*I_*/
    }
}
