/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: ControllerAdapter.java,v 1.41.14.31 2005/07/26 20:52:34 christianfoltin Exp $*/

package freemind.modes;

import java.awt.Color;
import java.awt.Point;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.filechooser.FileFilter;

import freemind.controller.Controller;
import freemind.controller.StructuredMenuHolder;
import freemind.controller.actions.ActionFactory;
import freemind.controller.actions.ModeControllerActionHandler;
import freemind.controller.actions.UndoActionHandler;
import freemind.controller.actions.generated.instance.ObjectFactory;
import freemind.controller.actions.generated.instance.XmlAction;
import freemind.extensions.HookFactory;
import freemind.extensions.ModeControllerHook;
import freemind.extensions.NodeHook;
import freemind.extensions.PermanentNodeHook;
import freemind.extensions.UndoEventReceiver;
import freemind.main.ExampleFileFilter;
import freemind.main.FreeMind;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.main.XMLParseException;
import freemind.modes.actions.AddArrowLinkAction;
import freemind.modes.actions.AddLocalLinkAction;
import freemind.modes.actions.ApplyPatternAction;
import freemind.modes.actions.BoldAction;
import freemind.modes.actions.ChangeArrowLinkEndPoints;
import freemind.modes.actions.ChangeArrowsInArrowLinkAction;
import freemind.modes.actions.CloudAction;
import freemind.modes.actions.ColorArrowLinkAction;
import freemind.modes.actions.CompoundActionHandler;
import freemind.modes.actions.CopyAction;
import freemind.modes.actions.CopySingleAction;
import freemind.modes.actions.CutAction;
import freemind.modes.actions.DeleteChildAction;
import freemind.modes.actions.EdgeColorAction;
import freemind.modes.actions.EdgeStyleAction;
import freemind.modes.actions.EdgeWidthAction;
import freemind.modes.actions.EditAction;
import freemind.modes.actions.FindAction;
import freemind.modes.actions.FontFamilyAction;
import freemind.modes.actions.FontSizeAction;
import freemind.modes.actions.GotoLinkNodeAction;
import freemind.modes.actions.IconAction;
import freemind.modes.actions.ImportExplorerFavoritesAction;
import freemind.modes.actions.ImportFolderStructureAction;
import freemind.modes.actions.ItalicAction;
import freemind.modes.actions.JoinNodesAction;
import freemind.modes.actions.MoveNodeAction;
import freemind.modes.actions.NewChildAction;
import freemind.modes.actions.NodeBackgroundColorAction;
import freemind.modes.actions.NodeColorAction;
import freemind.modes.actions.NodeColorBlendAction;
import freemind.modes.actions.NodeHookAction;
import freemind.modes.actions.NodeStyleAction;
import freemind.modes.actions.NodeUpAction;
import freemind.modes.actions.PasteAction;
import freemind.modes.actions.RedoAction;
import freemind.modes.actions.RemoveAllIconsAction;
import freemind.modes.actions.RemoveArrowLinkAction;
import freemind.modes.actions.RemoveLastIconAction;
import freemind.modes.actions.RevertAction;
import freemind.modes.actions.SelectAllAction;
import freemind.modes.actions.SelectBranchAction;
import freemind.modes.actions.SetLinkByTextFieldAction;
import freemind.modes.actions.ToggleChildrenFoldedAction;
import freemind.modes.actions.ToggleFoldedAction;
import freemind.modes.actions.UnderlinedAction;
import freemind.modes.actions.UndoAction;
import freemind.modes.actions.FindAction.FindNextAction;
import freemind.modes.actions.NodeBackgroundColorAction.RemoveNodeBackgroundColorAction;
import freemind.modes.mindmapmode.MindMapArrowLinkModel;
import freemind.view.MapModule;
import freemind.view.mindmapview.MapView;
import freemind.view.mindmapview.MindMapLayout;
import freemind.view.mindmapview.NodeView;


/**
 * Derive from this class to implement the Controller for your mode. Overload the methods
 * you need for your data model, or use the defaults. There are some default Actions you may want
 * to use for easy editing of your model. Take MindMapController as a sample.
 */
public abstract class ControllerAdapter implements ModeController {

	private final static int MINIMAL_LEAF_WIDTH = MindMapLayout.MINIMAL_LEAF_WIDTH;
	private final static int MINIMAL_WIDTH = MindMapLayout.MINIMAL_WIDTH;

    private ActionFactory actionFactory;
	// for cascading updates.
	private HashSet nodesAlreadyUpdated;
	private HashSet nodesToBeUpdated;
	// for MouseEventHandlers 
	private HashSet mRegisteredMouseWheelEventHandler = new HashSet();
	// Logging: 
	private static java.util.logging.Logger logger;
    public ApplyPatternAction patterns[] = new ApplyPatternAction[0]; // Make sure it is initialized

	private Mode mode;
    private Clipboard clipboard;
    private int status;
	public UndoAction undo=null;
	public RedoAction redo=null;
    public CopyAction copy = null;
    public Action copySingle = null;
    public CutAction cut = null;
    public PasteAction paste = null;
	public BoldAction bold = null;
	public ItalicAction italic = null;
	public UnderlinedAction underlined = null;
	public FontSizeAction fontSize = null;
	public FontFamilyAction fontFamily = null;
	public EditAction edit = null;
	public NewChildAction newChild = null;
	public DeleteChildAction deleteChild = null;
	public ToggleFoldedAction toggleFolded = null;
    public ToggleChildrenFoldedAction toggleChildrenFolded = null;
    public NodeUpAction nodeUp = null;
    public NodeDownAction nodeDown = null;
    public EdgeColorAction edgeColor = null;
    public EdgeWidthAction EdgeWidth_WIDTH_PARENT = null;
	public EdgeWidthAction EdgeWidth_WIDTH_THIN = null;
	public EdgeWidthAction EdgeWidth_1 = null;
	public EdgeWidthAction EdgeWidth_2 = null;
	public EdgeWidthAction EdgeWidth_4 = null;
	public EdgeWidthAction EdgeWidth_8 = null;
    public EdgeWidthAction edgeWidths[] = null;
	public EdgeStyleAction EdgeStyle_linear = null;
	public EdgeStyleAction EdgeStyle_bezier = null;
	public EdgeStyleAction EdgeStyle_sharp_linear = null;
	public EdgeStyleAction EdgeStyle_sharp_bezier = null;
    public EdgeStyleAction edgeStyles[] = null;
    public NodeColorBlendAction nodeColorBlend = null;
    public NodeStyleAction fork = null;
    public NodeStyleAction bubble = null;
    public CloudAction cloud = null;
    public freemind.modes.actions.CloudColorAction cloudColor = null;
    public AddArrowLinkAction addArrowLinkAction = null; 
    public RemoveArrowLinkAction removeArrowLinkAction = null;
    public ColorArrowLinkAction colorArrowLinkAction = null;
    public ChangeArrowsInArrowLinkAction changeArrowsInArrowLinkAction = null;
	public NodeBackgroundColorAction nodeBackgroundColor = null;
	public RemoveNodeBackgroundColorAction removeNodeBackgroundColor = null;

    public IconAction unknwonIconAction = null;
    public RemoveLastIconAction removeLastIconAction = null;
    public RemoveAllIconsAction removeAllIconsAction = null;
    public SetLinkByTextFieldAction setLinkByTextField = null;
    public AddLocalLinkAction addLocalLinkAction = null;
    public GotoLinkNodeAction gotoLinkNodeAction = null;
    public JoinNodesAction joinNodes = null;
    public MoveNodeAction moveNodeAction = null;
    public ImportExplorerFavoritesAction importExplorerFavorites = null;
    public ImportFolderStructureAction importFolderStructure = null;
    public ChangeArrowLinkEndPoints changeArrowLinkEndPoints = null;

    public FindAction find=null;
    public FindNextAction findNext=null;
    public NodeHookAction nodeHookAction = null;
    public RevertAction revertAction = null;
    public SelectBranchAction selectBranchAction = null;
    public SelectAllAction selectAllAction = null;


	/** Executes series of actions. */
	private CompoundActionHandler compound = null;

    private Color selectionColor = new Color(200,220,200);
    public NodeColorAction nodeColor = null;
    private Set globalPatternList;

    public ControllerAdapter(Mode mode) {/*_I*/_prof.prof.cnt[5461]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5462]++;/*I_*/this.setMode(mode);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5463]++;/*I_*/if(logger==null) {
        	/*_I*/{_prof.prof.cnt[5464]++;/*I_*/logger = getFrame().getLogger(this.getClass().getName());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        // for updates of nodes:
		/*_I*/{_prof.prof.cnt[5465]++;/*I_*/nodesAlreadyUpdated = new HashSet();/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5466]++;/*I_*/nodesToBeUpdated    = new HashSet();/*_I*/}/*I_*/
        // create action factory:
		/*_I*/{_prof.prof.cnt[5467]++;/*I_*/actionFactory = new ActionFactory(getController());/*_I*/}/*I_*/
		// prepare undo:
		/*_I*/{_prof.prof.cnt[5468]++;/*I_*/undo = new UndoAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5469]++;/*I_*/redo = new RedoAction(this);/*_I*/}/*I_*/
		// register default action handler:
		// the executor must be the first here, because it is executed last then.
		/*_I*/{_prof.prof.cnt[5470]++;/*I_*/getActionFactory().registerHandler(new ModeControllerActionHandler(getActionFactory()));/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5471]++;/*I_*/getActionFactory().registerHandler(new UndoActionHandler(this, undo, redo));/*_I*/}/*I_*/
		//debug:						getActionFactory().registerHandler(new freemind.controller.actions.PrintActionHandler(this));

        /*_I*/{_prof.prof.cnt[5472]++;/*I_*/cut = new CutAction(this);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5473]++;/*I_*/paste = new PasteAction(this);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5474]++;/*I_*/copy = new CopyAction(this);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5475]++;/*I_*/copySingle = new CopySingleAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5476]++;/*I_*/bold = new BoldAction (this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5477]++;/*I_*/italic = new ItalicAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5478]++;/*I_*/underlined = new UnderlinedAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5479]++;/*I_*/fontSize = new FontSizeAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5480]++;/*I_*/fontFamily = new FontFamilyAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5481]++;/*I_*/edit = new EditAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5482]++;/*I_*/newChild = new NewChildAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5483]++;/*I_*/deleteChild = new DeleteChildAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5484]++;/*I_*/toggleFolded = new ToggleFoldedAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5485]++;/*I_*/toggleChildrenFolded = new ToggleChildrenFoldedAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5486]++;/*I_*/nodeUp = new NodeUpAction(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5487]++;/*I_*/nodeDown = new NodeDownAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5488]++;/*I_*/edgeColor = new EdgeColorAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5489]++;/*I_*/nodeColor = new NodeColorAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5490]++;/*I_*/nodeColorBlend = new NodeColorBlendAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5491]++;/*I_*/fork = new NodeStyleAction(this, MindMapNode.STYLE_FORK);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5492]++;/*I_*/bubble = new NodeStyleAction(this, MindMapNode.STYLE_BUBBLE);/*_I*/}/*I_*/
	    // this is an unknown icon and thus corrected by mindicon:
	    /*_I*/{_prof.prof.cnt[5493]++;/*I_*/removeLastIconAction = new RemoveLastIconAction(this);/*_I*/}/*I_*/
	    // this action handles the xml stuff: (undo etc.)
	    /*_I*/{_prof.prof.cnt[5494]++;/*I_*/unknwonIconAction = new IconAction(this, MindIcon.factory((String) MindIcon
                .getAllIconNames().get(0)), removeLastIconAction);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5495]++;/*I_*/removeLastIconAction.setIconAction(unknwonIconAction);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5496]++;/*I_*/removeAllIconsAction = new RemoveAllIconsAction(this, unknwonIconAction);/*_I*/}/*I_*/
	    // load pattern actions:
		/*_I*/{_prof.prof.cnt[5497]++;/*I_*/try {
	           /*_I*/_prof.prof.cnt[5498]++;/*I_*/File patternsFile = getFrame().getPatternsFile();
	           /*_I*/{_prof.prof.cnt[5499]++;/*I_*/if (patternsFile != null && patternsFile.exists()) {
	              /*_I*/{_prof.prof.cnt[5500]++;/*I_*/loadPatterns(patternsFile);/*_I*/}/*I_*/ }
	           else {
	              /*_I*/{_prof.prof.cnt[5501]++;/*I_*/System.out.println("User patterns file "+patternsFile+" not found.");/*_I*/}/*I_*/
	              /*_I*/{_prof.prof.cnt[5502]++;/*I_*/loadPatterns(new InputStreamReader(getResource("patterns.xml").openStream()));/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
	        catch (XMLParseException e) {
	           /*_I*/{_prof.prof.cnt[5503]++;/*I_*/System.err.println("In patterns:"+e);/*_I*/}/*I_*/ }
		catch (Exception ex) {
	           /*_I*/{_prof.prof.cnt[5504]++;/*I_*/System.err.println("Patterns not loaded:"+ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5505]++;/*I_*/EdgeWidth_WIDTH_PARENT = new EdgeWidthAction(this, EdgeAdapter.WIDTH_PARENT);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5506]++;/*I_*/EdgeWidth_WIDTH_THIN = new EdgeWidthAction(this, EdgeAdapter.WIDTH_THIN);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5507]++;/*I_*/EdgeWidth_1 = new EdgeWidthAction(this, 1);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5508]++;/*I_*/EdgeWidth_2 = new EdgeWidthAction(this, 2);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5509]++;/*I_*/EdgeWidth_4 = new EdgeWidthAction(this, 4);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5510]++;/*I_*/EdgeWidth_8 = new EdgeWidthAction(this, 8);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5511]++;/*I_*/edgeWidths =  new EdgeWidthAction[]{
			EdgeWidth_WIDTH_PARENT, EdgeWidth_WIDTH_THIN, EdgeWidth_1, EdgeWidth_2, EdgeWidth_4, EdgeWidth_8
	    };/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5512]++;/*I_*/EdgeStyle_linear = new EdgeStyleAction(this, "linear");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5513]++;/*I_*/EdgeStyle_bezier = new EdgeStyleAction(this, "bezier");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5514]++;/*I_*/EdgeStyle_sharp_linear = new EdgeStyleAction(this, "sharp_linear");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5515]++;/*I_*/EdgeStyle_sharp_bezier = new EdgeStyleAction(this, "sharp_bezier");/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5516]++;/*I_*/edgeStyles =  new EdgeStyleAction[]{
			EdgeStyle_linear,
			EdgeStyle_bezier,
			EdgeStyle_sharp_linear,
			EdgeStyle_sharp_bezier
	    };/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5517]++;/*I_*/cloud = new CloudAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5518]++;/*I_*/cloudColor = new freemind.modes.actions.CloudColorAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5519]++;/*I_*/addArrowLinkAction = new AddArrowLinkAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5520]++;/*I_*/removeArrowLinkAction = new RemoveArrowLinkAction(this, null);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5521]++;/*I_*/addArrowLinkAction.setRemoveAction(removeArrowLinkAction);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5522]++;/*I_*/colorArrowLinkAction = new ColorArrowLinkAction(this, null);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5523]++;/*I_*/changeArrowsInArrowLinkAction = new ChangeArrowsInArrowLinkAction(this, "none", null, null, true, true);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5524]++;/*I_*/nodeBackgroundColor = new NodeBackgroundColorAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5525]++;/*I_*/removeNodeBackgroundColor = new RemoveNodeBackgroundColorAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5526]++;/*I_*/setLinkByTextField = new SetLinkByTextFieldAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5527]++;/*I_*/addLocalLinkAction = new AddLocalLinkAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5528]++;/*I_*/gotoLinkNodeAction = new GotoLinkNodeAction(this, null);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5529]++;/*I_*/moveNodeAction = new MoveNodeAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5530]++;/*I_*/joinNodes = new JoinNodesAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5531]++;/*I_*/importExplorerFavorites = new ImportExplorerFavoritesAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5532]++;/*I_*/importFolderStructure = new ImportFolderStructureAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5533]++;/*I_*/changeArrowLinkEndPoints = new ChangeArrowLinkEndPoints(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5534]++;/*I_*/find = new FindAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5535]++;/*I_*/findNext = new FindNextAction(this,find);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5536]++;/*I_*/nodeHookAction = new NodeHookAction("no_title", this);/*_I*/}/*I_*/ 
	    /*_I*/{_prof.prof.cnt[5537]++;/*I_*/revertAction = new RevertAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5538]++;/*I_*/selectBranchAction = new SelectBranchAction(this);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[5539]++;/*I_*/selectAllAction = new SelectAllAction(this);/*_I*/}/*I_*/
	    
	    /*_I*/{_prof.prof.cnt[5540]++;/*I_*/compound = new CompoundActionHandler(this);/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5541]++;/*I_*/DropTarget dropTarget = new DropTarget(getFrame().getViewport(),
                                               new FileOpener());

        /*_I*/{_prof.prof.cnt[5542]++;/*I_*/clipboard = getFrame().getViewport().getToolkit().getSystemSelection();/*_I*/}/*I_*/

        // SystemSelection is a strange clipboard used for instance on
        // Linux. To get data into this clipboard user just selects the area
        // without pressing Ctrl+C like on Windows.
        
        /*_I*/{_prof.prof.cnt[5543]++;/*I_*/if (clipboard == null) {
           /*_I*/{_prof.prof.cnt[5544]++;/*I_*/clipboard = getFrame().getViewport().getToolkit().getSystemClipboard();/*_I*/}/*I_*/ }/*_I*/}/*I_*/

    }

    private void loadPatterns(File file) throws Exception {/*_I*/_prof.prof.cnt[5545]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5546]++;/*I_*/createPatterns(StylePattern.loadPatterns(file));/*_I*/}/*I_*/
    }

    private void loadPatterns(Reader reader) throws Exception {/*_I*/_prof.prof.cnt[5547]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5548]++;/*I_*/createPatterns(StylePattern.loadPatterns(reader));/*_I*/}/*I_*/
    }

    private void createPatterns(List patternsList) throws Exception {/*_I*/_prof.prof.cnt[5549]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5550]++;/*I_*/globalPatternList = new HashSet();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5551]++;/*I_*/globalPatternList.addAll(patternsList);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5552]++;/*I_*/patterns = new ApplyPatternAction[patternsList.size()];/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5553]++;/*I_*/for (int i = 0; i < patterns.length; i++) {
            /*_I*/{_prof.prof.cnt[5554]++;/*I_*/patterns[i] = new ApplyPatternAction(this,
                    (StylePattern) patternsList.get(i));/*_I*/}/*I_*/

            // search icons for patterns:
            /*_I*/_prof.prof.cnt[5555]++;/*I_*/MindIcon patternIcon = ((StylePattern) patternsList.get(i))
                    .getNodeIcon();
            /*_I*/{_prof.prof.cnt[5556]++;/*I_*/if (patternIcon != null) {
                /*_I*/{_prof.prof.cnt[5557]++;/*I_*/patterns[i].putValue(Action.SMALL_ICON, patternIcon
                        .getIcon(getFrame()));/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    public ActionFactory getActionFactory() {/*_I*/_prof.prof.cnt[5558]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5559]++;/*I_*/return actionFactory;/*_I*/}/*I_*/
    }


    //
    // Methods that should be overloaded
    //

    public abstract MindMapNode newNode(Object userObject);
    protected MindMapNode newNode() {/*_I*/_prof.prof.cnt[5560]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5561]++;/*I_*/return newNode(null);/*_I*/}/*I_*/
    }

    /**
     * You _must_ implement this if you use one of the following actions:
     * OpenAction, NewMapAction.
     */
    public MapAdapter newModel() {/*_I*/_prof.prof.cnt[5562]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5563]++;/*I_*/throw new java.lang.UnsupportedOperationException();/*_I*/}/*I_*/
    }

    /**
     * You may want to implement this...
     * It returns the FileFilter that is used by the open() and save()
     * JFileChoosers.
     */
    protected FileFilter getFileFilter() {/*_I*/_prof.prof.cnt[5564]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5565]++;/*I_*/return null;/*_I*/}/*I_*/
    }

	
	/** Currently, this method is called by the mapAdapter. This is buggy, and is to be changed.*/
    public void nodeChanged(MindMapNode node) {/*_I*/_prof.prof.cnt[5566]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5567]++;/*I_*/getMap().setSaved(false);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5568]++;/*I_*/nodeRefresh(node,true);/*_I*/}/*I_*/
    }

    public void nodeRefresh(MindMapNode node) {/*_I*/_prof.prof.cnt[5569]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5570]++;/*I_*/nodeRefresh(node,false);/*_I*/}/*I_*/
    }
    private void nodeRefresh(MindMapNode node, boolean isUpdate) {/*_I*/_prof.prof.cnt[5571]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[5572]++;/*I_*/logger.finest("nodeChanged called for node "+node+" parent="+node.getParentNode());/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5573]++;/*I_*/if(nodesAlreadyUpdated.contains(node)) {			
			/*_I*/{_prof.prof.cnt[5574]++;/*I_*/return;/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5575]++;/*I_*/nodesToBeUpdated.add(node);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5576]++;/*I_*/nodesAlreadyUpdated.add(node);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5577]++;/*I_*/if (isUpdate) {
			// update modification times:
			/*_I*/{_prof.prof.cnt[5578]++;/*I_*/if(node.getHistoryInformation()!= null) {
				/*_I*/{_prof.prof.cnt[5579]++;/*I_*/node.getHistoryInformation().setLastModifiedAt(new Date());/*_I*/}/*I_*/
			}/*_I*/}/*I_*/
            // Tell any node hooks that the node is changed:
            /*_I*/{_prof.prof.cnt[5580]++;/*I_*/recursiveCallUpdateHooks((MindMapNode) node, (MindMapNode) node /* self update */);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5581]++;/*I_*/getMap().nodeChangedMapInternal(node);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5582]++;/*I_*/nodesToBeUpdated.remove(node);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5583]++;/*I_*/if(nodesToBeUpdated.size()==0) {
			// this is the end of all updates:
			/*_I*/{_prof.prof.cnt[5584]++;/*I_*/nodesAlreadyUpdated.clear();/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
    }

	/**
	 * @param parent
	 */
	public void nodeStructureChanged(MindMapNode node) {/*_I*/_prof.prof.cnt[5585]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5586]++;/*I_*/getMap().nodeStructureChanged(node);/*_I*/}/*I_*/
	}

	public boolean isUndoAction() {/*_I*/_prof.prof.cnt[5587]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[5588]++;/*I_*/return undo.isUndoAction() || redo.isUndoAction();/*_I*/}/*I_*/
	}

	/**
	 * @param node
	 */
	private void recursiveCallUpdateHooks(MindMapNode node, MindMapNode changedNode) {/*_I*/_prof.prof.cnt[5589]++;/*I_*/
	    // Tell any node hooks that the node is changed:
		/*_I*/{_prof.prof.cnt[5590]++;/*I_*/if(node instanceof MindMapNode) {
			/*_I*/{_prof.prof.cnt[5591]++;/*I_*/for(Iterator i=  ((MindMapNode)node).getActivatedHooks().iterator(); i.hasNext();) {
				/*_I*/_prof.prof.cnt[5592]++;/*I_*/PermanentNodeHook hook = (PermanentNodeHook) i.next();
				/*_I*/{_prof.prof.cnt[5593]++;/*I_*/if ( (! isUndoAction())  || hook instanceof UndoEventReceiver) {
                    /*_I*/{_prof.prof.cnt[5594]++;/*I_*/if (node == changedNode)
                        /*_I*/{_prof.prof.cnt[5595]++;/*I_*/hook.onUpdateNodeHook();/*_I*/}/*I_*/
                    else
                        /*_I*/{_prof.prof.cnt[5596]++;/*I_*/hook.onUpdateChildrenHook(changedNode);/*_I*/}/*I_*//*_I*/}/*I_*/
                }/*_I*/}/*I_*/
			}/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5597]++;/*I_*/if(!node.isRoot() && node.getParentNode()!= null)
			/*_I*/{_prof.prof.cnt[5598]++;/*I_*/recursiveCallUpdateHooks(node.getParentNode(), changedNode);/*_I*/}/*I_*//*_I*/}/*I_*/
	}

    public void anotherNodeSelected(MindMapNode n) {/*_I*/_prof.prof.cnt[5599]++;/*I_*/
    }

    public void doubleClick(MouseEvent e) {/*_I*/_prof.prof.cnt[5600]++;/*I_*/
        /* perform action only if one selected node.*/
        /*_I*/{_prof.prof.cnt[5601]++;/*I_*/if(getSelecteds().size() != 1)
            /*_I*/{_prof.prof.cnt[5602]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5603]++;/*I_*/MindMapNode node = ((NodeView)(e.getComponent())).getModel();
        // edit the node only if the node is a leaf (fc 0.7.1)
        /*_I*/{_prof.prof.cnt[5604]++;/*I_*/if (node.hasChildren()) {
            // the emulate the plain click. 
            /*_I*/{_prof.prof.cnt[5605]++;/*I_*/plainClick(e);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5606]++;/*I_*/return;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5607]++;/*I_*/if (!e.isAltDown() 
            && !e.isControlDown() 
            && !e.isShiftDown() 
            && !e.isPopupTrigger()
            && e.getButton() == MouseEvent.BUTTON1
            && (node.getLink() == null)) {
            /*_I*/{_prof.prof.cnt[5608]++;/*I_*/edit(null, false, false);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    public void plainClick(MouseEvent e) {/*_I*/_prof.prof.cnt[5609]++;/*I_*/
        /* perform action only if one selected node.*/
        /*_I*/{_prof.prof.cnt[5610]++;/*I_*/if(getSelecteds().size() != 1)
            /*_I*/{_prof.prof.cnt[5611]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5612]++;/*I_*/MindMapNode node = ((NodeView)(e.getComponent())).getModel();
        /*_I*/{_prof.prof.cnt[5613]++;/*I_*/if (getView().getSelected().isInFollowLinkRegion(e.getX())) {
            /*_I*/{_prof.prof.cnt[5614]++;/*I_*/loadURL();/*_I*/}/*I_*/ }
        else {
            /*_I*/{_prof.prof.cnt[5615]++;/*I_*/if (!node.hasChildren()) {
                // the emulate the plain click. 
                /*_I*/{_prof.prof.cnt[5616]++;/*I_*/doubleClick(e);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5617]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5618]++;/*I_*/toggleFolded();/*_I*/}/*I_*/ 
        }/*_I*/}/*I_*/
    }

    //
    // Map Management
    //

    public String getText(String textId) {/*_I*/_prof.prof.cnt[5619]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5620]++;/*I_*/return getController().getResourceString(textId);/*_I*/}/*I_*/ }

    protected boolean binOptionIsTrue(String option) {/*_I*/_prof.prof.cnt[5621]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5622]++;/*I_*/return getFrame().getProperty(option).equals("true");/*_I*/}/*I_*/}

    public MindMap newMap() {/*_I*/_prof.prof.cnt[5623]++;/*I_*/
        /*_I*/_prof.prof.cnt[5624]++;/*I_*/MapAdapter newModel = newModel();
        /*_I*/{_prof.prof.cnt[5625]++;/*I_*/newMap(newModel);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5626]++;/*I_*/return newModel;/*_I*/}/*I_*/
    }

    public void newMap(MindMap mapModel) {/*_I*/_prof.prof.cnt[5627]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5628]++;/*I_*/getController().getMapModuleManager().newMapModule(mapModel);/*_I*/}/*I_*/
    }

    /**
     * You may decide to overload this or take the default
     * and implement the functionality in your MapModel (implements MindMap)
     */
    public void load (File file) throws FileNotFoundException, IOException, XMLParseException {/*_I*/_prof.prof.cnt[5629]++;/*I_*/
        /*_I*/_prof.prof.cnt[5630]++;/*I_*/MapAdapter model = newModel();
		/*_I*/{_prof.prof.cnt[5631]++;/*I_*/model.load(file);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5632]++;/*I_*/getController().getMapModuleManager().newMapModule(model);/*_I*/}/*I_*/
		//FIXME: This must be done in the mapModuleManager. Ojo: The
		// ModeController has changed.
		/*_I*/{_prof.prof.cnt[5633]++;/*I_*/getController().getModeController().invokeHooksRecursively(
				(NodeAdapter) getModel().getRoot(), getModel());/*_I*/}/*I_*/
    }

    public boolean save() {/*_I*/_prof.prof.cnt[5634]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5635]++;/*I_*/if (getModel().isSaved()) /*_I*/{_prof.prof.cnt[5636]++;/*I_*/return true;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5637]++;/*I_*/if (getModel().getFile() == null || getModel().isReadOnly()) {
           /*_I*/{_prof.prof.cnt[5638]++;/*I_*/return saveAs();/*_I*/}/*I_*/ }
        else {
           /*_I*/{_prof.prof.cnt[5639]++;/*I_*/return save(getModel().getFile());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
    
    
    public void load(String xmlMapContents) {/*_I*/_prof.prof.cnt[5640]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5641]++;/*I_*/revertAction.openXmlInsteadOfMap(xmlMapContents);/*_I*/}/*I_*/
    }

    /** fc, 24.1.2004: having two methods getSelecteds with different return values 
     * (linkedlists of models resp. views) is asking for trouble. @see MapView
     * @return returns a list of MindMapNode s. */
    public List getSelecteds() {/*_I*/_prof.prof.cnt[5642]++;/*I_*/
	/*_I*/_prof.prof.cnt[5643]++;/*I_*/LinkedList selecteds = new LinkedList();
	/*_I*/_prof.prof.cnt[5644]++;/*I_*/ListIterator it = getView().getSelecteds().listIterator();
	/*_I*/{_prof.prof.cnt[5645]++;/*I_*/if (it != null) {
	    /*_I*/{_prof.prof.cnt[5646]++;/*I_*/while(it.hasNext()) {
		/*_I*/_prof.prof.cnt[5647]++;/*I_*/NodeView selected = (NodeView)it.next();
		/*_I*/{_prof.prof.cnt[5648]++;/*I_*/selecteds.add( selected.getModel() );/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[5649]++;/*I_*/return selecteds;/*_I*/}/*I_*/
    }

    
	/**
	 * This class sortes nodes by ascending depth of their paths to root. This
	 * is useful to assure that children are cutted <b>before </b> their
	 * fathers!!!.
	 * 
	 * Moreover, it sorts nodes with the same depth according to their
	 * position relative to each other.
	 */
	protected class nodesDepthComparator implements Comparator{
		public nodesDepthComparator() {/*_I*/_prof.prof.cnt[5650]++;/*I_*/}
		/* the < relation.*/
		public int compare(Object p1, Object p2) {/*_I*/_prof.prof.cnt[5651]++;/*I_*/
			/*_I*/_prof.prof.cnt[5652]++;/*I_*/MindMapNode n1 = ((MindMapNode) p1);
			/*_I*/_prof.prof.cnt[5653]++;/*I_*/MindMapNode n2 = ((MindMapNode) p2);
			/*_I*/_prof.prof.cnt[5654]++;/*I_*/Object[] path1 = getModel().getPathToRoot(n1);
			/*_I*/_prof.prof.cnt[5655]++;/*I_*/Object[] path2 = getModel().getPathToRoot(n2);
			/*_I*/_prof.prof.cnt[5656]++;/*I_*/int depth = path1.length - path2.length;
			/*_I*/{_prof.prof.cnt[5657]++;/*I_*/if(depth > 0)
				/*_I*/{_prof.prof.cnt[5658]++;/*I_*/return -1;/*_I*/}/*I_*//*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[5659]++;/*I_*/if(depth < 0)
				/*_I*/{_prof.prof.cnt[5660]++;/*I_*/return 1;/*_I*/}/*I_*//*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[5661]++;/*I_*/if(n1.isRoot()) // if n1 is root, n2 is root, too ;)
				/*_I*/{_prof.prof.cnt[5662]++;/*I_*/return 0;/*_I*/}/*I_*//*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[5663]++;/*I_*/return n1.getParentNode().getChildPosition(n1) - n2.getParentNode().getChildPosition(n2);/*_I*/}/*I_*/
		}
	}

	public List getSelectedsByDepth() {/*_I*/_prof.prof.cnt[5664]++;/*I_*/
		// return an ArrayList of MindMapNodes.
		/*_I*/_prof.prof.cnt[5665]++;/*I_*/List result = getSelecteds();
		/*_I*/{_prof.prof.cnt[5666]++;/*I_*/sortNodesByDepth(result);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5667]++;/*I_*/return result;/*_I*/}/*I_*/
	}


	public void sortNodesByDepth(List inPlaceList) {/*_I*/_prof.prof.cnt[5668]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5669]++;/*I_*/Collections.sort(inPlaceList, new nodesDepthComparator());/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5670]++;/*I_*/logger.finest("Sort result: "+inPlaceList);/*_I*/}/*I_*/
	}

	/**
     * Return false is the action was cancelled, e.g. when
     * it has to lead to saving as.
     */
    public boolean save(File file) {/*_I*/_prof.prof.cnt[5671]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5672]++;/*I_*/return getModel().save(file);/*_I*/}/*I_*/ }      

    /** @return returns the new JMenuItem.*/
    protected JMenuItem add(JMenu menu, Action action, String keystroke) {/*_I*/_prof.prof.cnt[5673]++;/*I_*/ 
       /*_I*/_prof.prof.cnt[5674]++;/*I_*/JMenuItem item = menu.add(action);
       /*_I*/{_prof.prof.cnt[5675]++;/*I_*/item.setAccelerator(KeyStroke.getKeyStroke(getFrame().getProperty(keystroke)));/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[5676]++;/*I_*/return item;/*_I*/}/*I_*/
    }

	/** @return returns the new JMenuItem.
	 * @param keystroke can be null, if no keystroke should be assigned. */
	protected JMenuItem add(StructuredMenuHolder holder, String category, Action action, String keystroke) {/*_I*/_prof.prof.cnt[5677]++;/*I_*/ 
	   /*_I*/_prof.prof.cnt[5678]++;/*I_*/JMenuItem item = holder.addMenuItem(new JMenuItem(action), category);
	   /*_I*/{_prof.prof.cnt[5679]++;/*I_*/if(keystroke != null) {
		/*_I*/{_prof.prof.cnt[5680]++;/*I_*/item.setAccelerator(KeyStroke.getKeyStroke(getFrame().getProperty(keystroke)));/*_I*/}/*I_*/
	   }/*_I*/}/*I_*/
	   /*_I*/{_prof.prof.cnt[5681]++;/*I_*/return item;/*_I*/}/*I_*/
	}

	/** @return returns the new JCheckBoxMenuItem.
	 * @param keystroke can be null, if no keystroke should be assigned. */
	protected JMenuItem addCheckBox(StructuredMenuHolder holder, String category, Action action, String keystroke) {/*_I*/_prof.prof.cnt[5682]++;/*I_*/ 
		/*_I*/_prof.prof.cnt[5683]++;/*I_*/JCheckBoxMenuItem item = (JCheckBoxMenuItem) holder.addMenuItem(new JCheckBoxMenuItem(action), category);
	   /*_I*/{_prof.prof.cnt[5684]++;/*I_*/if(keystroke != null) {
		/*_I*/{_prof.prof.cnt[5685]++;/*I_*/item.setAccelerator(KeyStroke.getKeyStroke(getFrame().getProperty(keystroke)));/*_I*/}/*I_*/
	   }/*_I*/}/*I_*/
	   /*_I*/{_prof.prof.cnt[5686]++;/*I_*/return item;/*_I*/}/*I_*/
	}

	protected void add(JMenu menu, Action action) {/*_I*/_prof.prof.cnt[5687]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5688]++;/*I_*/menu.add(action);/*_I*/}/*I_*/ }

    //
    // Dialogs with user
    //

    public void open() {/*_I*/_prof.prof.cnt[5689]++;/*I_*/
        /*_I*/_prof.prof.cnt[5690]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
        /*_I*/_prof.prof.cnt[5691]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir); 
//         if ((getMap() != null) && (getMap().getFile() != null) && (getMap().getFile().getParentFile() != null)) {
//             chooser = new JFileChooser(getMap().getFile().getParentFile());
//         } else {
//             chooser = new JFileChooser();
//         }
        //chooser.setLocale(currentLocale);
        /*_I*/{_prof.prof.cnt[5692]++;/*I_*/if (getFileFilter() != null) {
            /*_I*/{_prof.prof.cnt[5693]++;/*I_*/chooser.addChoosableFileFilter(getFileFilter());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5694]++;/*I_*/int returnVal = chooser.showOpenDialog(getView());
        /*_I*/{_prof.prof.cnt[5695]++;/*I_*/if (returnVal==JFileChooser.APPROVE_OPTION) {
            /*_I*/{_prof.prof.cnt[5696]++;/*I_*/try {
                /*_I*/{_prof.prof.cnt[5697]++;/*I_*/load(chooser.getSelectedFile());/*_I*/}/*I_*/
            } catch (Exception ex) {
               /*_I*/{_prof.prof.cnt[5698]++;/*I_*/handleLoadingException (ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/ {
            }
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5699]++;/*I_*/getController().setTitle();/*_I*/}/*I_*/
    }

    public void handleLoadingException (Exception ex) {/*_I*/_prof.prof.cnt[5700]++;/*I_*/
       /*_I*/_prof.prof.cnt[5701]++;/*I_*/String exceptionType = ex.getClass().getName();
       /*_I*/{_prof.prof.cnt[5702]++;/*I_*/if (exceptionType.equals("freemind.main.XMLParseException")) {
          /*_I*/_prof.prof.cnt[5703]++;/*I_*/int showDetail = JOptionPane.showConfirmDialog
             (getView(), getText("map_corrupted"),"FreeMind",JOptionPane.YES_NO_OPTION,JOptionPane.ERROR_MESSAGE);
          /*_I*/{_prof.prof.cnt[5704]++;/*I_*/if (showDetail==JOptionPane.YES_OPTION) {
             /*_I*/{_prof.prof.cnt[5705]++;/*I_*/getController().errorMessage(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
       else /*_I*/{_prof.prof.cnt[5706]++;/*I_*/if (exceptionType.equals("java.io.FileNotFoundException")) {
          /*_I*/{_prof.prof.cnt[5707]++;/*I_*/getController().errorMessage(ex.getMessage());/*_I*/}/*I_*/ }
       else {
       	  /*_I*/{_prof.prof.cnt[5708]++;/*I_*/ex.printStackTrace();/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[5709]++;/*I_*/getController().errorMessage(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*/
    }

    /**
     * Save as; return false is the action was cancelled
     */
    public boolean saveAs() {/*_I*/_prof.prof.cnt[5710]++;/*I_*/
        /*_I*/_prof.prof.cnt[5711]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
        /*_I*/_prof.prof.cnt[5712]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir); 
//         if ((getMap().getFile() != null) && (getMap().getFile().getParentFile() != null)) {
//             chooser = new JFileChooser(getMap().getFile().getParentFile()); }
//         else {
//            chooser = new JFileChooser();
//            chooser.setSelectedFile(new File(getFileNameProposal()+".mm"));
//         }
        //chooser.setLocale(currentLocale);
        /*_I*/{_prof.prof.cnt[5713]++;/*I_*/if (getFileFilter() != null) {
            /*_I*/{_prof.prof.cnt[5714]++;/*I_*/chooser.addChoosableFileFilter(getFileFilter());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        
        /*_I*/{_prof.prof.cnt[5715]++;/*I_*/chooser.setDialogTitle(getText("save_as"));/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5716]++;/*I_*/int returnVal = chooser.showSaveDialog(getView());
        /*_I*/{_prof.prof.cnt[5717]++;/*I_*/if (returnVal != JFileChooser.APPROVE_OPTION) {// not ok pressed
        	/*_I*/{_prof.prof.cnt[5718]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        
        // |= Pressed O.K.    
        /*_I*/_prof.prof.cnt[5719]++;/*I_*/File f = chooser.getSelectedFile();
        //Force the extension to be .mm
        /*_I*/_prof.prof.cnt[5720]++;/*I_*/String ext = Tools.getExtension(f.getName());
        /*_I*/{_prof.prof.cnt[5721]++;/*I_*/if(!ext.equals("mm")) {
           /*_I*/{_prof.prof.cnt[5722]++;/*I_*/f = new File(f.getParent(),f.getName()+".mm");/*_I*/}/*I_*/ }/*_I*/}/*I_*/        
                
        /*_I*/{_prof.prof.cnt[5723]++;/*I_*/if (f.exists()) { // If file exists, ask before overwriting.
			/*_I*/_prof.prof.cnt[5724]++;/*I_*/int overwriteMap = JOptionPane.showConfirmDialog
			   (getView(), getText("map_already_exists"), "FreeMind", JOptionPane.YES_NO_OPTION );
			/*_I*/{_prof.prof.cnt[5725]++;/*I_*/if (overwriteMap != JOptionPane.YES_OPTION) {
			   /*_I*/{_prof.prof.cnt[5726]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/

		/*_I*/{_prof.prof.cnt[5727]++;/*I_*/try { // We have to lock the file of the map even when it does not exist yet
		   /*_I*/_prof.prof.cnt[5728]++;/*I_*/String lockingUser = getModel().tryToLock(f);
		   /*_I*/{_prof.prof.cnt[5729]++;/*I_*/if (lockingUser != null) {          
		      /*_I*/{_prof.prof.cnt[5730]++;/*I_*/getFrame().getController().informationMessage(
			    Tools.expandPlaceholders(getText("map_locked_by_save_as"), f.getName(), lockingUser));/*_I*/}/*I_*/
		      /*_I*/{_prof.prof.cnt[5731]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
		catch (Exception e){ // Throwed by tryToLock
		  /*_I*/{_prof.prof.cnt[5732]++;/*I_*/getFrame().getController().informationMessage(
		    Tools.expandPlaceholders(getText("locking_failed_by_save_as"), f.getName()));/*_I*/}/*I_*/	 
		  /*_I*/{_prof.prof.cnt[5733]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/ 	        	                              
              
        /*_I*/{_prof.prof.cnt[5734]++;/*I_*/save(f);/*_I*/}/*I_*/
        //Update the name of the map
        /*_I*/{_prof.prof.cnt[5735]++;/*I_*/getController().getMapModuleManager().updateMapModuleName();/*_I*/}/*I_*/        
        /*_I*/{_prof.prof.cnt[5736]++;/*I_*/return true;/*_I*/}/*I_*/
    }
    /**
     * Creates a proposal for a file name to save the map. Removes all illegal
     * characters.
     * 
     * Fixed: When creating file names based on the text of the root node, now all the
     * extra unicode characters are replaced with _. This is not very good. For
     * chinese content, you would only get a list of ______ as a file name. Only
     * characters special for building file paths shall be removed (rather than
     * replaced with _), like : or /. The exact list of dangeous characters
     * needs to be investigated. 0.8.0RC3.
     * 
     * @return
     */
    private String getFileNameProposal() {/*_I*/_prof.prof.cnt[5737]++;/*I_*/
        /*_I*/_prof.prof.cnt[5738]++;/*I_*/String rootText = ((MindMapNode)getMap().getRoot()).toString();
        /*_I*/{_prof.prof.cnt[5739]++;/*I_*/rootText = rootText.replaceAll("[&:/\\\\\0%$#~\\?\\*]+", "");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5740]++;/*I_*/return rootText;/*_I*/}/*I_*/
    }

    /**
     * Return false if user has canceled. 
     */
    public boolean close(boolean force) {/*_I*/_prof.prof.cnt[5741]++;/*I_*/
        /*_I*/_prof.prof.cnt[5742]++;/*I_*/String[] options = {getText("yes"),
                            getText("no"),
                            getText("cancel")};
        /*_I*/{_prof.prof.cnt[5743]++;/*I_*/if (!force && !getModel().isSaved()) {
            /*_I*/_prof.prof.cnt[5744]++;/*I_*/String text = getText("save_unsaved")+"\n"+getMapModule().toString();
            /*_I*/_prof.prof.cnt[5745]++;/*I_*/String title = getText("save");
            /*_I*/_prof.prof.cnt[5746]++;/*I_*/int returnVal = JOptionPane.showOptionDialog(getFrame().getContentPane(),text,title,JOptionPane.YES_NO_CANCEL_OPTION,
                                                         JOptionPane.QUESTION_MESSAGE,null,options,options[0]);
            /*_I*/{_prof.prof.cnt[5747]++;/*I_*/if (returnVal==JOptionPane.YES_OPTION) {
               /*_I*/_prof.prof.cnt[5748]++;/*I_*/boolean savingNotCancelled = save();
               /*_I*/{_prof.prof.cnt[5749]++;/*I_*/if (!savingNotCancelled) {
               	  /*_I*/{_prof.prof.cnt[5750]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
			else /*_I*/{_prof.prof.cnt[5751]++;/*I_*/if ((returnVal==JOptionPane.CANCEL_OPTION) || (returnVal == JOptionPane.CLOSED_OPTION)) {
				/*_I*/{_prof.prof.cnt[5752]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*/}/*_I*/}/*I_*/
                
        /*_I*/{_prof.prof.cnt[5753]++;/*I_*/getModel().destroy();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5754]++;/*I_*/return true;/*_I*/}/*I_*/ }
    


	/* (non-Javadoc)
	 * @see freemind.modes.ModeController#setVisible(boolean)
	 */
	public void setVisible(boolean visible) {/*_I*/_prof.prof.cnt[5755]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5756]++;/*I_*/if (visible) {
			/*_I*/_prof.prof.cnt[5757]++;/*I_*/MindMapNode node = getSelected();
			/*_I*/{_prof.prof.cnt[5758]++;/*I_*/for (Iterator j = node.getActivatedHooks().iterator();
				j.hasNext();
				) {
				/*_I*/_prof.prof.cnt[5759]++;/*I_*/PermanentNodeHook hook = (PermanentNodeHook) j.next();
				/*_I*/{_prof.prof.cnt[5760]++;/*I_*/hook.onReceiveFocusHook();/*_I*/}/*I_*/
			}/*_I*/}/*I_*/
		} else {
			/*_I*/_prof.prof.cnt[5761]++;/*I_*/MindMapNode node = getSelected();
			// bug fix, fc 18.5.2004. This should not be here.
			/*_I*/{_prof.prof.cnt[5762]++;/*I_*/if (node != null) {
                /*_I*/{_prof.prof.cnt[5763]++;/*I_*/for (Iterator j = node.getActivatedHooks().iterator();
                    j.hasNext();
                    ) {
                    /*_I*/_prof.prof.cnt[5764]++;/*I_*/PermanentNodeHook hook = (PermanentNodeHook) j.next();
                    /*_I*/{_prof.prof.cnt[5765]++;/*I_*/hook.onLooseFocusHook();/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}


    /**
     * Overwrite this to set all of your actions which are
     * dependent on whether there is a map or not.
     */
    protected void setAllActions(boolean enabled) {/*_I*/_prof.prof.cnt[5766]++;/*I_*/
        // controller actions:
        /*_I*/{_prof.prof.cnt[5767]++;/*I_*/getController().zoomIn.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5768]++;/*I_*/getController().zoomOut.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5769]++;/*I_*/cut.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5770]++;/*I_*/copy.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5771]++;/*I_*/copySingle.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5772]++;/*I_*/paste.setEnabled(enabled);/*_I*/}/*I_*/
    	/*_I*/{_prof.prof.cnt[5773]++;/*I_*/undo.setEnabled(enabled);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5774]++;/*I_*/redo.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5775]++;/*I_*/edit.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5776]++;/*I_*/newChild.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5777]++;/*I_*/toggleFolded.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5778]++;/*I_*/toggleChildrenFolded.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5779]++;/*I_*/setLinkByTextField.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5780]++;/*I_*/italic.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5781]++;/*I_*/bold.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5782]++;/*I_*/find.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5783]++;/*I_*/findNext.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5784]++;/*I_*/addArrowLinkAction.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5785]++;/*I_*/addLocalLinkAction.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5786]++;/*I_*/nodeColorBlend.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5787]++;/*I_*/nodeUp.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5788]++;/*I_*/nodeBackgroundColor.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5789]++;/*I_*/nodeDown.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5790]++;/*I_*/importExplorerFavorites.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5791]++;/*I_*/importFolderStructure.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5792]++;/*I_*/joinNodes.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5793]++;/*I_*/deleteChild.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5794]++;/*I_*/cloud.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5795]++;/*I_*/cloudColor.setEnabled(enabled);/*_I*/}/*I_*/
//        normalFont.setEnabled(enabled);
        /*_I*/{_prof.prof.cnt[5796]++;/*I_*/nodeColor.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5797]++;/*I_*/edgeColor.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5798]++;/*I_*/removeLastIconAction.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5799]++;/*I_*/removeAllIconsAction.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5800]++;/*I_*/selectAllAction.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5801]++;/*I_*/selectBranchAction.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5802]++;/*I_*/removeNodeBackgroundColor.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5803]++;/*I_*/moveNodeAction.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5804]++;/*I_*/revertAction.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5805]++;/*I_*/for (int i=0; i<edgeWidths.length; ++i) { 
            /*_I*/{_prof.prof.cnt[5806]++;/*I_*/edgeWidths[i].setEnabled(enabled);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5807]++;/*I_*/fork.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5808]++;/*I_*/bubble.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5809]++;/*I_*/for (int i=0; i<edgeStyles.length; ++i) { 
            /*_I*/{_prof.prof.cnt[5810]++;/*I_*/edgeStyles[i].setEnabled(enabled);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5811]++;/*I_*/for (int i=0; i<patterns.length; ++i) { 
            /*_I*/{_prof.prof.cnt[5812]++;/*I_*/patterns[i].setEnabled(enabled);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    //
    // Node editing
    //

    private JPopupMenu popupmenu;

    /** listener, that blocks the controler if the menu is active (PN)
        Take care! This listener is also used for modelpopups (as for graphical links).*/
    private class ControllerPopupMenuListener implements PopupMenuListener  {
      public void popupMenuWillBecomeVisible(PopupMenuEvent e) {/*_I*/_prof.prof.cnt[5813]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5814]++;/*I_*/setBlocked(true);/*_I*/}/*I_*/         // block controller
      }
      public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {/*_I*/_prof.prof.cnt[5815]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5816]++;/*I_*/setBlocked(false);/*_I*/}/*I_*/        // unblock controller
      }
      public void popupMenuCanceled(PopupMenuEvent e) {/*_I*/_prof.prof.cnt[5817]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5818]++;/*I_*/setBlocked(false);/*_I*/}/*I_*/        // unblock controller
      }

    }
    /** Take care! This listener is also used for modelpopups (as for graphical links).*/
    protected final ControllerPopupMenuListener popupListenerSingleton
        = new ControllerPopupMenuListener();
    
    public void showPopupMenu(MouseEvent e) {/*_I*/_prof.prof.cnt[5819]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5820]++;/*I_*/if (e.isPopupTrigger()) {
        /*_I*/_prof.prof.cnt[5821]++;/*I_*/JPopupMenu popupmenu = getPopupMenu();
        /*_I*/{_prof.prof.cnt[5822]++;/*I_*/if (popupmenu != null) {
          // adding listener could be optimized but without much profit...
          /*_I*/{_prof.prof.cnt[5823]++;/*I_*/popupmenu.addPopupMenuListener( this.popupListenerSingleton );/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[5824]++;/*I_*/popupmenu.show(e.getComponent(),e.getX(),e.getY());/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[5825]++;/*I_*/e.consume();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
    }

    /** Default implementation: no context menu.*/
    public JPopupMenu getPopupForModel(java.lang.Object obj) {/*_I*/_prof.prof.cnt[5826]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5827]++;/*I_*/return null;/*_I*/}/*I_*/
    }
        


    private static final int SCROLL_SKIPS = 8;
    private static final int SCROLL_SKIP = 10;
    private static final int HORIZONTAL_SCROLL_MASK 
       = InputEvent.SHIFT_MASK | InputEvent.BUTTON1_MASK 
         | InputEvent.BUTTON2_MASK | InputEvent.BUTTON3_MASK;
    private static final int ZOOM_MASK 
       = InputEvent.CTRL_MASK;
      // |=   oldX >=0 iff we are in the drag

    public void mouseWheelMoved(MouseWheelEvent e) {/*_I*/_prof.prof.cnt[5828]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5829]++;/*I_*/if (isBlocked()) {
            /*_I*/{_prof.prof.cnt[5830]++;/*I_*/return;/*_I*/}/*I_*/ // block the scroll during edit (PN)
       }/*_I*/}/*I_*/	
       /*_I*/{_prof.prof.cnt[5831]++;/*I_*/for (Iterator i = mRegisteredMouseWheelEventHandler.iterator(); i.hasNext();) {
        /*_I*/_prof.prof.cnt[5832]++;/*I_*/MouseWheelEventHandler handler = (MouseWheelEventHandler) i.next();
        /*_I*/_prof.prof.cnt[5833]++;/*I_*/boolean result = handler.handleMouseWheelEvent(e);
        /*_I*/{_prof.prof.cnt[5834]++;/*I_*/if(result) {
            // event was consumed:
            /*_I*/{_prof.prof.cnt[5835]++;/*I_*/return;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }/*_I*/}/*I_*/
        
       /*_I*/{_prof.prof.cnt[5836]++;/*I_*/if ((e.getModifiers() & ZOOM_MASK) != 0) {
           // fc, 18.11.2003: when control pressed, then the zoom is changed.
           /*_I*/_prof.prof.cnt[5837]++;/*I_*/float newZoomFactor = 1f + Math.abs((float) e.getWheelRotation())/10f;
           /*_I*/{_prof.prof.cnt[5838]++;/*I_*/if(e.getWheelRotation() < 0)
               /*_I*/{_prof.prof.cnt[5839]++;/*I_*/newZoomFactor = 1 / newZoomFactor;/*_I*/}/*I_*//*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[5840]++;/*I_*/float newZoom = ((MapView)e.getComponent()).getZoom() * newZoomFactor;
           // round the value due to possible rounding problems.
           /*_I*/{_prof.prof.cnt[5841]++;/*I_*/newZoom =  (float) Math.rint(newZoom*1000f)/1000f;/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5842]++;/*I_*/getController().setZoom(newZoom);/*_I*/}/*I_*/
           // end zoomchange
       } else /*_I*/{_prof.prof.cnt[5843]++;/*I_*/if ((e.getModifiers() & HORIZONTAL_SCROLL_MASK) != 0) {
          /*_I*/{_prof.prof.cnt[5844]++;/*I_*/for (int i=0; i < SCROLL_SKIPS; i++) {
             /*_I*/{_prof.prof.cnt[5845]++;/*I_*/((MapView)e.getComponent()).scrollBy(
                 SCROLL_SKIP * e.getWheelRotation(), 0, false);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
       else {
          /*_I*/{_prof.prof.cnt[5846]++;/*I_*/for (int i=0; i < SCROLL_SKIPS; i++) {
             /*_I*/{_prof.prof.cnt[5847]++;/*I_*/((MapView)e.getComponent()).scrollBy(0, 
                 SCROLL_SKIP * e.getWheelRotation(), false);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*//*_I*/}/*I_*/
    }

    public void registerMouseWheelEventHandler(MouseWheelEventHandler handler) {/*_I*/_prof.prof.cnt[5848]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5849]++;/*I_*/logger.info("Registered   MouseWheelEventHandler "+handler);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5850]++;/*I_*/mRegisteredMouseWheelEventHandler.add(handler);/*_I*/}/*I_*/
    }
    public void deRegisterMouseWheelEventHandler(MouseWheelEventHandler handler) {/*_I*/_prof.prof.cnt[5851]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5852]++;/*I_*/logger.info("Deregistered MouseWheelEventHandler "+handler);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5853]++;/*I_*/mRegisteredMouseWheelEventHandler.remove(handler);/*_I*/}/*I_*/
    }
    // this enables from outside close the edit mode
    private FocusListener textFieldListener = null;
    
    
    // status, currently: default, blocked  (PN)
    // (blocked to protect against particular events e.g. in edit mode)
    private boolean isBlocked = false;

    public boolean isBlocked() {/*_I*/_prof.prof.cnt[5854]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5855]++;/*I_*/return this.isBlocked;/*_I*/}/*I_*/
    }
    public void setBlocked(boolean isBlocked) {/*_I*/_prof.prof.cnt[5856]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5857]++;/*I_*/this.isBlocked = isBlocked;/*_I*/}/*I_*/
    }
    


	public void setBold(MindMapNode node, boolean bolded) {/*_I*/_prof.prof.cnt[5858]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5859]++;/*I_*/bold.setBold(node, bolded);/*_I*/}/*I_*/
	}

	public void setItalic(MindMapNode node, boolean isItalic) {/*_I*/_prof.prof.cnt[5860]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5861]++;/*I_*/italic.setItalic(node, isItalic);/*_I*/}/*I_*/
	}

    public void setCloud(MindMapNode node, boolean enable) {/*_I*/_prof.prof.cnt[5862]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5863]++;/*I_*/cloud.setCloud(node, enable);/*_I*/}/*I_*/
    }
    public void setCloudColor(MindMapNode node, Color color) {/*_I*/_prof.prof.cnt[5864]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5865]++;/*I_*/cloudColor.setCloudColor(node, color);/*_I*/}/*I_*/
    }
    //Node editing
    public void setFontSize(MindMapNode node, String fontSizeValue) {/*_I*/_prof.prof.cnt[5866]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5867]++;/*I_*/fontSize.setFontSize(node, fontSizeValue);/*_I*/}/*I_*/
	}

    /**
     *
     */

    public void increaseFontSize(MindMapNode node, int increment) {/*_I*/_prof.prof.cnt[5868]++;/*I_*/
        /*_I*/_prof.prof.cnt[5869]++;/*I_*/int newSize = Integer.valueOf(node.getFontSize()).intValue()+increment;
        
        /*_I*/{_prof.prof.cnt[5870]++;/*I_*/if (newSize > 0) {
            /*_I*/{_prof.prof.cnt[5871]++;/*I_*/setFontSize(node, Integer.toString(newSize));/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }
    
    public void setFontFamily(MindMapNode node, String fontFamilyValue) {/*_I*/_prof.prof.cnt[5872]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5873]++;/*I_*/fontFamily.setFontFamily(node, fontFamilyValue);/*_I*/}/*I_*/
    }

    public void setNodeColor(MindMapNode node, Color color) {/*_I*/_prof.prof.cnt[5874]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5875]++;/*I_*/nodeColor.setNodeColor(node, color);/*_I*/}/*I_*/
    }
    
	public void setNodeBackgroundColor(MindMapNode node, Color color) {/*_I*/_prof.prof.cnt[5876]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5877]++;/*I_*/nodeBackgroundColor.setNodeBackgroundColor(node, color);/*_I*/}/*I_*/
	}
    public void blendNodeColor(MindMapNode node) {/*_I*/_prof.prof.cnt[5878]++;/*I_*/
        /*_I*/_prof.prof.cnt[5879]++;/*I_*/Color mapColor = getMap().getBackgroundColor();
        /*_I*/_prof.prof.cnt[5880]++;/*I_*/Color nodeColor = node.getColor();
        /*_I*/{_prof.prof.cnt[5881]++;/*I_*/if (nodeColor == null) {
            /*_I*/{_prof.prof.cnt[5882]++;/*I_*/nodeColor = Tools.xmlToColor(getFrame().getProperty(
                    FreeMind.RESOURCES_NODE_COLOR));/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5883]++;/*I_*/setNodeColor(node, new Color(
                (3 * mapColor.getRed() + nodeColor.getRed()) / 4, (3 * mapColor
                        .getGreen() + nodeColor.getGreen()) / 4, (3 * mapColor
                        .getBlue() + nodeColor.getBlue()) / 4));/*_I*/}/*I_*/
    }


    public void setEdgeColor(MindMapNode node, Color color) {/*_I*/_prof.prof.cnt[5884]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5885]++;/*I_*/edgeColor.setEdgeColor(node, color);/*_I*/}/*I_*/
    }

	public void applyPattern(MindMapNode node, String patternName){/*_I*/_prof.prof.cnt[5886]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5887]++;/*I_*/for (int i = 0; i < patterns.length; i++) {
            /*_I*/_prof.prof.cnt[5888]++;/*I_*/ApplyPatternAction patternAction = patterns[i];
            /*_I*/{_prof.prof.cnt[5889]++;/*I_*/if(patternAction.getPattern().getName().equals(patternName)){
                /*_I*/{_prof.prof.cnt[5890]++;/*I_*/patternAction.applyPattern(node, patternAction.getPattern());/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5891]++;/*I_*/break;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
	}

    

    public void applyPattern(MindMapNode node, StylePattern pattern) {/*_I*/_prof.prof.cnt[5892]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[5893]++;/*I_*/if(patterns.length > 0) {
	        /*_I*/{_prof.prof.cnt[5894]++;/*I_*/patterns[0].applyPattern(node, pattern);/*_I*/}/*I_*/
	    } else {
	        /*_I*/{_prof.prof.cnt[5895]++;/*I_*/throw new IllegalArgumentException("No pattern defined.");/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/
    }
    
    public void addIcon(MindMapNode node, MindIcon icon) {/*_I*/_prof.prof.cnt[5896]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5897]++;/*I_*/unknwonIconAction.addIcon(node, icon);/*_I*/}/*I_*/
    }


    public void removeAllIcons(MindMapNode node) {/*_I*/_prof.prof.cnt[5898]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5899]++;/*I_*/removeAllIconsAction.removeAllIcons(node);/*_I*/}/*I_*/
    }

    public int removeLastIcon(MindMapNode node) {/*_I*/_prof.prof.cnt[5900]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5901]++;/*I_*/return removeLastIconAction.removeLastIcon(node);/*_I*/}/*I_*/
    }
    /**
     *
     */

    public void addLink(MindMapNode source, MindMapNode target) {/*_I*/_prof.prof.cnt[5902]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5903]++;/*I_*/addArrowLinkAction.addLink(source, target);/*_I*/}/*I_*/
    }

	public void removeReference(MindMapLink arrowLink){/*_I*/_prof.prof.cnt[5904]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[5905]++;/*I_*/removeArrowLinkAction.removeReference(arrowLink);/*_I*/}/*I_*/
	}

    public void setArrowLinkColor(MindMapLink arrowLink, Color color) {/*_I*/_prof.prof.cnt[5906]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5907]++;/*I_*/colorArrowLinkAction.setArrowLinkColor(arrowLink, color);/*_I*/}/*I_*/
    }
    
    /**
     *
     */

    public void changeArrowsOfArrowLink(MindMapArrowLinkModel arrowLink,
            boolean hasStartArrow, boolean hasEndArrow) {/*_I*/_prof.prof.cnt[5908]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5909]++;/*I_*/changeArrowsInArrowLinkAction.changeArrowsOfArrowLink(arrowLink, hasStartArrow, hasEndArrow);/*_I*/}/*I_*/
    }
    
	public void setArrowLinkEndPoints(MindMapArrowLink link, Point startPoint,
			Point endPoint) {/*_I*/_prof.prof.cnt[5910]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5911]++;/*I_*/changeArrowLinkEndPoints.setArrowLinkEndPoints(link, startPoint, endPoint);/*_I*/}/*I_*/
	}
    public void setLink(MindMapNode node, String link) {/*_I*/_prof.prof.cnt[5912]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5913]++;/*I_*/setLinkByTextField.setLink(node, link);/*_I*/}/*I_*/
    }
    /**
     *
     */

    public void setToolTip(MindMapNode node, String key, String value) {/*_I*/_prof.prof.cnt[5914]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5915]++;/*I_*/node.setToolTip(key, value);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5916]++;/*I_*/nodeRefresh(node);/*_I*/}/*I_*/
    }
    // edit begins with home/end or typing (PN 6.2)
	public void edit(KeyEvent e, boolean addNew, boolean editLong) {/*_I*/_prof.prof.cnt[5917]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5918]++;/*I_*/edit.edit(e, addNew, editLong);/*_I*/}/*I_*/
	}

	
    public void setNodeText(MindMapNode selected, String newText) {/*_I*/_prof.prof.cnt[5919]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5920]++;/*I_*/edit.setNodeText(selected, newText);/*_I*/}/*I_*/
    }

    /**
     *
     */

    public void setEdgeWidth(MindMapNode node, int width) {/*_I*/_prof.prof.cnt[5921]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5922]++;/*I_*/EdgeWidth_1.setEdgeWidth(node, width);/*_I*/}/*I_*/
    }
    /**
     *
     */

    public void setEdgeStyle(MindMapNode node, String style) {/*_I*/_prof.prof.cnt[5923]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5924]++;/*I_*/EdgeStyle_bezier.setEdgeStyle(node, style);/*_I*/}/*I_*/
    }
    /**
     *
     */

    public void setNodeStyle(MindMapNode node, String style) {/*_I*/_prof.prof.cnt[5925]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5926]++;/*I_*/fork.setStyle(node, style);/*_I*/}/*I_*/
    }
     public Transferable cut() {/*_I*/_prof.prof.cnt[5927]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5928]++;/*I_*/return cut(getSelecteds());/*_I*/}/*I_*/
	}

     
     	
	public Transferable cut(List nodeList) {/*_I*/_prof.prof.cnt[5929]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5930]++;/*I_*/return cut.cut(nodeList);/*_I*/}/*I_*/
	}
	public void paste(Transferable t, MindMapNode parent) {/*_I*/_prof.prof.cnt[5931]++;/*I_*/
		/*_I*/_prof.prof.cnt[5932]++;/*I_*/boolean isLeft = false;
		/*_I*/{_prof.prof.cnt[5933]++;/*I_*/if(parent.isLeft()!= null)
			/*_I*/{_prof.prof.cnt[5934]++;/*I_*/isLeft = parent.isLeft().getValue();/*_I*/}/*I_*//*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5935]++;/*I_*/paste(t, /*target=*/parent, /*asSibling=*/ false, isLeft);/*_I*/}/*I_*/ }

	/** @param isLeft determines, whether or not the node is placed on the left or right. **/
	public void paste(Transferable t, MindMapNode target, boolean asSibling, boolean isLeft) {/*_I*/_prof.prof.cnt[5936]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5937]++;/*I_*/paste.paste(t, target, asSibling, isLeft);/*_I*/}/*I_*/
	}

	public void paste(MindMapNode node, MindMapNode parent) {/*_I*/_prof.prof.cnt[5938]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5939]++;/*I_*/paste.paste(node, parent);/*_I*/}/*I_*/
	}


    public static final int NEW_CHILD_WITHOUT_FOCUS = 1;  // old model of insertion
    public static final int NEW_CHILD = 2;
    public static final int NEW_SIBLING_BEHIND = 3;
    public static final int NEW_SIBLING_BEFORE = 4;

    public MindMapNode addNew(final MindMapNode target, final int newNodeMode, final KeyEvent e) {/*_I*/_prof.prof.cnt[5940]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[5941]++;/*I_*/return newChild.addNew(target, newNodeMode, e);/*_I*/}/*I_*/
    }
    
    public 	MindMapNode addNewNode(MindMapNode parent, int index, freemind.main.Tools.BooleanHolder newNodeIsLeft) {/*_I*/_prof.prof.cnt[5942]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5943]++;/*I_*/return newChild.addNewNode(parent, index, newNodeIsLeft);/*_I*/}/*I_*/
    }


	public void deleteNode(MindMapNode selectedNode) {/*_I*/_prof.prof.cnt[5944]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5945]++;/*I_*/deleteChild.deleteNode(selectedNode);/*_I*/}/*I_*/
	}

	public void toggleFolded() {/*_I*/_prof.prof.cnt[5946]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[5947]++;/*I_*/toggleFolded.toggleFolded();/*_I*/}/*I_*/
	}
	
	public void setFolded(MindMapNode node, boolean folded) {/*_I*/_prof.prof.cnt[5948]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[5949]++;/*I_*/toggleFolded.setFolded(node, folded);/*_I*/}/*I_*/
	}
	
	public void displayNode(MindMapNode node){/*_I*/_prof.prof.cnt[5950]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[5951]++;/*I_*/find.displayNode(node, null);/*_I*/}/*I_*/
	}
	
	public void centerNode(MindMapNode node){/*_I*/_prof.prof.cnt[5952]++;/*I_*/
	    	/*_I*/{_prof.prof.cnt[5953]++;/*I_*/if(node.getViewer()==null){
	    	    /*_I*/{_prof.prof.cnt[5954]++;/*I_*/displayNode(node);/*_I*/}/*I_*/
	    	}/*_I*/}/*I_*/
	        // Select the node and scroll to it.
	        /*_I*/{_prof.prof.cnt[5955]++;/*I_*/getView().centerNode(node.getViewer());/*_I*/}/*I_*/
	        /*_I*/{_prof.prof.cnt[5956]++;/*I_*/getView().selectAsTheOnlyOneSelected(node.getViewer());/*_I*/}/*I_*/
	        /*_I*/{_prof.prof.cnt[5957]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/
	}
	
	public String getLinkShortText(MindMapNode node) {/*_I*/_prof.prof.cnt[5958]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[5959]++;/*I_*/return gotoLinkNodeAction.getShortTextForLink(node);/*_I*/}/*I_*/
	}
	
	public void moveNodes(MindMapNode selected, List selecteds, int direction){/*_I*/_prof.prof.cnt[5960]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[5961]++;/*I_*/nodeUp.moveNodes(selected, selecteds, direction);/*_I*/}/*I_*/
	}

	public void joinNodes(MindMapNode selectedNode, List selectedNodes) {/*_I*/_prof.prof.cnt[5962]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[5963]++;/*I_*/joinNodes.joinNodes(selectedNode, selectedNodes);/*_I*/}/*I_*/
	}

    protected void setLinkByFileChooser() {/*_I*/_prof.prof.cnt[5964]++;/*I_*/
		/*_I*/_prof.prof.cnt[5965]++;/*I_*/String relative = getLinkByFileChooser(getFileFilter());
		/*_I*/{_prof.prof.cnt[5966]++;/*I_*/if (relative != null) 
		    /*_I*/{_prof.prof.cnt[5967]++;/*I_*/setLink((NodeAdapter) getSelected(),relative);/*_I*/}/*I_*//*_I*/}/*I_*/
	}
	
	protected void setImageByFileChooser() {/*_I*/_prof.prof.cnt[5968]++;/*I_*/
		/*_I*/_prof.prof.cnt[5969]++;/*I_*/ExampleFileFilter filter = new ExampleFileFilter();
		/*_I*/{_prof.prof.cnt[5970]++;/*I_*/filter.addExtension("jpg");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5971]++;/*I_*/filter.addExtension("jpeg");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5972]++;/*I_*/filter.addExtension("png");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5973]++;/*I_*/filter.addExtension("gif");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5974]++;/*I_*/filter.setDescription("JPG, PNG and GIF Images");/*_I*/}/*I_*/

                // Are there any selected nodes with pictures?                
                /*_I*/_prof.prof.cnt[5975]++;/*I_*/boolean picturesAmongSelecteds = false;
                /*_I*/{_prof.prof.cnt[5976]++;/*I_*/for (ListIterator e = getSelecteds().listIterator();e.hasNext();) {
                   /*_I*/_prof.prof.cnt[5977]++;/*I_*/String link = ((MindMapNode)e.next()).getLink();
                   /*_I*/{_prof.prof.cnt[5978]++;/*I_*/if (link != null) {
                      /*_I*/{_prof.prof.cnt[5979]++;/*I_*/if (filter.accept(new File(link))) {
                         /*_I*/{_prof.prof.cnt[5980]++;/*I_*/picturesAmongSelecteds = true;/*_I*/}/*I_*/
                         /*_I*/{_prof.prof.cnt[5981]++;/*I_*/break;/*_I*/}/*I_*/
                      }/*_I*/}/*I_*/
                   }/*_I*/}/*I_*/
                }/*_I*/}/*I_*/

                /*_I*/{_prof.prof.cnt[5982]++;/*I_*/try {
                	// FIXME: Is this used?????
                   /*_I*/{_prof.prof.cnt[5983]++;/*I_*/if (picturesAmongSelecteds) {
                      /*_I*/{_prof.prof.cnt[5984]++;/*I_*/for (ListIterator e = getSelecteds().listIterator();e.hasNext();) {
                         /*_I*/_prof.prof.cnt[5985]++;/*I_*/MindMapNode node = (MindMapNode)e.next();
                         /*_I*/{_prof.prof.cnt[5986]++;/*I_*/if (node.getLink() != null) {
                            /*_I*/_prof.prof.cnt[5987]++;/*I_*/String possiblyRelative = node.getLink();
                            /*_I*/_prof.prof.cnt[5988]++;/*I_*/String relative = Tools.isAbsolutePath(possiblyRelative) ?
                               new File(possiblyRelative).toURL().toString() : possiblyRelative;
                            /*_I*/{_prof.prof.cnt[5989]++;/*I_*/if (relative != null) {
                               /*_I*/_prof.prof.cnt[5990]++;/*I_*/String strText = "<html><img src=\"" + relative + "\">";
                               /*_I*/{_prof.prof.cnt[5991]++;/*I_*/setLink(node, null);/*_I*/}/*I_*/
                               /*_I*/{_prof.prof.cnt[5992]++;/*I_*/setNodeText(node, strText);/*_I*/}/*I_*/
                            }/*_I*/}/*I_*/
                         }/*_I*/}/*I_*/
                      }/*_I*/}/*I_*/
                   }
                   else {
                      /*_I*/_prof.prof.cnt[5993]++;/*I_*/String relative = getLinkByFileChooser(filter);
                      /*_I*/{_prof.prof.cnt[5994]++;/*I_*/if (relative != null) {
                         /*_I*/_prof.prof.cnt[5995]++;/*I_*/String strText = "<html><img src=\"" + relative + "\">"; 
                         /*_I*/{_prof.prof.cnt[5996]++;/*I_*/setNodeText((MindMapNode)getSelected(),strText);/*_I*/}/*I_*/
                      }/*_I*/}/*I_*/ 
                   }/*_I*/}/*I_*/
                }
                catch (MalformedURLException e) {/*_I*/{_prof.prof.cnt[5997]++;/*I_*/e.printStackTrace();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	}
   
	protected String getLinkByFileChooser(FileFilter fileFilter) {/*_I*/_prof.prof.cnt[5998]++;/*I_*/
        /*_I*/_prof.prof.cnt[5999]++;/*I_*/URL link;
		/*_I*/_prof.prof.cnt[6000]++;/*I_*/String relative = null;
        /*_I*/_prof.prof.cnt[6001]++;/*I_*/File input;
        /*_I*/_prof.prof.cnt[6002]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
        /*_I*/_prof.prof.cnt[6003]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir); 
        /*_I*/{_prof.prof.cnt[6004]++;/*I_*/if (getMap().getFile() == null) {
            /*_I*/{_prof.prof.cnt[6005]++;/*I_*/JOptionPane.showMessageDialog(getFrame().getContentPane(), getText("not_saved_for_link_error"), 
                                          "FreeMind", JOptionPane.WARNING_MESSAGE);/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[6006]++;/*I_*/return null;/*_I*/}/*I_*/
            // In the previous version Freemind automatically displayed save
            // dialog. It happened very often, that user took this save
            // dialog to be an open link dialog; as a result, the new map
            // overwrote the linked map.

        }/*_I*/}/*I_*/
//         if ((getMap().getFile() != null) && (getMap().getFile().getParentFile() != null)) {
//             chooser = new JFileChooser(getMap().getFile().getParentFile());
//         } else {
//             chooser = new JFileChooser();
//         }

		/*_I*/{_prof.prof.cnt[6007]++;/*I_*/if (fileFilter != null) {
           // Set filters, make sure AcceptAll filter comes first
		   /*_I*/{_prof.prof.cnt[6008]++;/*I_*/chooser.setFileFilter(fileFilter);/*_I*/}/*I_*/
		} else {
			/*_I*/{_prof.prof.cnt[6009]++;/*I_*/chooser.setFileFilter(chooser.getAcceptAllFileFilter());/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
 
        /*_I*/_prof.prof.cnt[6010]++;/*I_*/int returnVal = chooser.showOpenDialog(getFrame().getContentPane());
        /*_I*/{_prof.prof.cnt[6011]++;/*I_*/if (returnVal==JFileChooser.APPROVE_OPTION) {
            /*_I*/{_prof.prof.cnt[6012]++;/*I_*/input = chooser.getSelectedFile();/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6013]++;/*I_*/try {
                /*_I*/{_prof.prof.cnt[6014]++;/*I_*/link = input.toURL();/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6015]++;/*I_*/relative = link.toString();/*_I*/}/*I_*/
            } catch (MalformedURLException ex) {
                /*_I*/{_prof.prof.cnt[6016]++;/*I_*/getController().errorMessage(getText("url_error"));/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6017]++;/*I_*/return null;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6018]++;/*I_*/if (getFrame().getProperty("links").equals("relative")) {
                //Create relative URL
                /*_I*/{_prof.prof.cnt[6019]++;/*I_*/try {
                    /*_I*/{_prof.prof.cnt[6020]++;/*I_*/relative = Tools.toRelativeURL(getMap().getFile().toURL(), link);/*_I*/}/*I_*/
                } catch (MalformedURLException ex) {
                    /*_I*/{_prof.prof.cnt[6021]++;/*I_*/getController().errorMessage(getText("url_error"));/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6022]++;/*I_*/return null;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[6023]++;/*I_*/return relative;/*_I*/}/*I_*/
    }

    public void loadURL(String relative) {/*_I*/_prof.prof.cnt[6024]++;/*I_*/
        /*_I*/_prof.prof.cnt[6025]++;/*I_*/URL absolute = null;
        /*_I*/{_prof.prof.cnt[6026]++;/*I_*/if (getMap().getFile() == null) {
            /*_I*/{_prof.prof.cnt[6027]++;/*I_*/getFrame().out("You must save the current map first!");/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6028]++;/*I_*/save();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6029]++;/*I_*/try {
           /*_I*/{_prof.prof.cnt[6030]++;/*I_*/if (Tools.isAbsolutePath(relative)) {
              // Protocol can be identified by rexep pattern "[a-zA-Z]://.*".
              // This should distinguish a protocol path from a file path on most platforms.
              // 1)  UNIX / Linux - obviously
              // 2)  Windows - relative path does not contain :, in absolute path is : followed by \.
              // 3)  Mac - cannot remember

              // If relative is an absolute path, then it cannot be a protocol.
              // At least on Unix and Windows. But this is not true for Mac!!

              // Here is hidden an assumption that the existence of protocol implies !Tools.isAbsolutePath(relative).
              // The code should probably be rewritten to convey more logical meaning, on the other hand
              // it works on Windows and Linux.

              //absolute = new URL("file://"+relative); }
              /*_I*/{_prof.prof.cnt[6031]++;/*I_*/absolute = new File(relative).toURL();/*_I*/}/*I_*/ }
            else /*_I*/{_prof.prof.cnt[6032]++;/*I_*/if(relative.startsWith("#")){
                // inner map link, fc, 12.10.2004
                /*_I*/{_prof.prof.cnt[6033]++;/*I_*/logger.finest("found relative link to "+relative);/*_I*/}/*I_*/
                /*_I*/_prof.prof.cnt[6034]++;/*I_*/String target = relative.substring(1);
                /*_I*/{_prof.prof.cnt[6035]++;/*I_*/try {
                    /*_I*/_prof.prof.cnt[6036]++;/*I_*/MindMapNode node = getNodeFromID(target);
                    /*_I*/{_prof.prof.cnt[6037]++;/*I_*/centerNode(node);/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6038]++;/*I_*/return;/*_I*/}/*I_*/
                } catch (Exception e) {
                    // give "not found" message
                    /*_I*/{_prof.prof.cnt[6039]++;/*I_*/throw new FileNotFoundException(null);/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                
            } else{
              /*_I*/{_prof.prof.cnt[6040]++;/*I_*/absolute = new URL(getMap().getFile().toURL(), relative);/*_I*/}/*I_*/
              // Remark: getMap().getFile().toURL() returns URLs like file:/C:/...
              // It seems, that it does not cause any problems.
            }/*_I*/}/*I_*//*_I*/}/*I_*/

           /*_I*/_prof.prof.cnt[6041]++;/*I_*/String extension = Tools.getExtension(absolute.toString());
           /*_I*/{_prof.prof.cnt[6042]++;/*I_*/if ((extension != null) && extension.equals("mm")) {   // ---- Open Mind Map
              /*_I*/_prof.prof.cnt[6043]++;/*I_*/String fileName = absolute.getFile();
              /*_I*/_prof.prof.cnt[6044]++;/*I_*/File file = new File(fileName);
              /*_I*/{_prof.prof.cnt[6045]++;/*I_*/if(!getController().getMapModuleManager().tryToChangeToMapModule(file.getName())) {
                 //this can lead to confusion if the user handles multiple maps with the same name.
                 /*_I*/{_prof.prof.cnt[6046]++;/*I_*/getFrame().setWaitingCursor(true);/*_I*/}/*I_*/
                 /*_I*/{_prof.prof.cnt[6047]++;/*I_*/load(file);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
           else {                                                 // ---- Open URL in browser
               // fc, 14.12.2003: The following code seems not to be very good. Imagine file names with spaces. Then they occur as %20, now the OS does not find the file, 
               // etc. If this is necessary, this should be done in the openDocument command.
//               if (absolute.getProtocol().equals("file")) {                 
//                  File file = new File (Tools.urlGetFile(absolute));
//                  // If file does not exist, try http protocol (but only if it is reasonable)
//                  if (!file.exists()) {
//                     if (relative.matches("^[-\\.a-z0-9]*/?$")) {
//                        absolute = new URL("http://"+relative); }
//                     else {
//                        // This cannot be a base, to which http:// may be added.
//                        getController().errorMessage("File \""+file+"\" does not exist.");
//                        return; }}}
              /*_I*/{_prof.prof.cnt[6048]++;/*I_*/getFrame().openDocument(absolute);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
        catch (MalformedURLException ex) {
            /*_I*/{_prof.prof.cnt[6049]++;/*I_*/getController().errorMessage(getText("url_error")+"\n"+ex);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6050]++;/*I_*/return;/*_I*/}/*I_*/ }
        catch (FileNotFoundException e) {
            /*_I*/_prof.prof.cnt[6051]++;/*I_*/int returnVal = JOptionPane.showConfirmDialog
               (getView(),
                getText("repair_link_question"),
                getText("repair_link"),
                JOptionPane.YES_NO_OPTION);
            /*_I*/{_prof.prof.cnt[6052]++;/*I_*/if (returnVal==JOptionPane.YES_OPTION) {
                /*_I*/{_prof.prof.cnt[6053]++;/*I_*/setLinkByTextField.actionPerformed(null);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        } catch (Exception e) { 
            /*_I*/{_prof.prof.cnt[6054]++;/*I_*/e.printStackTrace();/*_I*/}/*I_*/ 
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6055]++;/*I_*/getFrame().setWaitingCursor(false);/*_I*/}/*I_*/
    }

    public void loadURL() {/*_I*/_prof.prof.cnt[6056]++;/*I_*/
        /*_I*/_prof.prof.cnt[6057]++;/*I_*/String link = getSelected().getLink();
        /*_I*/{_prof.prof.cnt[6058]++;/*I_*/if (link != null) {
            /*_I*/{_prof.prof.cnt[6059]++;/*I_*/loadURL(link);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    //
    // Convenience methods
    //

    protected Mode getMode() {/*_I*/_prof.prof.cnt[6060]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6061]++;/*I_*/return mode;/*_I*/}/*I_*/
    }

    protected void setMode(Mode mode) {/*_I*/_prof.prof.cnt[6062]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6063]++;/*I_*/this.mode = mode;/*_I*/}/*I_*/
    }

    //FIXME: Wrong as the actual controller is eventually not used (think of more than one map opened).
    protected MapModule getMapModule() {/*_I*/_prof.prof.cnt[6064]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6065]++;/*I_*/return getController().getMapModuleManager().getMapModule();/*_I*/}/*I_*/
    }

    //FIXME: Wrong as the actual controller is eventually not used (think of more than one map opened).
    public MapAdapter getMap() {/*_I*/_prof.prof.cnt[6066]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6067]++;/*I_*/if (getMapModule() != null) {
            /*_I*/{_prof.prof.cnt[6068]++;/*I_*/return (MapAdapter)getMapModule().getModel();/*_I*/}/*I_*/
        } else {
            /*_I*/{_prof.prof.cnt[6069]++;/*I_*/return null;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    public MindMapNode getRootNode(){/*_I*/_prof.prof.cnt[6070]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6071]++;/*I_*/return (MindMapNode) getMap().getRoot();/*_I*/}/*I_*/
    }
    
    public URL getResource (String name) {/*_I*/_prof.prof.cnt[6072]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6073]++;/*I_*/return getFrame().getResource(name);/*_I*/}/*I_*/
    }

    public Controller getController() {/*_I*/_prof.prof.cnt[6074]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6075]++;/*I_*/return getMode().getController();/*_I*/}/*I_*/
    }

    public FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[6076]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6077]++;/*I_*/return getController().getFrame();/*_I*/}/*I_*/
    }

	/** This was inserted by fc, 10.03.04 to enable all actions to refer to its controller easily.*/
	public ControllerAdapter getModeController() {/*_I*/_prof.prof.cnt[6078]++;/*I_*/
		/*_I*/{_prof.prof.cnt[6079]++;/*I_*/return this;/*_I*/}/*I_*/	
	}

	// fc, 29.2.2004: there is no sense in having this private and the controller public,
	// because the getController().getModel() method is available anyway.
    public MapAdapter getModel() {/*_I*/_prof.prof.cnt[6080]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6081]++;/*I_*/return (MapAdapter)getController().getModel();/*_I*/}/*I_*/
    }

    public MapView getView() {/*_I*/_prof.prof.cnt[6082]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6083]++;/*I_*/return getController().getView();/*_I*/}/*I_*/
    }

    protected void updateMapModuleName() {/*_I*/_prof.prof.cnt[6084]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6085]++;/*I_*/getController().getMapModuleManager().updateMapModuleName();/*_I*/}/*I_*/
    }

	/* ***********************************************************
	*  Helper methods
	* ***********************************************************/
	public NodeAdapter getNodeFromID(String nodeID) {/*_I*/_prof.prof.cnt[6086]++;/*I_*/
		/*_I*/_prof.prof.cnt[6087]++;/*I_*/NodeAdapter node =
			(NodeAdapter) getMap().getLinkRegistry().getTargetForID(nodeID);
		/*_I*/{_prof.prof.cnt[6088]++;/*I_*/if(node == null) {
		    /*_I*/{_prof.prof.cnt[6089]++;/*I_*/throw new IllegalArgumentException("Node belonging to the node id "+ nodeID + " not found.");/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[6090]++;/*I_*/return node;/*_I*/}/*I_*/
	}
	public String getNodeID(MindMapNode selected) {/*_I*/_prof.prof.cnt[6091]++;/*I_*/
		/*_I*/{_prof.prof.cnt[6092]++;/*I_*/getMap().getLinkRegistry().registerLinkTarget(selected);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[6093]++;/*I_*/return getMap().getLinkRegistry().getLabel(selected);/*_I*/}/*I_*/
	}


    public MindMapNode getSelected() {/*_I*/_prof.prof.cnt[6094]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[6095]++;/*I_*/if(getView() != null && getView().getSelected()!=null)
        	/*_I*/{_prof.prof.cnt[6096]++;/*I_*/return (MindMapNode)getView().getSelected().getModel();/*_I*/}/*I_*//*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[6097]++;/*I_*/return null;/*_I*/}/*I_*/        	
    }

    public boolean extendSelection(MouseEvent e) {/*_I*/_prof.prof.cnt[6098]++;/*I_*/
        /*_I*/_prof.prof.cnt[6099]++;/*I_*/NodeView newlySelectedNodeView = (NodeView)e.getSource();
        //MindMapNode newlySelectedNode = newlySelectedNodeView.getModel();
        /*_I*/_prof.prof.cnt[6100]++;/*I_*/boolean extend = e.isControlDown(); 
        /*_I*/_prof.prof.cnt[6101]++;/*I_*/boolean range = e.isShiftDown(); 
        /*_I*/_prof.prof.cnt[6102]++;/*I_*/boolean branch = e.isAltGraphDown() || e.isAltDown(); /* windows alt, linux altgraph .... */ 
        /*_I*/_prof.prof.cnt[6103]++;/*I_*/boolean retValue = false;

        /*_I*/{_prof.prof.cnt[6104]++;/*I_*/if (extend || range || branch || !getView().isSelected(newlySelectedNodeView)) {
            /*_I*/{_prof.prof.cnt[6105]++;/*I_*/if (!range) {
                /*_I*/{_prof.prof.cnt[6106]++;/*I_*/if (extend)
                    /*_I*/{_prof.prof.cnt[6107]++;/*I_*/getView().toggleSelected(newlySelectedNodeView);/*_I*/}/*I_*/
                else
                    /*_I*/{_prof.prof.cnt[6108]++;/*I_*/select(newlySelectedNodeView);/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6109]++;/*I_*/retValue = true;/*_I*/}/*I_*/
            }
            else {
                /*_I*/{_prof.prof.cnt[6110]++;/*I_*/retValue = getView().selectContinuous(newlySelectedNodeView);/*_I*/}/*I_*/ 
//                 /* fc, 25.1.2004: replace getView by controller methods.*/
//                 if (newlySelectedNodeView != getView().getSelected() &&
//                     newlySelectedNodeView.isSiblingOf(getView().getSelected())) {
//                     getView().selectContinuous(newlySelectedNodeView); 
//                     retValue = true;
//                 } else {
//                     /* if shift was down, but no range can be selected, then the new node is simply selected: */
//                     if(!getView().isSelected(newlySelectedNodeView)) {
//                         getView().toggleSelected(newlySelectedNodeView);
//                         retValue = true;
//                     }
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6111]++;/*I_*/if(branch) {
                /*_I*/{_prof.prof.cnt[6112]++;/*I_*/getView().selectBranch(newlySelectedNodeView, extend);/*_I*/}/*I_*/ 
                /*_I*/{_prof.prof.cnt[6113]++;/*I_*/retValue = true;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/    
        }/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[6114]++;/*I_*/if(retValue) {
            /*_I*/{_prof.prof.cnt[6115]++;/*I_*/e.consume();/*_I*/}/*I_*/
        
            // Display link in status line
            /*_I*/_prof.prof.cnt[6116]++;/*I_*/String link = newlySelectedNodeView.getModel().getLink();
            /*_I*/{_prof.prof.cnt[6117]++;/*I_*/link = (link != null ? link : " ");/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6118]++;/*I_*/getController().getFrame().out(link);/*_I*/}/*I_*/ 
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6119]++;/*I_*/return retValue;/*_I*/}/*I_*/
    }

    public void select( NodeView node) {/*_I*/_prof.prof.cnt[6120]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6121]++;/*I_*/getView().scrollNodeToVisible(node);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6122]++;/*I_*/getView().selectAsTheOnlyOneSelected(node);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6123]++;/*I_*/getView().setSiblingMaxLevel(node.getModel().getNodeLevel());/*_I*/}/*I_*/ // this level is default
    }

    public void select( MindMapNode selected) {/*_I*/_prof.prof.cnt[6124]++;/*I_*/
        // are they visible visible?
        /*_I*/{_prof.prof.cnt[6125]++;/*I_*/displayNode(selected);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6126]++;/*I_*/select(selected.getViewer());/*_I*/}/*I_*/
    }

    public void selectMultipleNodes(MindMapNode focussed, Collection selecteds) {/*_I*/_prof.prof.cnt[6127]++;/*I_*/
        // are they visible visible?
        /*_I*/{_prof.prof.cnt[6128]++;/*I_*/for (Iterator i = selecteds.iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[6129]++;/*I_*/MindMapNode node = (MindMapNode) i.next();
	        /*_I*/{_prof.prof.cnt[6130]++;/*I_*/displayNode(node);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        // this one must be visible.
        /*_I*/{_prof.prof.cnt[6131]++;/*I_*/select(focussed);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6132]++;/*I_*/for (Iterator i = selecteds.iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[6133]++;/*I_*/MindMapNode node = (MindMapNode) i.next();
            /*_I*/{_prof.prof.cnt[6134]++;/*I_*/getView().makeTheSelected(node.getViewer());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6135]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/ // focus fix
    }
    
    public void selectBranch(MindMapNode selected, boolean extend) {/*_I*/_prof.prof.cnt[6136]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6137]++;/*I_*/displayNode(selected);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6138]++;/*I_*/getView().selectBranch(selected.getViewer(), extend);/*_I*/}/*I_*/
    }
    
	public void invokeHook(ModeControllerHook hook) {/*_I*/_prof.prof.cnt[6139]++;/*I_*/
		/*_I*/{_prof.prof.cnt[6140]++;/*I_*/hook.setController(this);/*_I*/}/*I_*/
		// initialize:
		// the main invocation:
		/*_I*/{_prof.prof.cnt[6141]++;/*I_*/hook.startupMapHook();/*_I*/}/*I_*/
		// and good bye.
		/*_I*/{_prof.prof.cnt[6142]++;/*I_*/hook.shutdownMapHook();/*_I*/}/*I_*/
	}
	
	/**
	  *  
	  */
	public void invokeHooksRecursively(NodeAdapter node, MindMap map) {/*_I*/_prof.prof.cnt[6143]++;/*I_*/
		 /*_I*/{_prof.prof.cnt[6144]++;/*I_*/for(Iterator i = node.childrenUnfolded(); i.hasNext();) {
			 /*_I*/_prof.prof.cnt[6145]++;/*I_*/NodeAdapter child = (NodeAdapter) i.next();
			 /*_I*/{_prof.prof.cnt[6146]++;/*I_*/invokeHooksRecursively(child, map);/*_I*/}/*I_*/
		 }/*_I*/}/*I_*/
		 /*_I*/{_prof.prof.cnt[6147]++;/*I_*/for(Iterator i = node.getHooks().iterator(); i.hasNext();) {
			 /*_I*/_prof.prof.cnt[6148]++;/*I_*/PermanentNodeHook hook = (PermanentNodeHook) i.next();
			 /*_I*/{_prof.prof.cnt[6149]++;/*I_*/hook.setController(this);/*_I*/}/*I_*/
			 /*_I*/{_prof.prof.cnt[6150]++;/*I_*/hook.setMap(map);/*_I*/}/*I_*/
			 /*_I*/{_prof.prof.cnt[6151]++;/*I_*/node.invokeHook(hook);/*_I*/}/*I_*/ 
		 }/*_I*/}/*I_*/
	}



	public NodeHook createNodeHook(String hookName, MindMapNode node,
			MindMap map) {/*_I*/_prof.prof.cnt[6152]++;/*I_*/
		/*_I*/_prof.prof.cnt[6153]++;/*I_*/HookFactory hookFactory = getFrame().getHookFactory();
		/*_I*/_prof.prof.cnt[6154]++;/*I_*/NodeHook hook = (NodeHook) hookFactory.createNodeHook(hookName);
		/*_I*/{_prof.prof.cnt[6155]++;/*I_*/hook.setController(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[6156]++;/*I_*/hook.setMap(map);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[6157]++;/*I_*/if (hook instanceof PermanentNodeHook) {
			/*_I*/_prof.prof.cnt[6158]++;/*I_*/PermanentNodeHook permHook = (PermanentNodeHook) hook;
			/*_I*/{_prof.prof.cnt[6159]++;/*I_*/if(hookFactory.getInstanciationMethod(hookName).isSingleton()) {
				// search for already instanciated hooks of this type:
				/*_I*/_prof.prof.cnt[6160]++;/*I_*/PermanentNodeHook otherHook = hookFactory.getHookInNode(node, hookName);
				/*_I*/{_prof.prof.cnt[6161]++;/*I_*/if(otherHook != null) {
					/*_I*/{_prof.prof.cnt[6162]++;/*I_*/return otherHook;/*_I*/}/*I_*/
				}/*_I*/}/*I_*/
			}/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[6163]++;/*I_*/node.addHook(permHook);/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[6164]++;/*I_*/return hook;/*_I*/}/*I_*/
	}
	
	public void addHook(MindMapNode focussed, List selecteds, String hookName) {/*_I*/_prof.prof.cnt[6165]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[6166]++;/*I_*/nodeHookAction.addHook(focussed, selecteds, hookName);/*_I*/}/*I_*/
	}
	
	
    protected class OpenAction extends AbstractAction {
        ControllerAdapter mc;
        public OpenAction(ControllerAdapter modeController) {
            super(getText("open"), new ImageIcon(getResource("images/fileopen.png")));/*_I*/_prof.prof.cnt[6167]++;/*I_*//*_I*/_prof.prof.cnt[6168]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6169]++;/*I_*/mc = modeController;/*_I*/}/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6170]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6171]++;/*I_*/mc.open();/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[6172]++;/*I_*/getController().setTitle();/*_I*/}/*I_*/ // Possible update of read-only
        }
    }

    protected class SaveAction extends AbstractAction {
        ControllerAdapter mc;
        public SaveAction(ControllerAdapter modeController) {
            super(getText("save"), new ImageIcon(getResource("images/filesave.png")));/*_I*/_prof.prof.cnt[6173]++;/*I_*//*_I*/_prof.prof.cnt[6174]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6175]++;/*I_*/mc = modeController;/*_I*/}/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6176]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6177]++;/*I_*/mc.save();/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6178]++;/*I_*/getFrame().out(getText("saved"));/*_I*/}/*I_*/ // perhaps... (PN)
			/*_I*/{_prof.prof.cnt[6179]++;/*I_*/getController().setTitle();/*_I*/}/*I_*/ // Possible update of read-only
        }
    }

    protected class SaveAsAction extends AbstractAction {
        ControllerAdapter mc;
        public SaveAsAction(ControllerAdapter modeController) {
            super(getText("save_as"), new ImageIcon(getResource("images/filesaveas.png")));/*_I*/_prof.prof.cnt[6180]++;/*I_*//*_I*/_prof.prof.cnt[6181]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6182]++;/*I_*/mc = modeController;/*_I*/}/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6183]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6184]++;/*I_*/mc.saveAs();/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[6185]++;/*I_*/getController().setTitle();/*_I*/}/*I_*/ // Possible update of read-only
        }
    }

	protected class ModeControllerHookAction extends AbstractAction {
		String hookName;
		ModeController controller;
		public ModeControllerHookAction(String hookName, ModeController controller) {
			super(hookName);/*_I*/_prof.prof.cnt[6186]++;/*I_*//*_I*/_prof.prof.cnt[6187]++;/*I_*/
			/*_I*/{_prof.prof.cnt[6188]++;/*I_*/this.hookName = hookName;/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[6189]++;/*I_*/this.controller = controller;/*_I*/}/*I_*/
		}

		public void actionPerformed(ActionEvent arg0) {/*_I*/_prof.prof.cnt[6190]++;/*I_*/
			/*_I*/_prof.prof.cnt[6191]++;/*I_*/HookFactory hookFactory = getFrame().getHookFactory();
			// two different invocation methods:single or selecteds
			/*_I*/_prof.prof.cnt[6192]++;/*I_*/ModeControllerHook hook = hookFactory.createModeControllerHook(hookName);
			/*_I*/{_prof.prof.cnt[6193]++;/*I_*/hook.setController(controller);/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[6194]++;/*I_*/invokeHook(hook);/*_I*/}/*I_*/							
		} 
  			
	}


    public String marshall(XmlAction action) {/*_I*/_prof.prof.cnt[6195]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6196]++;/*I_*/return getController().marshall(action);/*_I*/}/*I_*/
	}

	public XmlAction unMarshall(String inputString) {/*_I*/_prof.prof.cnt[6197]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6198]++;/*I_*/return getController().unMarshall(inputString);/*_I*/}/*I_*/
	}

	protected class EditLongAction extends AbstractAction {
        public EditLongAction() {
            super(getText("edit_long_node"));/*_I*/_prof.prof.cnt[6199]++;/*I_*//*_I*/_prof.prof.cnt[6200]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6201]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6202]++;/*I_*/edit(null, false, true);/*_I*/}/*I_*/
        }
    }

    protected class SetLinkByFileChooserAction extends AbstractAction {
        public SetLinkByFileChooserAction() {
            super(getText("set_link_by_filechooser"));/*_I*/_prof.prof.cnt[6203]++;/*I_*//*_I*/_prof.prof.cnt[6204]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6205]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6206]++;/*I_*/setLinkByFileChooser();/*_I*/}/*I_*/
        }
    }

	protected class SetImageByFileChooserAction extends AbstractAction {
		public SetImageByFileChooserAction() {
			super(getText("set_image_by_filechooser"));/*_I*/_prof.prof.cnt[6207]++;/*I_*//*_I*/_prof.prof.cnt[6208]++;/*I_*/
		}
		public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6209]++;/*I_*/
                       /*_I*/{_prof.prof.cnt[6210]++;/*I_*/setImageByFileChooser();/*_I*/}/*I_*/
                        /*_I*/{_prof.prof.cnt[6211]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/
		}
	}

    protected class FollowLinkAction extends AbstractAction {
        public FollowLinkAction() {
            super(getText("follow_link"));/*_I*/_prof.prof.cnt[6212]++;/*I_*//*_I*/_prof.prof.cnt[6213]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6214]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6215]++;/*I_*/loadURL();/*_I*/}/*I_*/
        }
    }


    protected class FileOpener implements DropTargetListener {
        private boolean isDragAcceptable(DropTargetDragEvent event) {/*_I*/_prof.prof.cnt[6216]++;/*I_*/
            // check if there is at least one File Type in the list
            /*_I*/_prof.prof.cnt[6217]++;/*I_*/DataFlavor[] flavors = event.getCurrentDataFlavors();
            /*_I*/{_prof.prof.cnt[6218]++;/*I_*/for (int i = 0; i < flavors.length; i++) {
                /*_I*/{_prof.prof.cnt[6219]++;/*I_*/if (flavors[i].isFlavorJavaFileListType()) {
                    //              event.acceptDrag(DnDConstants.ACTION_COPY);
                    /*_I*/{_prof.prof.cnt[6220]++;/*I_*/return true;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            //      event.rejectDrag();
            /*_I*/{_prof.prof.cnt[6221]++;/*I_*/return false;/*_I*/}/*I_*/
        }

        private boolean isDropAcceptable(DropTargetDropEvent event) {/*_I*/_prof.prof.cnt[6222]++;/*I_*/
            // check if there is at least one File Type in the list
            /*_I*/_prof.prof.cnt[6223]++;/*I_*/DataFlavor[] flavors = event.getCurrentDataFlavors();
            /*_I*/{_prof.prof.cnt[6224]++;/*I_*/for (int i = 0; i < flavors.length; i++) {
                /*_I*/{_prof.prof.cnt[6225]++;/*I_*/if (flavors[i].isFlavorJavaFileListType()) {
                    /*_I*/{_prof.prof.cnt[6226]++;/*I_*/return true;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6227]++;/*I_*/return false;/*_I*/}/*I_*/
        }

        public void drop (DropTargetDropEvent dtde) {/*_I*/_prof.prof.cnt[6228]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6229]++;/*I_*/if(!isDropAcceptable(dtde)) {
                /*_I*/{_prof.prof.cnt[6230]++;/*I_*/dtde.rejectDrop();/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6231]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6232]++;/*I_*/dtde.acceptDrop(DnDConstants.ACTION_COPY);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6233]++;/*I_*/try {
                /*_I*/_prof.prof.cnt[6234]++;/*I_*/Object data =
                    dtde.getTransferable().getTransferData(DataFlavor.javaFileListFlavor);
                /*_I*/{_prof.prof.cnt[6235]++;/*I_*/if (data == null) {
                    // Shouldn't happen because dragEnter() rejects drags w/out at least
                    // one javaFileListFlavor. But just in case it does ...
                    /*_I*/{_prof.prof.cnt[6236]++;/*I_*/dtde.dropComplete(false);/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6237]++;/*I_*/return;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                /*_I*/_prof.prof.cnt[6238]++;/*I_*/Iterator iterator = ((List)data).iterator();
                /*_I*/{_prof.prof.cnt[6239]++;/*I_*/while (iterator.hasNext()) {
                    /*_I*/_prof.prof.cnt[6240]++;/*I_*/File file = (File)iterator.next();
                    /*_I*/{_prof.prof.cnt[6241]++;/*I_*/load(file);/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }
            catch (Exception e) {
                /*_I*/{_prof.prof.cnt[6242]++;/*I_*/JOptionPane.showMessageDialog(getView(),
                                              "Couldn't open dropped file(s). Reason: " + e.getMessage()
                                              //getText("file_not_found")
                                              );/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6243]++;/*I_*/dtde.dropComplete(false);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6244]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6245]++;/*I_*/dtde.dropComplete(true);/*_I*/}/*I_*/
        }

        public void dragEnter (DropTargetDragEvent dtde) {/*_I*/_prof.prof.cnt[6246]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6247]++;/*I_*/if(!isDragAcceptable(dtde)) {
                /*_I*/{_prof.prof.cnt[6248]++;/*I_*/dtde.rejectDrag();/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6249]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }

        public void dragOver (DropTargetDragEvent e) {/*_I*/_prof.prof.cnt[6250]++;/*I_*/}
        public void dragExit (DropTargetEvent e) {/*_I*/_prof.prof.cnt[6251]++;/*I_*/}
        public void dragScroll (DropTargetDragEvent e) {/*_I*/_prof.prof.cnt[6252]++;/*I_*/}
        public void dropActionChanged (DropTargetDragEvent e) {/*_I*/_prof.prof.cnt[6253]++;/*I_*/}
    }


	/**
	 * @return
	 */
	public ObjectFactory getActionXmlFactory() {/*_I*/_prof.prof.cnt[6254]++;/*I_*/
		/*_I*/{_prof.prof.cnt[6255]++;/*I_*/return getController().getActionXmlFactory();/*_I*/}/*I_*/
	}

    /**
     * @return
     */
    public Color getSelectionColor() {/*_I*/_prof.prof.cnt[6256]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6257]++;/*I_*/return selectionColor;/*_I*/}/*I_*/
    }

    /**
     * @return
     */
    public Clipboard getClipboard() {/*_I*/_prof.prof.cnt[6258]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6259]++;/*I_*/return clipboard;/*_I*/}/*I_*/
    }


    /* (non-Javadoc)
     * @see freemind.modes.ModeController#updatePopupMenu(freemind.controller.StructuredMenuHolder)
     */
    public void updatePopupMenu(StructuredMenuHolder holder) {/*_I*/_prof.prof.cnt[6260]++;/*I_*/

    }

    /**
     *
     */

    public void shutdownController() {/*_I*/_prof.prof.cnt[6261]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6262]++;/*I_*/setAllActions(false);/*_I*/}/*I_*/
    }
    /**
     *
     */

    public void startupController() {/*_I*/_prof.prof.cnt[6263]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6264]++;/*I_*/setAllActions(true);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6265]++;/*I_*/if (getFrame().getView() != null) {
            /*_I*/_prof.prof.cnt[6266]++;/*I_*/DropTarget dropTarget = new DropTarget(getFrame().getView(),
                    new FileOpener());
        }/*_I*/}/*I_*/
    }
    /**
     *
     */

    /**
     * @param node
     * @param position
     * @param newText
     */
    public void splitNode(MindMapNode node, int caretPosition, String newText) {/*_I*/_prof.prof.cnt[6267]++;/*I_*/
            //If there are children, they go to the node below
            /*_I*/_prof.prof.cnt[6268]++;/*I_*/String futureText = newText != null ? newText : node.toString();

            /*_I*/_prof.prof.cnt[6269]++;/*I_*/String newLowerContent = futureText.substring(caretPosition, futureText.length());
            /*_I*/_prof.prof.cnt[6270]++;/*I_*/String newUpperContent = futureText.substring(0,caretPosition);

            /*_I*/{_prof.prof.cnt[6271]++;/*I_*/setNodeText(node, newLowerContent);/*_I*/}/*I_*/

            /*_I*/_prof.prof.cnt[6272]++;/*I_*/MindMapNode parent = node.getParentNode();
            /*_I*/_prof.prof.cnt[6273]++;/*I_*/MindMapNode upperNode = addNewNode(parent, parent.getChildPosition(node), parent.isLeft());
            /*_I*/{_prof.prof.cnt[6274]++;/*I_*/upperNode.setColor(node.getColor());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6275]++;/*I_*/upperNode.setFont(node.getFont());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6276]++;/*I_*/setNodeText(upperNode, newUpperContent);/*_I*/}/*I_*/
        
    }
	public void moveNodePosition(MindMapNode node, int vGap, int hGap,
            int shiftY) {/*_I*/_prof.prof.cnt[6277]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[6278]++;/*I_*/moveNodeAction.moveNodeTo(node, vGap, hGap, shiftY);/*_I*/}/*I_*/
	}


    
    
}
