/*
 * FreeMind - A Program for creating and viewing Mindmaps Copyright (C)
 * 2000-2001 Joerg Mueller <joergmueller@bigfoot.com> See COPYING for Details
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* $Id: EdgeAdapter.java,v 1.14.18.3 2005/06/14 20:38:07 christianfoltin Exp $ */

package freemind.modes;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Stroke;

import freemind.controller.Controller;
import freemind.main.FreeMind;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.main.XMLElement;

public abstract class EdgeAdapter extends LineAdapter implements MindMapEdge {


    private static Color standardColor = null;
    private static String standardStyle = null;
    private static LineAdapterListener listener = null;

    public static final int WIDTH_PARENT = -1;

    public static final int WIDTH_THIN = 0;

    private static Color standardEdgeColor = new Color(0);

    public EdgeAdapter(MindMapNode target, FreeMindMain frame) {
        this(target, frame, FreeMind.RESOURCES_EDGE_COLOR, FreeMind.RESOURCES_EDGE_STYLE);/*_I*/_prof.prof.cnt[9709]++;/*I_*//*_I*/_prof.prof.cnt[9710]++;/*I_*/
    }

    /** For derived classes. */
    protected EdgeAdapter(MindMapNode target, FreeMindMain frame,
            String standardColorPropertyString,
            String standardStylePropertyString) {
        super(target, frame, standardColorPropertyString,
                standardStylePropertyString);/*_I*/_prof.prof.cnt[9711]++;/*I_*//*_I*/_prof.prof.cnt[9712]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9713]++;/*I_*/NORMAL_WIDTH = WIDTH_PARENT;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9714]++;/*I_*/if(listener == null) {
            /*_I*/{_prof.prof.cnt[9715]++;/*I_*/listener = new LineAdapterListener();/*_I*/}/*I_*/ 
            /*_I*/{_prof.prof.cnt[9716]++;/*I_*/Controller.addPropertyChangeListener(listener);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    //
    // Attributes
    //

    public Color getColor() {/*_I*/_prof.prof.cnt[9717]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9718]++;/*I_*/if (color == null) {
            /*_I*/{_prof.prof.cnt[9719]++;/*I_*/if (getTarget().isRoot()) {
                /*_I*/{_prof.prof.cnt[9720]++;/*I_*/return getStandardColor();/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[9721]++;/*I_*/return getSource().getEdge().getColor();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9722]++;/*I_*/return color;/*_I*/}/*I_*/
    }

    public Color getRealColor() {/*_I*/_prof.prof.cnt[9723]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9724]++;/*I_*/return color;/*_I*/}/*I_*/
    }

    public int getWidth() {/*_I*/_prof.prof.cnt[9725]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9726]++;/*I_*/if (width == WIDTH_PARENT) {
            /*_I*/{_prof.prof.cnt[9727]++;/*I_*/if (getTarget().isRoot()) {
                /*_I*/{_prof.prof.cnt[9728]++;/*I_*/return WIDTH_THIN;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[9729]++;/*I_*/return getSource().getEdge().getWidth();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9730]++;/*I_*/return width;/*_I*/}/*I_*/
    }

    public Stroke getStroke() {/*_I*/_prof.prof.cnt[9731]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9732]++;/*I_*/if (width == WIDTH_THIN)
            /*_I*/{_prof.prof.cnt[9733]++;/*I_*/return null;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9734]++;/*I_*/if (stroke == null) {
            /*_I*/{_prof.prof.cnt[9735]++;/*I_*/if (getTarget().isRoot()) {
                /*_I*/{_prof.prof.cnt[9736]++;/*I_*/return null;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[9737]++;/*I_*/return getSource().getEdge().getStroke();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9738]++;/*I_*/return stroke;/*_I*/}/*I_*/
    }

    public void setWidth(int width) {/*_I*/_prof.prof.cnt[9739]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9740]++;/*I_*/this.width = width;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9741]++;/*I_*/stroke = ((width == WIDTH_PARENT) || (width == WIDTH_THIN)) ? null
                : new BasicStroke(getWidth(), BasicStroke.CAP_BUTT,
                        BasicStroke.JOIN_MITER);/*_I*/}/*I_*/
    }

    public int getRealWidth() {/*_I*/_prof.prof.cnt[9742]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9743]++;/*I_*/return width;/*_I*/}/*I_*/
    }

    public String getStyle() {/*_I*/_prof.prof.cnt[9744]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9745]++;/*I_*/if (style == null) {
            /*_I*/{_prof.prof.cnt[9746]++;/*I_*/if (getTarget().isRoot()) {
                /*_I*/{_prof.prof.cnt[9747]++;/*I_*/return getFrame().getProperty(standardStylePropertyString);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[9748]++;/*I_*/return getSource().getEdge().getStyle();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9749]++;/*I_*/return style;/*_I*/}/*I_*/
    }

    ///////////
    // Private Methods
    /////////

    private MindMapNode getSource() {/*_I*/_prof.prof.cnt[9750]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9751]++;/*I_*/return target.getParentNode();/*_I*/}/*I_*/
    }

    public XMLElement save() {/*_I*/_prof.prof.cnt[9752]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9753]++;/*I_*/if (style != null || color != null || width != WIDTH_PARENT) {
            /*_I*/_prof.prof.cnt[9754]++;/*I_*/XMLElement edge = new XMLElement();
            /*_I*/{_prof.prof.cnt[9755]++;/*I_*/edge.setName("edge");/*_I*/}/*I_*/

            /*_I*/{_prof.prof.cnt[9756]++;/*I_*/if (style != null) {
                /*_I*/{_prof.prof.cnt[9757]++;/*I_*/edge.setAttribute("STYLE", style);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[9758]++;/*I_*/if (color != null) {
                /*_I*/{_prof.prof.cnt[9759]++;/*I_*/edge.setAttribute("COLOR", Tools.colorToXml(color));/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[9760]++;/*I_*/if (width != WIDTH_PARENT) {
                /*_I*/{_prof.prof.cnt[9761]++;/*I_*/if (width == WIDTH_THIN)
                    /*_I*/{_prof.prof.cnt[9762]++;/*I_*/edge.setAttribute("WIDTH", "thin");/*_I*/}/*I_*/
                else
                    /*_I*/{_prof.prof.cnt[9763]++;/*I_*/edge.setAttribute("WIDTH", Integer.toString(width));/*_I*/}/*I_*//*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[9764]++;/*I_*/return edge;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9765]++;/*I_*/return null;/*_I*/}/*I_*/
    }
    protected Color getStandardColor() {/*_I*/_prof.prof.cnt[9766]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9767]++;/*I_*/return standardColor;/*_I*/}/*I_*/
    }
    protected void setStandardColor(Color standardColor) {/*_I*/_prof.prof.cnt[9768]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9769]++;/*I_*/EdgeAdapter.standardColor = standardColor;/*_I*/}/*I_*/
    }
    protected String getStandardStyle() {/*_I*/_prof.prof.cnt[9770]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9771]++;/*I_*/return standardStyle;/*_I*/}/*I_*/
    }
    protected void setStandardStyle(String standardStyle) {/*_I*/_prof.prof.cnt[9772]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9773]++;/*I_*/EdgeAdapter.standardStyle = standardStyle;/*_I*/}/*I_*/
    }
}
