/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: LineAdapter.java,v 1.2.18.2 2005/06/14 20:38:07 christianfoltin Exp $*/

package freemind.modes;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Stroke;

import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.preferences.FreemindPropertyListener;

public abstract class LineAdapter implements MindMapLine {

    protected FreeMindMain frame;
    protected MindMapNode target;
    protected String standardColorPropertyString;
    protected String standardStylePropertyString;
    
	public static final int DEFAULT_WIDTH = -1;
	protected int NORMAL_WIDTH = 1;

    //recursive attributes. may be accessed directly by the save() method.
    protected Color color; 
    protected String style;
    protected int width;
	protected Stroke stroke;

    //
    // Constructors
    //
    public LineAdapter(MindMapNode target,FreeMindMain frame, String standardColorPropertyString, String standardStylePropertyString) {/*_I*/_prof.prof.cnt[4961]++;/*I_*/
        /*_I*/{_prof.prof.cnt[4962]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4963]++;/*I_*/this.target = target;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4964]++;/*I_*/this.standardColorPropertyString = standardColorPropertyString;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4965]++;/*I_*/this.standardStylePropertyString = standardStylePropertyString;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4966]++;/*I_*/width = DEFAULT_WIDTH;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4967]++;/*I_*/stroke = null;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4968]++;/*I_*/updateStandards();/*_I*/}/*I_*/

    }

    //
    // Attributes
    //

    /**
     * @param standardColorPropertyString
     */
    protected void updateStandards() {/*_I*/_prof.prof.cnt[4969]++;/*I_*/
        /*_I*/{_prof.prof.cnt[4970]++;/*I_*/if (getStandardColor() == null) {
            /*_I*/_prof.prof.cnt[4971]++;/*I_*/String stdColor = getFrame().getProperty(
                    standardColorPropertyString);
            /*_I*/{_prof.prof.cnt[4972]++;/*I_*/if (stdColor != null && stdColor.length() == 7) {
                /*_I*/{_prof.prof.cnt[4973]++;/*I_*/setStandardColor(Tools.xmlToColor(stdColor));/*_I*/}/*I_*/
            } else {
                /*_I*/{_prof.prof.cnt[4974]++;/*I_*/setStandardColor(Color.RED);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4975]++;/*I_*/if (getStandardStyle() == null) {
            /*_I*/_prof.prof.cnt[4976]++;/*I_*/String stdStyle = getFrame().getProperty(
                    standardStylePropertyString);
            /*_I*/{_prof.prof.cnt[4977]++;/*I_*/if (stdStyle != null ) {
                /*_I*/{_prof.prof.cnt[4978]++;/*I_*/setStandardStyle(stdStyle);/*_I*/}/*I_*/
            } else {
//                setStandardStyle(Style.RED);
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    public FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[4979]++;/*I_*/
        /*_I*/{_prof.prof.cnt[4980]++;/*I_*/return frame;/*_I*/}/*I_*/
    }

    public Color getColor() {/*_I*/_prof.prof.cnt[4981]++;/*I_*/
        /*_I*/{_prof.prof.cnt[4982]++;/*I_*/if(color==null) {
            /*_I*/{_prof.prof.cnt[4983]++;/*I_*/return getStandardColor();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4984]++;/*I_*/return color;/*_I*/}/*I_*/
    }

    public void setColor(Color color) {/*_I*/_prof.prof.cnt[4985]++;/*I_*/
        /*_I*/{_prof.prof.cnt[4986]++;/*I_*/this.color = color;/*_I*/}/*I_*/
    }


    public int getWidth() {/*_I*/_prof.prof.cnt[4987]++;/*I_*/
        /*_I*/{_prof.prof.cnt[4988]++;/*I_*/if(width == DEFAULT_WIDTH)
            /*_I*/{_prof.prof.cnt[4989]++;/*I_*/return NORMAL_WIDTH;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4990]++;/*I_*/return width;/*_I*/}/*I_*/ 
    }

    public Stroke getStroke() {/*_I*/_prof.prof.cnt[4991]++;/*I_*/
        /*_I*/{_prof.prof.cnt[4992]++;/*I_*/return stroke;/*_I*/}/*I_*/
    }
		
    public void setWidth(int width) {/*_I*/_prof.prof.cnt[4993]++;/*I_*/
        /*_I*/{_prof.prof.cnt[4994]++;/*I_*/this.width = width;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4995]++;/*I_*/stroke = new BasicStroke(getWidth(),BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);/*_I*/}/*I_*/ 
    }

    public String getStyle() {/*_I*/_prof.prof.cnt[4996]++;/*I_*/
        /*_I*/{_prof.prof.cnt[4997]++;/*I_*/if(style==null) {
            /*_I*/{_prof.prof.cnt[4998]++;/*I_*/return getStandardStyle();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[4999]++;/*I_*/return style;/*_I*/}/*I_*/
    }

    public void setStyle(String style) {/*_I*/_prof.prof.cnt[5000]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5001]++;/*I_*/this.style = style;/*_I*/}/*I_*/
    }

    public String toString() {/*_I*/_prof.prof.cnt[5002]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5003]++;/*I_*/return "";/*_I*/}/*I_*/
    }

    public void setTarget(MindMapNode target) {/*_I*/_prof.prof.cnt[5004]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5005]++;/*I_*/this.target = target;/*_I*/}/*I_*/ 
    }


    ///////////
    // Private Methods
    /////////
    /** I see no reason to hide the node, the line belongs to, to the public, but... fc. */
    public MindMapNode getTarget() {/*_I*/_prof.prof.cnt[5006]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5007]++;/*I_*/return target;/*_I*/}/*I_*/
    }

    public Object clone() {/*_I*/_prof.prof.cnt[5008]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5009]++;/*I_*/try {
            /*_I*/_prof.prof.cnt[5010]++;/*I_*/LineAdapter link = (LineAdapter) super.clone();
            // color, ...
            /*_I*/{_prof.prof.cnt[5011]++;/*I_*/link.color = (color==null)?null:new Color(color.getRGB());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5012]++;/*I_*/return link;/*_I*/}/*I_*/
        } catch(java.lang.CloneNotSupportedException e) {
            /*_I*/{_prof.prof.cnt[5013]++;/*I_*/return null;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }
    /** As this color is static but used in at least three different 
     * objects (edges, clouds and links), the abstract mechanism was chosen.
     * The derived classes set and get the static instance variable. */
    protected abstract void setStandardColor(Color standardColor);

    /** See @see setStandardColor
     */
    protected abstract Color getStandardColor();
    
    protected abstract void setStandardStyle(String standardStyle);

    protected abstract String getStandardStyle() ;

    protected class LineAdapterListener implements FreemindPropertyListener {
        public void propertyChanged(String propertyName,
                String newValue, String oldValue) {/*_I*/_prof.prof.cnt[5014]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5015]++;/*I_*/if (propertyName.equals(standardColorPropertyString)) {
                /*_I*/{_prof.prof.cnt[5016]++;/*I_*/setStandardColor(Tools.xmlToColor(newValue));/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5017]++;/*I_*/if (propertyName.equals(standardStylePropertyString)) {
                /*_I*/{_prof.prof.cnt[5018]++;/*I_*/setStandardStyle(newValue);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }
    }
    
}
