/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: MapAdapter.java,v 1.24.14.9 2005/07/18 20:46:44 christianfoltin Exp $*/

package freemind.modes;

import java.awt.Color;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import javax.swing.event.EventListenerList;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import freemind.controller.MindMapNodesSelection;
import freemind.extensions.PermanentNodeHook;
import freemind.main.FreeMind;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.main.XMLParseException;

public abstract class MapAdapter implements MindMap {

    private MindMapNode root;
    private EventListenerList treeModelListeners = new EventListenerList();
    /** denotes the amount of changes since the last save. The initial value is one, to ensure, that the model is dirty.*/
    protected int changesPerformedSinceLastSave = 1; 
    protected boolean readOnly = true;
    private Color backgroundColor;
    private File file;
    private FreeMindMain frame;
    static protected Logger logger;



    public MapAdapter (FreeMindMain frame) {/*_I*/_prof.prof.cnt[9808]++;/*I_*/
		/*_I*/{_prof.prof.cnt[9809]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[9810]++;/*I_*/if(logger == null) {
		    /*_I*/{_prof.prof.cnt[9811]++;/*I_*/logger = frame.getLogger(this.getClass().getName());/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
    }

    //
    // Abstract methods that _must_ be implemented.
    //

    public abstract boolean save(File file); 
    
    public abstract void load(File file) throws FileNotFoundException, IOException, XMLParseException ;

	/**
	 * Attempts to lock the map using semaphore file.
	 * @param file
	 * @return If the map is locked, return the name of the locking user, return null
	 * otherwise.
	 * @throws Exception
	 */
	public String tryToLock(File file) throws Exception {/*_I*/_prof.prof.cnt[9812]++;/*I_*/ 
	   /*_I*/{_prof.prof.cnt[9813]++;/*I_*/return null;/*_I*/}/*I_*/
	}

	public void destroy() {/*_I*/_prof.prof.cnt[9814]++;/*I_*/
		// Do all the necessary destructions in your model,
		// e.g. remove file locks.
		// and remove all hooks:
		/*_I*/{_prof.prof.cnt[9815]++;/*I_*/removeHooks((MindMapNode) getRoot());/*_I*/}/*I_*/
	}

    // (PN)
    //    public void close() {
    //    }

	/**
	 * @param node
	 */
	private void removeHooks(MindMapNode node) {/*_I*/_prof.prof.cnt[9816]++;/*I_*/
		/*_I*/{_prof.prof.cnt[9817]++;/*I_*/while(node.getHooks().size()>0) {
			/*_I*/_prof.prof.cnt[9818]++;/*I_*/PermanentNodeHook hook = (PermanentNodeHook) node.getHooks().get(0);
			/*_I*/{_prof.prof.cnt[9819]++;/*I_*/node.removeHook(hook);/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		// and all children:
		/*_I*/{_prof.prof.cnt[9820]++;/*I_*/for(Iterator i= node.childrenUnfolded(); i.hasNext();) {
			/*_I*/_prof.prof.cnt[9821]++;/*I_*/MindMapNode child = (MindMapNode) i.next();
		    /*_I*/{_prof.prof.cnt[9822]++;/*I_*/removeHooks(child);/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}

	public FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[9823]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9824]++;/*I_*/return frame;/*_I*/}/*I_*/
    }

    //
    // Attributes
    //

    public boolean isSaved() {/*_I*/_prof.prof.cnt[9825]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9826]++;/*I_*/return (changesPerformedSinceLastSave==0);/*_I*/}/*I_*/
    }

	public boolean isReadOnly() {/*_I*/_prof.prof.cnt[9827]++;/*I_*/
		/*_I*/{_prof.prof.cnt[9828]++;/*I_*/return readOnly;/*_I*/}/*I_*/ 
	}

    /** Counts the amount of actions performed. 
     * @param saved true if the file was saved recently. False otherwise.
     */
    protected void setSaved(boolean saved) {/*_I*/_prof.prof.cnt[9829]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9830]++;/*I_*/if(saved)
            /*_I*/{_prof.prof.cnt[9831]++;/*I_*/changesPerformedSinceLastSave = 0;/*_I*/}/*I_*/
        else
            /*_I*/{_prof.prof.cnt[9832]++;/*I_*/++changesPerformedSinceLastSave;/*_I*/}/*I_*//*_I*/}/*I_*/
    }

    protected int getNumberOfChangesSinceLastSave() {/*_I*/_prof.prof.cnt[9833]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9834]++;/*I_*/return changesPerformedSinceLastSave;/*_I*/}/*I_*/
    }

    public Color getBackgroundColor() {/*_I*/_prof.prof.cnt[9835]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9836]++;/*I_*/if (backgroundColor==null) {
	    /*_I*/{_prof.prof.cnt[9837]++;/*I_*/return Tools.xmlToColor(getFrame().getProperty(FreeMind.RESOURCES_BACKGROUND_COLOR));/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[9838]++;/*I_*/return backgroundColor;/*_I*/}/*I_*/
    }

    public void setBackgroundColor(Color backgroundColor) {/*_I*/_prof.prof.cnt[9839]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9840]++;/*I_*/this.backgroundColor = backgroundColor;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[9841]++;/*I_*/nodeChanged((MindMapNode)getRoot());/*_I*/}/*I_*///update view anyhow
    }

    public Object getRoot() {/*_I*/_prof.prof.cnt[9842]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9843]++;/*I_*/return root;/*_I*/}/*I_*/
    }

    protected void setRoot(MindMapNode root) {/*_I*/_prof.prof.cnt[9844]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9845]++;/*I_*/this.root = root;/*_I*/}/*I_*/
    }

    /**
     * Change this to always return null if your model doesn't support files.
     */
    public File getFile() {/*_I*/_prof.prof.cnt[9846]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9847]++;/*I_*/return file;/*_I*/}/*I_*/
    }

    /**
     * Return URL of the map (whether as local file or a web location)
     */
    public URL getURL() throws MalformedURLException {/*_I*/_prof.prof.cnt[9848]++;/*I_*/
       /*_I*/{_prof.prof.cnt[9849]++;/*I_*/return getFile() != null ? getFile().toURL() : null;/*_I*/}/*I_*/
    }


    protected void setFile(File file) {/*_I*/_prof.prof.cnt[9850]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9851]++;/*I_*/this.file = file;/*_I*/}/*I_*/
    }

    protected String getText(String textId) {/*_I*/_prof.prof.cnt[9852]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9853]++;/*I_*/return getFrame().getResourceString(textId);/*_I*/}/*I_*/ 
    }
    //
    // Node editing
    //

 
    public Object[] getPathToRoot( TreeNode node ) {/*_I*/_prof.prof.cnt[9854]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9855]++;/*I_*/return ( ((MindMapNode)node).getPath() ).getPath();/*_I*/}/*I_*///Create Object[] from TreePath
    }

    //
    // cut'n'paste
    //
    public final Transferable cut(MindMapNode node) {/*_I*/_prof.prof.cnt[9856]++;/*I_*/
       // This cut does not need any reimplementation in child Classes!
       /*_I*/_prof.prof.cnt[9857]++;/*I_*/Transferable t = copy(node);
       /*_I*/{_prof.prof.cnt[9858]++;/*I_*/removeNodeFromParent(node);/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[9859]++;/*I_*/return t;/*_I*/}/*I_*/
    }


   public Transferable copy(MindMapNode node) {/*_I*/_prof.prof.cnt[9860]++;/*I_*/
     /*_I*/{_prof.prof.cnt[9861]++;/*I_*/return null;/*_I*/}/*I_*/
   }

   public Transferable copy() {/*_I*/_prof.prof.cnt[9862]++;/*I_*/
      /*_I*/{_prof.prof.cnt[9863]++;/*I_*/return copy(getFrame().getView().getSelectedNodesSortedByY(), null);/*_I*/}/*I_*/ }

   public Transferable copySingle() {/*_I*/_prof.prof.cnt[9864]++;/*I_*/
      /*_I*/_prof.prof.cnt[9865]++;/*I_*/MindMapNode selectedNode = (MindMapNode)getFrame().getView().getSelected().getModel();
      /*_I*/_prof.prof.cnt[9866]++;/*I_*/ArrayList selectedNodes  = new ArrayList();
      /*_I*/{_prof.prof.cnt[9867]++;/*I_*/selectedNodes.add(selectedNode.shallowCopy());/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[9868]++;/*I_*/return copy(selectedNodes, selectedNode.toString());/*_I*/}/*I_*/ }

   public Transferable copy(List selectedNodes, String inPlainText) {/*_I*/_prof.prof.cnt[9869]++;/*I_*/
      /*_I*/{_prof.prof.cnt[9870]++;/*I_*/try {
         /*_I*/_prof.prof.cnt[9871]++;/*I_*/String forNodesFlavor = "";
         /*_I*/_prof.prof.cnt[9872]++;/*I_*/boolean firstLoop = true;
         /*_I*/{_prof.prof.cnt[9873]++;/*I_*/for(Iterator it = selectedNodes.iterator();it.hasNext();) {
            /*_I*/_prof.prof.cnt[9874]++;/*I_*/MindMapNode tmpNode =  (MindMapNode)it.next();
            /*_I*/{_prof.prof.cnt[9875]++;/*I_*/if (firstLoop) {
               /*_I*/{_prof.prof.cnt[9876]++;/*I_*/firstLoop = false;/*_I*/}/*I_*/ }
            else {
               /*_I*/{_prof.prof.cnt[9877]++;/*I_*/forNodesFlavor += "<nodeseparator>";/*_I*/}/*I_*/ }/*_I*/}/*I_*/

            /*_I*/{_prof.prof.cnt[9878]++;/*I_*/forNodesFlavor += copy(tmpNode).getTransferData(MindMapNodesSelection.mindMapNodesFlavor);/*_I*/}/*I_*/ 
         }/*_I*/}/*I_*/

         /*_I*/_prof.prof.cnt[9879]++;/*I_*/String plainText = inPlainText != null ? inPlainText : getAsPlainText(selectedNodes);
         /*_I*/{_prof.prof.cnt[9880]++;/*I_*/return new MindMapNodesSelection
            (forNodesFlavor, plainText, getAsRTF(selectedNodes), null, null, null);/*_I*/}/*I_*/ }
         //return new StringSelection(forClipboard); }


      catch (UnsupportedFlavorException ex) { /*_I*/{_prof.prof.cnt[9881]++;/*I_*/ex.printStackTrace();/*_I*/}/*I_*/ }
      catch (IOException ex) { /*_I*/{_prof.prof.cnt[9882]++;/*I_*/ex.printStackTrace();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[9883]++;/*I_*/return null;/*_I*/}/*I_*/ }

    public String getAsPlainText(List mindMapNodes) {/*_I*/_prof.prof.cnt[9884]++;/*I_*/
       /*_I*/{_prof.prof.cnt[9885]++;/*I_*/return "";/*_I*/}/*I_*/ }
 

    public String getAsRTF(List mindMapNodes) {/*_I*/_prof.prof.cnt[9886]++;/*I_*/
       /*_I*/{_prof.prof.cnt[9887]++;/*I_*/return "";/*_I*/}/*I_*/ }


    public String getRestoreable() {/*_I*/_prof.prof.cnt[9888]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9889]++;/*I_*/return null;/*_I*/}/*I_*/
    }

    public MindMapLinkRegistry getLinkRegistry() {/*_I*/_prof.prof.cnt[9890]++;/*I_*/ /*_I*/{_prof.prof.cnt[9891]++;/*I_*/return null;/*_I*/}/*I_*/ }


    public void insertNodeIntoNoEvent(MindMapNode newChild, MindMapNode parent) {/*_I*/_prof.prof.cnt[9892]++;/*I_*/
       /*_I*/{_prof.prof.cnt[9893]++;/*I_*/insertNodeIntoNoEvent(newChild, parent, false);/*_I*/}/*I_*/ }

    public void insertNodeIntoNoEvent(MindMapNode newChild, MindMapNode parent,
                                      boolean asSibling){/*_I*/_prof.prof.cnt[9894]++;/*I_*/
        // Daniel: This is a hackery in the sense that I don't
        // understand the method "nodesWereInserted".

        // This code is based on empirical observation that in
        // some cases avoiding the mentioned method does not throw
        // exception.

        // The reason why we do such a thing at all is that calling the
        // method "nodesWereInserted" dramatically lowers the performance.

        // The default position is after the last node
        /*_I*/{_prof.prof.cnt[9895]++;/*I_*/if (asSibling) {
            /*_I*/_prof.prof.cnt[9896]++;/*I_*/MindMapNode oldpa = parent.getParentNode();
            /*_I*/{_prof.prof.cnt[9897]++;/*I_*/oldpa.insert(newChild, oldpa.getChildPosition(parent));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[9898]++;/*I_*/recursiveCallAddChildren(oldpa, newChild);/*_I*/}/*I_*/
        } 
        else {
            /*_I*/{_prof.prof.cnt[9899]++;/*I_*/parent.insert(newChild, parent.getChildCount());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[9900]++;/*I_*/recursiveCallAddChildren(parent, newChild);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        
    }

    public void insertNodeInto(MindMapNode newChild,
			       MindMapNode parent){/*_I*/_prof.prof.cnt[9901]++;/*I_*/
        // The default position is after the last node
        /*_I*/{_prof.prof.cnt[9902]++;/*I_*/insertNodeInto(newChild, parent, parent.getChildCount());/*_I*/}/*I_*/
    }

	public void insertNodeInto(
		MindMapNode newChild,
		MindMapNode parent,
		int index) {/*_I*/_prof.prof.cnt[9903]++;/*I_*/
		/*_I*/{_prof.prof.cnt[9904]++;/*I_*/insertNodeInto((MutableTreeNode) newChild, (MutableTreeNode) parent, index);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[9905]++;/*I_*/recursiveCallAddChildren(parent, newChild);/*_I*/}/*I_*/
	}

    /**
     * Use this method to add children because it will cause the appropriate event.
     */
    public void insertNodeInto(MutableTreeNode newChild,
			       MutableTreeNode parent, int index){/*_I*/_prof.prof.cnt[9906]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9907]++;/*I_*/parent.insert(newChild, index);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[9908]++;/*I_*/nodesWereInserted(parent, new int[]{ index });/*_I*/}/*I_*/
        // ^ Daniel: I don't understand the reason why this is here.  But it
        // is necessary, because user's action "New Node" throws
        // an exception otherwise.
    }

	/**URGENT: This method must be moved to the ControllerAdapter.
	 * @param node
	 */
	private void recursiveCallAddChildren(MindMapNode node, MindMapNode addedChild) {/*_I*/_prof.prof.cnt[9909]++;/*I_*/
		// Tell any node hooks that the node is added:
		/*_I*/{_prof.prof.cnt[9910]++;/*I_*/if(node instanceof MindMapNode) {
			/*_I*/{_prof.prof.cnt[9911]++;/*I_*/for(Iterator i=  ((MindMapNode)node).getActivatedHooks().iterator(); i.hasNext();) {
				/*_I*/_prof.prof.cnt[9912]++;/*I_*/PermanentNodeHook hook = (PermanentNodeHook) i.next();
                /*_I*/{_prof.prof.cnt[9913]++;/*I_*/if (addedChild.getParentNode() == node) {
                    /*_I*/{_prof.prof.cnt[9914]++;/*I_*/hook.onAddChild(addedChild);/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[9915]++;/*I_*/hook.onAddChildren(addedChild);/*_I*/}/*I_*/
			}/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[9916]++;/*I_*/if(!node.isRoot() && node.getParentNode()!= null)
		    /*_I*/{_prof.prof.cnt[9917]++;/*I_*/recursiveCallAddChildren(node.getParentNode(), addedChild);/*_I*/}/*I_*//*_I*/}/*I_*/
	}

    
    /**
     * Joerg: Message this to remove node from its parent. This will message
     * nodesWereRemoved to create the appropriate event. This is the
     * preferred way to remove a node as it handles the event creation
     * for you.
     */
    public void removeNodeFromParent(MutableTreeNode node) {/*_I*/_prof.prof.cnt[9918]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9919]++;/*I_*/removeNodeFromParent(node, /*notify=*/ true);/*_I*/}/*I_*/ }

    public void removeNodeFromParent(MutableTreeNode node, boolean notify) {/*_I*/_prof.prof.cnt[9920]++;/*I_*/
	/*_I*/_prof.prof.cnt[9921]++;/*I_*/MutableTreeNode parent = (MutableTreeNode)node.getParent();
	    
	/*_I*/{_prof.prof.cnt[9922]++;/*I_*/if (parent == null)
           /*_I*/{_prof.prof.cnt[9923]++;/*I_*/throw new IllegalArgumentException("node does not have a parent.");/*_I*/}/*I_*//*_I*/}/*I_*/

	/*_I*/_prof.prof.cnt[9924]++;/*I_*/int[] childIndex = new int[1];
	/*_I*/_prof.prof.cnt[9925]++;/*I_*/Object[] removedArray = new Object[1];
	    
	/*_I*/{_prof.prof.cnt[9926]++;/*I_*/childIndex[0] = parent.getIndex(node);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[9927]++;/*I_*/parent.remove(node);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9928]++;/*I_*/if (notify) {
           /*_I*/{_prof.prof.cnt[9929]++;/*I_*/removedArray[0] = node;/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[9930]++;/*I_*/nodesWereRemoved(parent, childIndex, removedArray);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

//    public void changeNode(MindMapNode node, String newText) {
//	node.setUserObject(newText);
//	nodeChanged(node);
//    }

    //
    // Interface TreeModel
    //

    public Object getChild( Object parent, int index ) {/*_I*/_prof.prof.cnt[9931]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9932]++;/*I_*/return ( (TreeNode)parent ).getChildAt( index );/*_I*/}/*I_*/
    }

    public int getChildCount( Object parent ) {/*_I*/_prof.prof.cnt[9933]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9934]++;/*I_*/return ( (TreeNode)parent ).getChildCount();/*_I*/}/*I_*/
    }

    public int getIndexOfChild( Object parent, Object child ) {/*_I*/_prof.prof.cnt[9935]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9936]++;/*I_*/return ( (TreeNode)parent ).getIndex( (TreeNode)child );/*_I*/}/*I_*/
    }
	
    public boolean isLeaf( Object node ) {/*_I*/_prof.prof.cnt[9937]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9938]++;/*I_*/return ( (TreeNode)node ).isLeaf();/*_I*/}/*I_*/
    }

    public void addTreeModelListener( TreeModelListener l ) {/*_I*/_prof.prof.cnt[9939]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9940]++;/*I_*/treeModelListeners.add( TreeModelListener.class, l );/*_I*/}/*I_*/
    }

    public void removeTreeModelListener( TreeModelListener l ) {/*_I*/_prof.prof.cnt[9941]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9942]++;/*I_*/treeModelListeners.remove( TreeModelListener.class, l );/*_I*/}/*I_*/
    }

    public void valueForPathChanged( TreePath path, Object newValue ) {/*_I*/_prof.prof.cnt[9943]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9944]++;/*I_*/( (MutableTreeNode)path.getLastPathComponent() ).setUserObject( newValue );/*_I*/}/*I_*/
    }


    //
    // API for updating the view.
    //

    //
    // TreeNodesRemoved
    //

    /**
     * Invoke this method after you've removed some TreeNodes from
     * node.  childIndices should be the index of the removed elements and
     * must be sorted in ascending order. And removedChildren should be
     * the array of the children objects that were removed.
     */
    protected void nodesWereRemoved(TreeNode parent, int[] childIndices,
				 Object[] removedChildren) {/*_I*/_prof.prof.cnt[9945]++;/*I_*/
	/*_I*/{_prof.prof.cnt[9946]++;/*I_*/setSaved(false);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[9947]++;/*I_*/if(parent != null && childIndices != null) {
	    /*_I*/{_prof.prof.cnt[9948]++;/*I_*/fireTreeNodesRemoved(this, getPathToRoot( (MindMapNode)parent ), childIndices, 
				 removedChildren);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    //
    // TreeNodesInserted
    //

    /**
     * Invoke this method after you've inserted some TreeNodes into
     * node.  childIndices should be the index of the new elements and
     * must be sorted in ascending order.
     */
    protected void nodesWereInserted(TreeNode node, int[] childIndices) {/*_I*/_prof.prof.cnt[9949]++;/*I_*/
       /*_I*/{_prof.prof.cnt[9950]++;/*I_*/if (treeModelListeners != null && node != null && 
           childIndices != null && childIndices.length > 0) {
          /*_I*/{_prof.prof.cnt[9951]++;/*I_*/setSaved(false);/*_I*/}/*I_*/

	  /*_I*/_prof.prof.cnt[9952]++;/*I_*/Object[] newChildren = new Object[childIndices.length];	    
          /*_I*/{_prof.prof.cnt[9953]++;/*I_*/for(int counter = 0; counter < childIndices.length; counter++) {
             /*_I*/{_prof.prof.cnt[9954]++;/*I_*/newChildren[counter] = node.getChildAt(childIndices[counter]);/*_I*/}/*I_*/ }/*_I*/}/*I_*/

          /*_I*/{_prof.prof.cnt[9955]++;/*I_*/fireTreeNodesInserted(this, getPathToRoot( (MindMapNode)node ), childIndices, newChildren);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }
    
    //
    // TreeNodesChanged
    //

    /**
      * This method should not be called directly!
      */
    public void nodeChanged(TreeNode node) {/*_I*/_prof.prof.cnt[9956]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9957]++;/*I_*/frame.getController().getMode().getModeController().nodeChanged((MindMapNode)node);/*_I*/}/*I_*/
    }

	public void nodeChangedMapInternal(TreeNode node) {/*_I*/_prof.prof.cnt[9958]++;/*I_*/
		/*_I*/{_prof.prof.cnt[9959]++;/*I_*/if (treeModelListeners != null && node != null) {
			/*_I*/_prof.prof.cnt[9960]++;/*I_*/TreeNode parent = node.getParent();

			/*_I*/{_prof.prof.cnt[9961]++;/*I_*/if (parent != null) {
				/*_I*/_prof.prof.cnt[9962]++;/*I_*/int anIndex = parent.getIndex(node);
				/*_I*/{_prof.prof.cnt[9963]++;/*I_*/if (anIndex != -1) {
					/*_I*/_prof.prof.cnt[9964]++;/*I_*/int[] cIndexs = new int[1];

					/*_I*/{_prof.prof.cnt[9965]++;/*I_*/cIndexs[0] = anIndex;/*_I*/}/*I_*/
					/*_I*/{_prof.prof.cnt[9966]++;/*I_*/nodesChanged(parent, cIndexs);/*_I*/}/*I_*/
				}/*_I*/}/*I_*/
			}
			else /*_I*/{_prof.prof.cnt[9967]++;/*I_*/if (((MindMapNode)node).isRoot()) {
				/*_I*/{_prof.prof.cnt[9968]++;/*I_*/nodesChanged(node, null);/*_I*/}/*I_*/
			}/*_I*/}/*I_*//*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}


	/**
      * Invoke this method after you've changed how the children identified by
      * childIndicies are to be represented in the tree.
      */
    protected void nodesChanged(TreeNode node, int[] childIndices) {/*_I*/_prof.prof.cnt[9969]++;/*I_*/

       /*_I*/{_prof.prof.cnt[9970]++;/*I_*/if (node != null) {
          /*_I*/{_prof.prof.cnt[9971]++;/*I_*/if (childIndices != null) {
             /*_I*/_prof.prof.cnt[9972]++;/*I_*/int cCount = childIndices.length;
             
             /*_I*/{_prof.prof.cnt[9973]++;/*I_*/if (cCount > 0) {
                /*_I*/_prof.prof.cnt[9974]++;/*I_*/Object[] cChildren = new Object[cCount];
                
                /*_I*/{_prof.prof.cnt[9975]++;/*I_*/for(int counter = 0; counter < cCount; counter++) {
                   /*_I*/{_prof.prof.cnt[9976]++;/*I_*/cChildren[counter] = node.getChildAt
                      (childIndices[counter]);/*_I*/}/*I_*/
                   /*_I*/{_prof.prof.cnt[9977]++;/*I_*/logger.finest("Children array at "+counter+" = "+ cChildren[counter]);/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[9978]++;/*I_*/fireTreeNodesChanged(this, getPathToRoot(node), childIndices, cChildren);/*_I*/}/*I_*/
             }/*_I*/}/*I_*/
          }
          else /*_I*/{_prof.prof.cnt[9979]++;/*I_*/if (((MindMapNode)node).isRoot()) {
             /*_I*/{_prof.prof.cnt[9980]++;/*I_*/fireTreeNodesChanged(this, getPathToRoot(node), null, null);/*_I*/}/*I_*/
          }/*_I*/}/*I_*//*_I*/}/*I_*/
       }/*_I*/}/*I_*/
    }

    //
    // TreeStructureChanged
    //    

    /**
      * Invoke this method if you've totally changed the children of
      * node and its childrens children...  This will post a
      * treeStructureChanged event.
      */
    protected void nodeStructureChanged(TreeNode node) {/*_I*/_prof.prof.cnt[9981]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9982]++;/*I_*/setSaved(false);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[9983]++;/*I_*/if (node != null) {
           /*_I*/{_prof.prof.cnt[9984]++;/*I_*/fireTreeStructureChanged(this, getPathToRoot(node), null, null);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    /**
     * Invoke this method if you've modified the TreeNodes upon which this
     * model depends.  The model will notify all of its listeners that the
     * model has changed below the node <code>node</code> (PENDING).
     */
    protected void reload(TreeNode node) {/*_I*/_prof.prof.cnt[9985]++;/*I_*/
        /*_I*/{_prof.prof.cnt[9986]++;/*I_*/if (node != null) {
           /*_I*/{_prof.prof.cnt[9987]++;/*I_*/fireTreeStructureChanged(this, getPathToRoot(node), null, null);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}


    /////////
    // private methods. Internal implementation.
    ////////

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    private void fireTreeNodesRemoved(Object source, Object[] path, 
					int[] childIndices, 
					Object[] children) {/*_I*/_prof.prof.cnt[9988]++;/*I_*/
	// Guaranteed to return a non-null array
	/*_I*/_prof.prof.cnt[9989]++;/*I_*/Object[] listeners = treeModelListeners.getListenerList();
	/*_I*/_prof.prof.cnt[9990]++;/*I_*/TreeModelEvent e = null;
	// Process the listeners last to first, notifying
	// those that are interested in this event
	/*_I*/{_prof.prof.cnt[9991]++;/*I_*/for (int i = listeners.length-2; i>=0; i-=2) {
	    /*_I*/{_prof.prof.cnt[9992]++;/*I_*/if (listeners[i]==TreeModelListener.class) {
		// Lazily create the event:
		/*_I*/{_prof.prof.cnt[9993]++;/*I_*/if (e == null)
		    /*_I*/{_prof.prof.cnt[9994]++;/*I_*/e = new TreeModelEvent(source, path, 
					   childIndices, children);/*_I*/}/*I_*//*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[9995]++;/*I_*/((TreeModelListener)listeners[i+1]).treeNodesRemoved(e);/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/          
	}/*_I*/}/*I_*/
    }

    /**
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    private void fireTreeNodesInserted(Object source, Object[] path, 
					 int[] childIndices, 
					 Object[] children) {/*_I*/_prof.prof.cnt[9996]++;/*I_*/
	// Guaranteed to return a non-null array
	/*_I*/_prof.prof.cnt[9997]++;/*I_*/Object[] listeners = treeModelListeners.getListenerList();
	/*_I*/_prof.prof.cnt[9998]++;/*I_*/TreeModelEvent e = null;
	// Process the listeners last to first, notifying
	// those that are interested in this event
	/*_I*/{_prof.prof.cnt[9999]++;/*I_*/for (int i = listeners.length-2; i>=0; i-=2) {
            /*_I*/{_prof.prof.cnt[10000]++;/*I_*/if (listeners[i]==TreeModelListener.class) {
                // Lazily create the event:
                /*_I*/{_prof.prof.cnt[10001]++;/*I_*/if (e == null)
                    /*_I*/{_prof.prof.cnt[10002]++;/*I_*/e = new TreeModelEvent(source, path, 
                                           childIndices, children);/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[10003]++;/*I_*/((TreeModelListener)listeners[i+1]).treeNodesInserted(e);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/          
        }/*_I*/}/*I_*/
    }

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    private void fireTreeNodesChanged(Object source, Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {/*_I*/_prof.prof.cnt[10004]++;/*I_*/
        // Guaranteed to return a non-null array
        /*_I*/_prof.prof.cnt[10005]++;/*I_*/Object[] listeners = treeModelListeners.getListenerList();
        /*_I*/_prof.prof.cnt[10006]++;/*I_*/TreeModelEvent e = null;
        // Process the listeners last to first, notifying
        // those that are interested in this event
        /*_I*/{_prof.prof.cnt[10007]++;/*I_*/for (int i = listeners.length-2; i>=0; i-=2) {
            /*_I*/{_prof.prof.cnt[10008]++;/*I_*/if (listeners[i]==TreeModelListener.class) {
                // Lazily create the event:
                /*_I*/{_prof.prof.cnt[10009]++;/*I_*/if (e == null)
                    /*_I*/{_prof.prof.cnt[10010]++;/*I_*/e = new TreeModelEvent(source, path, 
                                           childIndices, children);/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[10011]++;/*I_*/((TreeModelListener)listeners[i+1]).treeNodesChanged(e);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/          
        }/*_I*/}/*I_*/
    }

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    private void fireTreeStructureChanged(Object source, Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {/*_I*/_prof.prof.cnt[10012]++;/*I_*/
        // Guaranteed to return a non-null array
        /*_I*/_prof.prof.cnt[10013]++;/*I_*/Object[] listeners = treeModelListeners.getListenerList();
        /*_I*/_prof.prof.cnt[10014]++;/*I_*/TreeModelEvent e = null;
        // Process the listeners last to first, notifying
        // those that are interested in this event
        /*_I*/{_prof.prof.cnt[10015]++;/*I_*/for (int i = listeners.length-2; i>=0; i-=2) {
            /*_I*/{_prof.prof.cnt[10016]++;/*I_*/if (listeners[i]==TreeModelListener.class) {
                // Lazily create the event:
                /*_I*/{_prof.prof.cnt[10017]++;/*I_*/if (e == null)
                    /*_I*/{_prof.prof.cnt[10018]++;/*I_*/e = new TreeModelEvent(source, path, childIndices, children);/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[10019]++;/*I_*/((TreeModelListener)listeners[i+1]).treeStructureChanged(e);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/          
        }/*_I*/}/*I_*/
    }

}

