/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: NodeAdapter.java,v 1.20.16.19 2005/07/26 20:52:34 christianfoltin Exp $*/

package freemind.modes;

import java.awt.Color;
import java.awt.Font;
import java.io.IOException;
import java.io.Writer;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import freemind.extensions.NodeHook;
import freemind.extensions.PermanentNodeHook;
import freemind.main.FreeMind;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.main.XMLElement;
import freemind.view.mindmapview.NodeView;

/**
 * This class represents a single Node of a Tree. It contains direct handles 
 * to its parent and children and to its view.
 */
public abstract class NodeAdapter implements MindMapNode {
	
    final static int SHIFT = -2;//height of the vertical shift between node and its closest child
    public final static int HGAP = 20;//width of the horizontal gap that contains the edges
    public final static int VGAP = 3;//height of the vertical gap between nodes

    private HashSet activatedHooks;
	private List hooks;
	protected Object userObject = "no text";
    private String link = null; //Change this to vector in future for full graph support
    private HashMap toolTip = null; // lazy, fc, 30.6.2005

    //these Attributes have default values, so it can be useful to directly access them in
    //the save() method instead of using getXXX(). This way the stored file is smaller and looks better.
    //(if the default is used, it is not stored) Look at mindmapmode for an example.
    protected String style;
    /**stores the icons associated with this node.*/
    protected Vector/*<MindIcon>*/ icons = null; // lazy, fc, 30.6.2005
    
    protected TreeMap /* of String to MindIcon s*/ stateIcons = null; // lazy, fc, 30.6.2005
//     /**stores the label associated with this node:*/
//     protected String mLabel;
    /** parameters of an eventually associated cloud*/
    protected MindMapCloud cloud;

    protected Color color;
    protected Color backgroundColor;
    protected boolean folded;
    private Tools.BooleanHolder left;

	private int vGap = AUTO;
	private int hGap = HGAP;
    private int shiftY = 0;

    protected List children;
    private MindMapNode preferredChild; 

    protected Font font;
    protected boolean underlined = false;

    private MindMapNode parent;
    private MindMapEdge edge;//the edge which leads to this node, only root has none
    //In future it has to hold more than one view, maybe with a Vector in which the index specifies
    //the MapView which contains the NodeViews
    private NodeView viewer = null;
    private FreeMindMain frame;
    private static final boolean ALLOWSCHILDREN = true;
    private static final boolean ISLEAF = false; //all nodes may have children
    
    private HistoryInformation historyInformation = null;
	// Logging: 
    static protected java.util.logging.Logger logger;


    //
    // Constructors
    //

    protected NodeAdapter(FreeMindMain frame) {
		this(null, frame);/*_I*/_prof.prof.cnt[10337]++;/*I_*//*_I*/_prof.prof.cnt[10338]++;/*I_*/
    }

    protected NodeAdapter(Object userObject, FreeMindMain frame) {/*_I*/_prof.prof.cnt[10339]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10340]++;/*I_*/this.userObject = userObject;/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10341]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10342]++;/*I_*/hooks = null;/*_I*/}/*I_*/ // lazy, fc, 30.6.2005.
		/*_I*/{_prof.prof.cnt[10343]++;/*I_*/activatedHooks = null;/*_I*/}/*I_*/ //lazy, fc, 30.6.2005
		/*_I*/{_prof.prof.cnt[10344]++;/*I_*/if(logger == null)
			/*_I*/{_prof.prof.cnt[10345]++;/*I_*/logger = frame.getLogger(this.getClass().getName());/*_I*/}/*I_*//*_I*/}/*I_*/
		// create creation time:
		/*_I*/{_prof.prof.cnt[10346]++;/*I_*/setHistoryInformation(new HistoryInformation());/*_I*/}/*I_*/

    }

    /**
     *
     */

    public String getText() {/*_I*/_prof.prof.cnt[10347]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10348]++;/*I_*/return toString();/*_I*/}/*I_*/
    }
    /**
     *
     */

    public void setText(String text) {/*_I*/_prof.prof.cnt[10349]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10350]++;/*I_*/setUserObject(text);/*_I*/}/*I_*/
    }

    public String getLink() {/*_I*/_prof.prof.cnt[10351]++;/*I_*/
 	/*_I*/{_prof.prof.cnt[10352]++;/*I_*/return link;/*_I*/}/*I_*/
    }
    
    public String getShortText(ModeController controller) {/*_I*/_prof.prof.cnt[10353]++;/*I_*/
	    /*_I*/_prof.prof.cnt[10354]++;/*I_*/String adaptedText = toString();
        /*_I*/{_prof.prof.cnt[10355]++;/*I_*/adaptedText = adaptedText.replaceAll("<html>", "");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10356]++;/*I_*/if (adaptedText.length() > 40)
            /*_I*/{_prof.prof.cnt[10357]++;/*I_*/adaptedText = adaptedText.substring(0, 40) + " ...";/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10358]++;/*I_*/return adaptedText;/*_I*/}/*I_*/
    }
    
    public void setLink(String link) {/*_I*/_prof.prof.cnt[10359]++;/*I_*/
 	/*_I*/{_prof.prof.cnt[10360]++;/*I_*/this.link = link;/*_I*/}/*I_*/
    }

    public FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[10361]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10362]++;/*I_*/return frame;/*_I*/}/*I_*/
    }

    //
    // Interface MindMapNode
    //

    //
    // get/set methods
    //

    public NodeView getViewer() {/*_I*/_prof.prof.cnt[10363]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10364]++;/*I_*/return viewer;/*_I*/}/*I_*/
    }

    public void setViewer( NodeView viewer ) {/*_I*/_prof.prof.cnt[10365]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10366]++;/*I_*/this.viewer = viewer;/*_I*/}/*I_*/
    }

    /** Creates the TreePath recursively */
    public TreePath getPath() {/*_I*/_prof.prof.cnt[10367]++;/*I_*/
	/*_I*/_prof.prof.cnt[10368]++;/*I_*/Vector pathVector = new Vector();
	/*_I*/_prof.prof.cnt[10369]++;/*I_*/TreePath treePath;
	/*_I*/{_prof.prof.cnt[10370]++;/*I_*/this.addToPathVector( pathVector );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[10371]++;/*I_*/treePath = new TreePath( pathVector.toArray() );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[10372]++;/*I_*/return treePath;/*_I*/}/*I_*/
    }

    public MindMapEdge getEdge() {/*_I*/_prof.prof.cnt[10373]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10374]++;/*I_*/return edge;/*_I*/}/*I_*/
    }

    public void setEdge( MindMapEdge edge ) {/*_I*/_prof.prof.cnt[10375]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10376]++;/*I_*/this.edge = edge;/*_I*/}/*I_*/
    }

    public MindMapCloud getCloud() {/*_I*/_prof.prof.cnt[10377]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10378]++;/*I_*/return cloud;/*_I*/}/*I_*/
    }

    public void setCloud( MindMapCloud cloud ) {/*_I*/_prof.prof.cnt[10379]++;/*I_*/
    	// Take care to keep the calculated iterative levels consistent
		/*_I*/{_prof.prof.cnt[10380]++;/*I_*/if (cloud != null && this.cloud == null) {
			/*_I*/{_prof.prof.cnt[10381]++;/*I_*/changeChildCloudIterativeLevels(1);/*_I*/}/*I_*/
		}
		else /*_I*/{_prof.prof.cnt[10382]++;/*I_*/if (cloud == null && this.cloud != null) {
			/*_I*/{_prof.prof.cnt[10383]++;/*I_*/changeChildCloudIterativeLevels(-1);/*_I*/}/*I_*/
		}/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10384]++;/*I_*/this.cloud = cloud;/*_I*/}/*I_*/
    }

	/**
	*  Correct iterative level values of children
	*/
	private void changeChildCloudIterativeLevels(int deltaLevel) {/*_I*/_prof.prof.cnt[10385]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10386]++;/*I_*/for (ListIterator e = childrenUnfolded(); e.hasNext(); ) {
			/*_I*/_prof.prof.cnt[10387]++;/*I_*/NodeAdapter childNode = (NodeAdapter)e.next();
			/*_I*/_prof.prof.cnt[10388]++;/*I_*/MindMapCloud childCloud = childNode.getCloud();
			/*_I*/{_prof.prof.cnt[10389]++;/*I_*/if (childCloud != null) {
				/*_I*/{_prof.prof.cnt[10390]++;/*I_*/childCloud.changeIterativeLevel(deltaLevel);/*_I*/}/*I_*/
			}/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[10391]++;/*I_*/childNode.changeChildCloudIterativeLevels(deltaLevel);/*_I*/}/*I_*/ 
		}/*_I*/}/*I_*/
	}

    /**A Node-Style like MindMapNode.STYLE_FORK or MindMapNode.STYLE_BUBBLE*/
	public String getStyle() {/*_I*/_prof.prof.cnt[10392]++;/*I_*/
    	/*_I*/_prof.prof.cnt[10393]++;/*I_*/String returnedString = style; /* Style string returned */
    	/*_I*/{_prof.prof.cnt[10394]++;/*I_*/if (style==null) {
    		/*_I*/{_prof.prof.cnt[10395]++;/*I_*/if (this.isRoot()) {
    			/*_I*/{_prof.prof.cnt[10396]++;/*I_*/returnedString=getFrame().getProperty(FreeMind.RESOURCES_ROOT_NODE_STYLE);/*_I*/}/*I_*/
    		}
    		else{ 
    			/*_I*/_prof.prof.cnt[10397]++;/*I_*/String stdstyle = getFrame().getProperty(FreeMind.RESOURCES_NODE_STYLE);
    			/*_I*/{_prof.prof.cnt[10398]++;/*I_*/if( stdstyle.equals(MindMapNode.STYLE_AS_PARENT)){
    				/*_I*/{_prof.prof.cnt[10399]++;/*I_*/returnedString=getParentNode().getStyle();/*_I*/}/*I_*/
    			}
    			else{
    				/*_I*/{_prof.prof.cnt[10400]++;/*I_*/returnedString = stdstyle;/*_I*/}/*I_*/
    			}/*_I*/}/*I_*/
    		}/*_I*/}/*I_*/
    	}
    	else /*_I*/{_prof.prof.cnt[10401]++;/*I_*/if (this.isRoot() && style.equals(MindMapNode.STYLE_AS_PARENT)) {
    		/*_I*/{_prof.prof.cnt[10402]++;/*I_*/returnedString =  getFrame().getProperty(FreeMind.RESOURCES_ROOT_NODE_STYLE);/*_I*/}/*I_*/
     	}
     	else /*_I*/{_prof.prof.cnt[10403]++;/*I_*/if( style.equals(MindMapNode.STYLE_AS_PARENT)){
    		/*_I*/{_prof.prof.cnt[10404]++;/*I_*/returnedString = getParentNode().getStyle();/*_I*/}/*I_*/
     	}/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
    	
    	// Handle the combined node style
     	/*_I*/{_prof.prof.cnt[10405]++;/*I_*/if (returnedString.equals(MindMapNode.STYLE_COMBINED))
     	{
     		/*_I*/{_prof.prof.cnt[10406]++;/*I_*/if (this.isFolded()){
     			/*_I*/{_prof.prof.cnt[10407]++;/*I_*/return MindMapNode.STYLE_BUBBLE;/*_I*/}/*I_*/
     		}
     		else{
    			/*_I*/{_prof.prof.cnt[10408]++;/*I_*/return MindMapNode.STYLE_FORK;/*_I*/}/*I_*/
     		}/*_I*/}/*I_*/
     	}/*_I*/}/*I_*/
    	/*_I*/{_prof.prof.cnt[10409]++;/*I_*/return returnedString;/*_I*/}/*I_*/
	}
	
	

    /**The Foreground/Font Color*/
    public Color getColor() {/*_I*/_prof.prof.cnt[10410]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10411]++;/*I_*/return color;/*_I*/}/*I_*/ }

	
    //////
    // The set methods. I'm not sure if they should be here or in the implementing class.
    /////

    public void setStyle(String style) {/*_I*/_prof.prof.cnt[10412]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10413]++;/*I_*/this.style = style;/*_I*/}/*I_*/
    }

    public void setColor(Color color) {/*_I*/_prof.prof.cnt[10414]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10415]++;/*I_*/this.color = color;/*_I*/}/*I_*/
    }

    //fc, 24.2.2004: background color:
    public Color getBackgroundColor(           ) {/*_I*/_prof.prof.cnt[10416]++;/*I_*/ /*_I*/{_prof.prof.cnt[10417]++;/*I_*/return backgroundColor;/*_I*/}/*I_*/ };
    public void  setBackgroundColor(Color color) {/*_I*/_prof.prof.cnt[10418]++;/*I_*/ /*_I*/{_prof.prof.cnt[10419]++;/*I_*/this.backgroundColor = color;/*_I*/}/*I_*/ };

    //   
    //  font handling
    // 

    
    //   Remark to setBold and setItalic implemetation
    //
    // Using deriveFont() is a bad idea, because it does not really choose
    // the appropriate face. For example, instead of choosing face
    // "Arial Bold", it derives the bold face from "Arial".

    // Node holds font only in the case that the font is not default.

    public void estabilishOwnFont() {/*_I*/_prof.prof.cnt[10420]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10421]++;/*I_*/font = (font != null) ? font : getFrame().getController().getDefaultFont();/*_I*/}/*I_*/ }

    public void setBold(boolean bold) {/*_I*/_prof.prof.cnt[10422]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10423]++;/*I_*/if (bold != isBold()) {
           /*_I*/{_prof.prof.cnt[10424]++;/*I_*/toggleBold();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    public void toggleBold() {/*_I*/_prof.prof.cnt[10425]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10426]++;/*I_*/estabilishOwnFont();/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[10427]++;/*I_*/setFont(getFrame().getController().getFontThroughMap
               (new Font(font.getFamily(), font.getStyle() ^ Font.BOLD, font.getSize())));/*_I*/}/*I_*/ }

    public void setItalic(boolean italic) {/*_I*/_prof.prof.cnt[10428]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10429]++;/*I_*/if (italic != isItalic()) {
           /*_I*/{_prof.prof.cnt[10430]++;/*I_*/toggleItalic();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    public void toggleItalic() {/*_I*/_prof.prof.cnt[10431]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10432]++;/*I_*/estabilishOwnFont();/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[10433]++;/*I_*/setFont(getFrame().getController().getFontThroughMap
               (new Font(font.getFamily(), font.getStyle() ^ Font.ITALIC, font.getSize())));/*_I*/}/*I_*/ }

    public void setUnderlined(boolean underlined) {/*_I*/_prof.prof.cnt[10434]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10435]++;/*I_*/this.underlined = underlined;/*_I*/}/*I_*/ }

    public void setFont(Font font) {/*_I*/_prof.prof.cnt[10436]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10437]++;/*I_*/this.font = font;/*_I*/}/*I_*/ }

    public MindMapNode getParentNode() {/*_I*/_prof.prof.cnt[10438]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10439]++;/*I_*/return parent;/*_I*/}/*I_*/ }

    public void setFontSize(int fontSize) {/*_I*/_prof.prof.cnt[10440]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10441]++;/*I_*/estabilishOwnFont();/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[10442]++;/*I_*/setFont(getFrame().getController().getFontThroughMap
               (new Font(font.getFamily(), font.getStyle(), fontSize)));/*_I*/}/*I_*/ }

    public Font getFont() {/*_I*/_prof.prof.cnt[10443]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10444]++;/*I_*/return font;/*_I*/}/*I_*/ }

    public String getFontSize(){/*_I*/_prof.prof.cnt[10445]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10446]++;/*I_*/if (getFont() != null){
          /*_I*/{_prof.prof.cnt[10447]++;/*I_*/return new Integer (getFont().getSize()).toString();/*_I*/}/*I_*/}
       else {
          /*_I*/{_prof.prof.cnt[10448]++;/*I_*/return getFrame().getProperty("defaultfontsize");/*_I*/}/*I_*/}/*_I*/}/*I_*/
    }

    public String getFontFamilyName(){/*_I*/_prof.prof.cnt[10449]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10450]++;/*I_*/if (getFont() != null){
          /*_I*/{_prof.prof.cnt[10451]++;/*I_*/return getFont().getFamily();/*_I*/}/*I_*/}
       else {
          /*_I*/{_prof.prof.cnt[10452]++;/*I_*/return getFrame().getProperty("defaultfont");/*_I*/}/*I_*/}/*_I*/}/*I_*/
    }

    public boolean isBold() {/*_I*/_prof.prof.cnt[10453]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10454]++;/*I_*/return font != null ? font.isBold() : false;/*_I*/}/*I_*/ }

    public boolean isItalic() {/*_I*/_prof.prof.cnt[10455]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10456]++;/*I_*/return font != null ? font.isItalic() : false;/*_I*/}/*I_*/ }

    public boolean isUnderlined() {/*_I*/_prof.prof.cnt[10457]++;/*I_*/        // not implemented
       /*_I*/{_prof.prof.cnt[10458]++;/*I_*/return underlined;/*_I*/}/*I_*/ }

    public boolean isFolded() {/*_I*/_prof.prof.cnt[10459]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10460]++;/*I_*/return folded;/*_I*/}/*I_*/ }

    // fc, 24.9.2003:
    public List getIcons() {/*_I*/_prof.prof.cnt[10461]++;/*I_*/
    		/*_I*/{_prof.prof.cnt[10462]++;/*I_*/if(icons==null)
    			/*_I*/{_prof.prof.cnt[10463]++;/*I_*/return Collections.EMPTY_LIST;/*_I*/}/*I_*//*_I*/}/*I_*/
    		/*_I*/{_prof.prof.cnt[10464]++;/*I_*/return icons;/*_I*/}/*I_*/
    	}

    public void   addIcon(MindIcon _icon) {/*_I*/_prof.prof.cnt[10465]++;/*I_*/
    		/*_I*/{_prof.prof.cnt[10466]++;/*I_*/createIcons();/*_I*/}/*I_*/
    		/*_I*/{_prof.prof.cnt[10467]++;/*I_*/icons.add(_icon);/*_I*/}/*I_*/ 
    	}

    /** @return returns the number of remaining icons. */
	public int removeLastIcon() {/*_I*/_prof.prof.cnt[10468]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10469]++;/*I_*/createIcons();/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10470]++;/*I_*/if (icons.size() > 0)
			/*_I*/{_prof.prof.cnt[10471]++;/*I_*/icons.setSize(icons.size() - 1);/*_I*/}/*I_*//*_I*/}/*I_*/
		/*_I*/_prof.prof.cnt[10472]++;/*I_*/int returnSize = icons.size();
		/*_I*/{_prof.prof.cnt[10473]++;/*I_*/if(returnSize==0) {
			/*_I*/{_prof.prof.cnt[10474]++;/*I_*/icons = null;/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10475]++;/*I_*/return returnSize;/*_I*/}/*I_*/
	};

    // end, fc, 24.9.2003

//     public     String getLabel() { return mLabel; }

//     public     void setLabel(String newLabel) { mLabel = newLabel; /* bad hack: registry fragen.*/ };

//     public Vector/* of NodeLinkStruct*/ getReferences() { return mNodeLinkVector; };

//     public void removeReferenceAt(int i) {
//         if(mNodeLinkVector.size() > i) {
//             mNodeLinkVector.removeElementAt(i);
//         } else {
//             /* exception. */
//         }
//     }
    
//     public     void addReference(MindMapLink mindMapLink) { mNodeLinkVector.add(mindMapLink); };


    /**
     *  True iff one of node's <i>strict</i> descendants is folded. A node N
     *  is not its strict descendant - the fact that node itself is folded
     *  is not sufficient to return true.
     */
    public boolean hasFoldedStrictDescendant() {/*_I*/_prof.prof.cnt[10476]++;/*I_*/
       
       /*_I*/{_prof.prof.cnt[10477]++;/*I_*/for (ListIterator e = childrenUnfolded(); e.hasNext(); ) {
          /*_I*/_prof.prof.cnt[10478]++;/*I_*/NodeAdapter child = (NodeAdapter)e.next();
          /*_I*/{_prof.prof.cnt[10479]++;/*I_*/if (child.isFolded() || child.hasFoldedStrictDescendant()) {
             /*_I*/{_prof.prof.cnt[10480]++;/*I_*/return true;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[10481]++;/*I_*/return false;/*_I*/}/*I_*/
    }

    public void setFolded(boolean folded) {/*_I*/_prof.prof.cnt[10482]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10483]++;/*I_*/this.folded = folded;/*_I*/}/*I_*/
    }

    protected MindMapNode basicCopy() {/*_I*/_prof.prof.cnt[10484]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10485]++;/*I_*/return null;/*_I*/}/*I_*/ }
	
    public MindMapNode shallowCopy() {/*_I*/_prof.prof.cnt[10486]++;/*I_*/
       /*_I*/_prof.prof.cnt[10487]++;/*I_*/MindMapNode copy = basicCopy();
       /*_I*/{_prof.prof.cnt[10488]++;/*I_*/copy.setColor(getColor());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[10489]++;/*I_*/copy.setFont(getFont());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[10490]++;/*I_*/copy.setLink(getLink());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[10491]++;/*I_*/if(isLeft() != null)
           /*_I*/{_prof.prof.cnt[10492]++;/*I_*/copy.setLeft(isLeft().getValue());/*_I*/}/*I_*//*_I*/}/*I_*/
       /*_I*/_prof.prof.cnt[10493]++;/*I_*/List icons = getIcons();
       /*_I*/{_prof.prof.cnt[10494]++;/*I_*/for(int i = 0; i < icons.size(); ++i) {
           /*_I*/{_prof.prof.cnt[10495]++;/*I_*/copy.addIcon((MindIcon) icons.get(i));/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[10496]++;/*I_*/return copy;/*_I*/}/*I_*/ }

    //
    // other
    //

    public String toString() {/*_I*/_prof.prof.cnt[10497]++;/*I_*/
	/*_I*/_prof.prof.cnt[10498]++;/*I_*/String string="";
    /*_I*/{_prof.prof.cnt[10499]++;/*I_*/if (userObject!=null) {
        /*_I*/{_prof.prof.cnt[10500]++;/*I_*/string = userObject.toString();/*_I*/}/*I_*/
    }/*_I*/}/*I_*/
// (PN) %%%
// Why? If because of presentation, this level shall be self responsible... 
// Model shall NOT change the real data!!!
//
//	if (string.equals("")) {
//	    string = "   ";
//	}
	/*_I*/{_prof.prof.cnt[10501]++;/*I_*/return string;/*_I*/}/*I_*/
    }

    /**
     * Returns whether the argument is parent
     * or parent of one of the grandpa's of this node.
     * (transitive)
     */
    public boolean isDescendantOf(MindMapNode node) {/*_I*/_prof.prof.cnt[10502]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10503]++;/*I_*/if(this.isRoot())
	    /*_I*/{_prof.prof.cnt[10504]++;/*I_*/return false;/*_I*/}/*I_*/
	else /*_I*/{_prof.prof.cnt[10505]++;/*I_*/if (node == getParentNode())
	    /*_I*/{_prof.prof.cnt[10506]++;/*I_*/return true;/*_I*/}/*I_*/
	else
	    /*_I*/{_prof.prof.cnt[10507]++;/*I_*/return getParentNode().isDescendantOf(node);/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
    }	    
	 
    public boolean isRoot() {/*_I*/_prof.prof.cnt[10508]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10509]++;/*I_*/return (parent==null);/*_I*/}/*I_*/
    }

    public boolean hasChildren() {/*_I*/_prof.prof.cnt[10510]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10511]++;/*I_*/return children != null && !children.isEmpty();/*_I*/}/*I_*/ }

    public int getChildPosition(MindMapNode childNode) {/*_I*/_prof.prof.cnt[10512]++;/*I_*/
       /*_I*/_prof.prof.cnt[10513]++;/*I_*/int position = 0;
       /*_I*/{_prof.prof.cnt[10514]++;/*I_*/for (ListIterator i=children.listIterator(); i.hasNext(); ++position) {
          /*_I*/{_prof.prof.cnt[10515]++;/*I_*/if (((MindMapNode)i.next()) == childNode) {
             /*_I*/{_prof.prof.cnt[10516]++;/*I_*/return position;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[10517]++;/*I_*/return -1;/*_I*/}/*I_*/
    }

    public ListIterator childrenUnfolded() {/*_I*/_prof.prof.cnt[10518]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10519]++;/*I_*/return children != null ? children.listIterator() :
           Collections.EMPTY_LIST.listIterator();/*_I*/}/*I_*/ }

    public ListIterator childrenFolded() {/*_I*/_prof.prof.cnt[10520]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10521]++;/*I_*/if (isFolded()) {
          /*_I*/{_prof.prof.cnt[10522]++;/*I_*/return Collections.EMPTY_LIST.listIterator();/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[10523]++;/*I_*/return childrenUnfolded();/*_I*/}/*I_*/
    }

    //
    //  Interface TreeNode
    //

    /**
     * AFAIK there is no way to get an enumeration out of a linked list. So this exception must be
     * thrown, or we can't implement TreeNode anymore (maybe we shouldn't?)
     */
    public Enumeration children() {/*_I*/_prof.prof.cnt[10524]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10525]++;/*I_*/throw new UnsupportedOperationException("Use childrenFolded or childrenUnfolded instead");/*_I*/}/*I_*/
    }
	
    public boolean getAllowsChildren() {/*_I*/_prof.prof.cnt[10526]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10527]++;/*I_*/return ALLOWSCHILDREN;/*_I*/}/*I_*/
    }
	
    public TreeNode getChildAt(int childIndex) {/*_I*/_prof.prof.cnt[10528]++;/*I_*/
        // fc, 11.12.2004: This is not understandable, that a child does not
        // exist if the parent is folded.
        //	if (isFolded()) {
        //	    return null;
        //	}
        /*_I*/{_prof.prof.cnt[10529]++;/*I_*/return (TreeNode) children.get(childIndex);/*_I*/}/*I_*/
    }

    public int getChildCount() {/*_I*/_prof.prof.cnt[10530]++;/*I_*/
       /*_I*/{_prof.prof.cnt[10531]++;/*I_*/return children == null ? 0 : children.size();/*_I*/}/*I_*/
    }

// (PN)
//    public int getChildCount() {
//	if (isFolded()) {
//	    return 0;
//	}
//	return children.size();
//    }
//    // Daniel: ^ The name of this method is confusing. It does nto convey
//    // the meaning, at least not to me.

    public int getIndex( TreeNode node ) {/*_I*/_prof.prof.cnt[10532]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10533]++;/*I_*/return children.indexOf( (MindMapNode)node );/*_I*/}/*I_*/ //uses equals()
    } 

    public TreeNode getParent() {/*_I*/_prof.prof.cnt[10534]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10535]++;/*I_*/return parent;/*_I*/}/*I_*/
    }

    public boolean isLeaf() {/*_I*/_prof.prof.cnt[10536]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10537]++;/*I_*/return getChildCount() == 0;/*_I*/}/*I_*/
    }

    // fc, 16.12.2003 left-right bug:
    public Tools.BooleanHolder isLeft() {/*_I*/_prof.prof.cnt[10538]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10539]++;/*I_*/return left;/*_I*/}/*I_*/
    }
    
    
    
	/* (non-Javadoc)
	 * @see freemind.modes.MindMapNode#isOneLeftSideOfRoot()
	 */
	public boolean isOneLeftSideOfRoot() {/*_I*/_prof.prof.cnt[10540]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10541]++;/*I_*/if(isRoot()) 
			/*_I*/{_prof.prof.cnt[10542]++;/*I_*/return false;/*_I*/}/*I_*//*_I*/}/*I_*/
		// now, the node has a parent:
		/*_I*/{_prof.prof.cnt[10543]++;/*I_*/if(getParentNode().isRoot()) {
			/*_I*/{_prof.prof.cnt[10544]++;/*I_*/return (isLeft() == null)?false:isLeft().getValue();/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10545]++;/*I_*/return getParentNode().isOneLeftSideOfRoot();/*_I*/}/*I_*/
	}

	public void setLeft(boolean isLeft){/*_I*/_prof.prof.cnt[10546]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10547]++;/*I_*/if(left == null)
            /*_I*/{_prof.prof.cnt[10548]++;/*I_*/left = new Tools.BooleanHolder();/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10549]++;/*I_*/left.setValue(isLeft);/*_I*/}/*I_*/
    }

    //
    //  Interface MutableTreeNode
    //
    
    //do all remove methods have to work recursively to make the 
    //Garbage Collection work (Nodes in removed Sub-Trees reference each other)?
    
    public void insert( MutableTreeNode child, int index) {/*_I*/_prof.prof.cnt[10550]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10551]++;/*I_*/logger.finest("Insert at " + index + " the node "+child);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10552]++;/*I_*/if (index < 0) { // add to the end (used in xml load) (PN) 
          /*_I*/{_prof.prof.cnt[10553]++;/*I_*/index = getChildCount();/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[10554]++;/*I_*/children.add( index, child );/*_I*/}/*I_*/
        }
        else {           // mind preferred child :-)
          /*_I*/{_prof.prof.cnt[10555]++;/*I_*/children.add( index, child );/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[10556]++;/*I_*/preferredChild = (MindMapNode)child;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    	/*_I*/{_prof.prof.cnt[10557]++;/*I_*/child.setParent( this );/*_I*/}/*I_*/
    }
    


    
    public void remove( int index ) {/*_I*/_prof.prof.cnt[10558]++;/*I_*/
        /*_I*/_prof.prof.cnt[10559]++;/*I_*/MutableTreeNode node = (MutableTreeNode)children.get(index);
		/*_I*/{_prof.prof.cnt[10560]++;/*I_*/remove(node);/*_I*/}/*I_*/
    }
    
    public void remove( MutableTreeNode node ) {/*_I*/_prof.prof.cnt[10561]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10562]++;/*I_*/if (node == this.preferredChild) { // mind preferred child :-) (PN) 
          /*_I*/_prof.prof.cnt[10563]++;/*I_*/int index = children.indexOf(node);
          /*_I*/{_prof.prof.cnt[10564]++;/*I_*/if (children.size() > index + 1) {
            /*_I*/{_prof.prof.cnt[10565]++;/*I_*/this.preferredChild = (MindMapNode)(children.get(index + 1));/*_I*/}/*I_*/
          }
          else {
            /*_I*/{_prof.prof.cnt[10566]++;/*I_*/this.preferredChild = (index > 0) ? 
                (MindMapNode)(children.get(index - 1)) : null;/*_I*/}/*I_*/
          }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10567]++;/*I_*/node.setParent(null);/*_I*/}/*I_*/
    	/*_I*/{_prof.prof.cnt[10568]++;/*I_*/children.remove( node );/*_I*/}/*I_*/
    	// call remove child hook after removal.
    	/*_I*/{_prof.prof.cnt[10569]++;/*I_*/recursiveCallRemoveChildren(this, (MindMapNode) node, this);/*_I*/}/*I_*/
    }

	/**
	 * @param node
	 * @param oldDad the last dad node had.
	 */
	private void recursiveCallRemoveChildren(MindMapNode node, MindMapNode removedChild, MindMapNode oldDad) {/*_I*/_prof.prof.cnt[10570]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10571]++;/*I_*/for(Iterator i=  node.getActivatedHooks().iterator(); i.hasNext();) {
        	/*_I*/_prof.prof.cnt[10572]++;/*I_*/PermanentNodeHook hook = (PermanentNodeHook) i.next();
            /*_I*/{_prof.prof.cnt[10573]++;/*I_*/if (removedChild.getParentNode() == node) {
                /*_I*/{_prof.prof.cnt[10574]++;/*I_*/hook.onRemoveChild(removedChild);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[10575]++;/*I_*/hook.onRemoveChildren(removedChild, oldDad);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10576]++;/*I_*/if(!node.isRoot() && node.getParentNode()!= null)
		    /*_I*/{_prof.prof.cnt[10577]++;/*I_*/recursiveCallRemoveChildren(node.getParentNode(), removedChild, oldDad);/*_I*/}/*I_*//*_I*/}/*I_*/
	}


    
    public MindMapNode getPreferredChild() {/*_I*/_prof.prof.cnt[10578]++;/*I_*/ // mind preferred child :-) (PN) 
      /*_I*/{_prof.prof.cnt[10579]++;/*I_*/if (this.children.contains(this.preferredChild)) {
        /*_I*/{_prof.prof.cnt[10580]++;/*I_*/return this.preferredChild;/*_I*/}/*I_*/
      }
      else /*_I*/{_prof.prof.cnt[10581]++;/*I_*/if (!isLeaf()) {
        /*_I*/{_prof.prof.cnt[10582]++;/*I_*/return (MindMapNode)(this.children.get((getChildCount() + 1) / 2 - 1));/*_I*/}/*I_*/
      }
      else {
        /*_I*/{_prof.prof.cnt[10583]++;/*I_*/return null;/*_I*/}/*I_*/
      }/*_I*/}/*I_*//*_I*/}/*I_*/
    }
    public void setPreferredChild(MindMapNode node) {/*_I*/_prof.prof.cnt[10584]++;/*I_*/
      /*_I*/{_prof.prof.cnt[10585]++;/*I_*/this.preferredChild = node;/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[10586]++;/*I_*/if (node == null) {
        /*_I*/{_prof.prof.cnt[10587]++;/*I_*/return;/*_I*/}/*I_*/
      }
      else /*_I*/{_prof.prof.cnt[10588]++;/*I_*/if (this.parent != null) {
        // set also preffered child of parents...
        /*_I*/{_prof.prof.cnt[10589]++;/*I_*/this.parent.setPreferredChild(this);/*_I*/}/*I_*/
      }/*_I*/}/*I_*//*_I*/}/*I_*/
    }

    public void removeFromParent() {/*_I*/_prof.prof.cnt[10590]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[10591]++;/*I_*/parent.remove( this );/*_I*/}/*I_*/
    }

    public void setParent( MutableTreeNode newParent ) {/*_I*/_prof.prof.cnt[10592]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10593]++;/*I_*/parent = (MindMapNode) newParent;/*_I*/}/*I_*/
    }

    public void setParent( MindMapNode newParent ) {/*_I*/_prof.prof.cnt[10594]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[10595]++;/*I_*/parent = newParent;/*_I*/}/*I_*/
    }

    public void setUserObject( Object object ) {/*_I*/_prof.prof.cnt[10596]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10597]++;/*I_*/userObject = object;/*_I*/}/*I_*/
    }

    ////////////////
    // Private methods. Internal Implementation
    //////////////

    /** Recursive Method for getPath() */
    private void addToPathVector( Vector pathVector ) {/*_I*/_prof.prof.cnt[10598]++;/*I_*/
	/*_I*/{_prof.prof.cnt[10599]++;/*I_*/pathVector.add( 0, this );/*_I*/}/*I_*/ //Add myself to beginning of Vector
	/*_I*/{_prof.prof.cnt[10600]++;/*I_*/if ( parent != null ) {
	    /*_I*/{_prof.prof.cnt[10601]++;/*I_*/( (NodeAdapter)parent ).addToPathVector( pathVector );/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    public int getNodeLevel() {/*_I*/_prof.prof.cnt[10602]++;/*I_*/  //for cursor navigation within a level (PN)  
      /*_I*/_prof.prof.cnt[10603]++;/*I_*/int level = 0;
      /*_I*/_prof.prof.cnt[10604]++;/*I_*/MindMapNode parent;
      /*_I*/{_prof.prof.cnt[10605]++;/*I_*/for (parent = this; !parent.isRoot(); parent = parent.getParentNode()) {
        /*_I*/{_prof.prof.cnt[10606]++;/*I_*/level++;/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[10607]++;/*I_*/return level;/*_I*/}/*I_*/
    }
	/* (non-Javadoc)
	 * @see freemind.modes.MindMapNode#addHook(freemind.modes.NodeHook)
	 */
	public PermanentNodeHook addHook(PermanentNodeHook hook) {/*_I*/_prof.prof.cnt[10608]++;/*I_*/
		// add then
		/*_I*/{_prof.prof.cnt[10609]++;/*I_*/if(hook == null) 
			/*_I*/{_prof.prof.cnt[10610]++;/*I_*/throw new IllegalArgumentException("Added null hook.");/*_I*/}/*I_*//*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10611]++;/*I_*/createHooks();/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10612]++;/*I_*/hooks.add(hook);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10613]++;/*I_*/return hook;/*_I*/}/*I_*/
	}

	public void invokeHook(NodeHook hook) {/*_I*/_prof.prof.cnt[10614]++;/*I_*/
		// initialize:
		/*_I*/{_prof.prof.cnt[10615]++;/*I_*/hook.startupMapHook();/*_I*/}/*I_*/
		// the main invocation:
		/*_I*/{_prof.prof.cnt[10616]++;/*I_*/hook.setNode(this);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10617]++;/*I_*/try {
			/*_I*/{_prof.prof.cnt[10618]++;/*I_*/hook.invoke(this);/*_I*/}/*I_*/
		} catch (Exception e) {
			//FIXME: Do something special here, but in any case, do not add the hook
			// to the activatedHooks:
			/*_I*/{_prof.prof.cnt[10619]++;/*I_*/e.printStackTrace();/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[10620]++;/*I_*/return;/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[10621]++;/*I_*/if (hook instanceof PermanentNodeHook) {
	    		/*_I*/{_prof.prof.cnt[10622]++;/*I_*/createActivatedHooks();/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[10623]++;/*I_*/activatedHooks.add(hook);/*_I*/}/*I_*/
		} else {
		    // end of its short life:
		    /*_I*/{_prof.prof.cnt[10624]++;/*I_*/hook.shutdownMapHook();/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}

	private void createActivatedHooks() {/*_I*/_prof.prof.cnt[10625]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10626]++;/*I_*/if(activatedHooks == null) {
			/*_I*/{_prof.prof.cnt[10627]++;/*I_*/activatedHooks = new HashSet();/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}
	private void createToolTip() {/*_I*/_prof.prof.cnt[10628]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10629]++;/*I_*/if(toolTip == null) {
			/*_I*/{_prof.prof.cnt[10630]++;/*I_*/toolTip = new HashMap();/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}
	private void createHooks() {/*_I*/_prof.prof.cnt[10631]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10632]++;/*I_*/if(hooks == null) {
			/*_I*/{_prof.prof.cnt[10633]++;/*I_*/hooks = new Vector();/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}
	private void createStateIcons() {/*_I*/_prof.prof.cnt[10634]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10635]++;/*I_*/if(stateIcons == null) {
			/*_I*/{_prof.prof.cnt[10636]++;/*I_*/stateIcons = new TreeMap();/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}
	private void createIcons() {/*_I*/_prof.prof.cnt[10637]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10638]++;/*I_*/if(icons == null) {
			/*_I*/{_prof.prof.cnt[10639]++;/*I_*/icons = new Vector();/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}

	/* (non-Javadoc)
	 * @see freemind.modes.MindMapNode#getHooks()
	 */
	public List getHooks() {/*_I*/_prof.prof.cnt[10640]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10641]++;/*I_*/if(hooks==null)
			/*_I*/{_prof.prof.cnt[10642]++;/*I_*/return Collections.EMPTY_LIST;/*_I*/}/*I_*//*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10643]++;/*I_*/return Collections.unmodifiableList(hooks);/*_I*/}/*I_*/
	}

	/* (non-Javadoc)
	 * @see freemind.modes.MindMapNode#getActivatedHooks()
	 */
	public Collection getActivatedHooks() {/*_I*/_prof.prof.cnt[10644]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10645]++;/*I_*/if(activatedHooks==null) {
			/*_I*/{_prof.prof.cnt[10646]++;/*I_*/return Collections.EMPTY_LIST;/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10647]++;/*I_*/return Collections.unmodifiableCollection(activatedHooks);/*_I*/}/*I_*/
	}

	/* (non-Javadoc)
	 * @see freemind.modes.MindMapNode#removeHook(freemind.modes.NodeHook)
	 */
	public void removeHook(PermanentNodeHook hook) {/*_I*/_prof.prof.cnt[10648]++;/*I_*/
	    // the order is crucial here: the shutdown method should be able to perform "nodeChanged" 
	    // calls without having its own updateNodeHook method to be called again.
		/*_I*/{_prof.prof.cnt[10649]++;/*I_*/createActivatedHooks();/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10650]++;/*I_*/activatedHooks.remove(hook);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10651]++;/*I_*/if(activatedHooks.size()==0) {
			/*_I*/{_prof.prof.cnt[10652]++;/*I_*/activatedHooks=null;/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10653]++;/*I_*/hook.shutdownMapHook();/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10654]++;/*I_*/createHooks();/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10655]++;/*I_*/hooks.remove(hook);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10656]++;/*I_*/if(hooks.size()==0)
			/*_I*/{_prof.prof.cnt[10657]++;/*I_*/hooks=null;/*_I*/}/*I_*//*_I*/}/*I_*/
	}

	/**
	 * @return
	 */
	public java.util.Map getToolTip() {/*_I*/_prof.prof.cnt[10658]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10659]++;/*I_*/if(toolTip==null)
			/*_I*/{_prof.prof.cnt[10660]++;/*I_*/return Collections.EMPTY_MAP;/*_I*/}/*I_*//*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10661]++;/*I_*/return Collections.unmodifiableMap(toolTip);/*_I*/}/*I_*/
	}

	/**
	 * @param string
	 */
	public void setToolTip(String key, String string) {/*_I*/_prof.prof.cnt[10662]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10663]++;/*I_*/createToolTip();/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10664]++;/*I_*/if (string == null) {
		    /*_I*/{_prof.prof.cnt[10665]++;/*I_*/if (toolTip.containsKey(key)) {
                /*_I*/{_prof.prof.cnt[10666]++;/*I_*/toolTip.remove(key);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[10667]++;/*I_*/if(toolTip.size()==0)
            	/*_I*/{_prof.prof.cnt[10668]++;/*I_*/toolTip=null;/*_I*/}/*I_*//*_I*/}/*I_*/
        } else {
            /*_I*/{_prof.prof.cnt[10669]++;/*I_*/toolTip.put(key, string);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
	}


	/* (non-Javadoc)
	 * @see freemind.modes.MindMapNode#getNodeId()
	 */
	public String getObjectId(ModeController controller) {/*_I*/_prof.prof.cnt[10670]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[10671]++;/*I_*/return controller.getNodeID(this);/*_I*/}/*I_*/
	}

    public XMLElement save(Writer writer, MindMapLinkRegistry registry) throws IOException {/*_I*/_prof.prof.cnt[10672]++;/*I_*/
    	/*_I*/_prof.prof.cnt[10673]++;/*I_*/XMLElement node = new XMLElement();
    	
//    	if (!isNodeClassToBeSaved()) {
    	/*_I*/{_prof.prof.cnt[10674]++;/*I_*/node.setName(XMLElementAdapter.XML_NODE);/*_I*/}/*I_*/
//        } else {
//            node.setName(XMLElementAdapter.XML_NODE_CLASS_PREFIX
//                    + this.getClass().getName());
//        }
    
        /** fc, 12.6.2005: XML must not contain any zero characters. */
        /*_I*/_prof.prof.cnt[10675]++;/*I_*/String text = this.toString().replace('\0', ' ');
        /*_I*/{_prof.prof.cnt[10676]++;/*I_*/node.setAttribute(XMLElementAdapter.XML_NODE_TEXT,text);/*_I*/}/*I_*/
    	// save additional info:
    	/*_I*/{_prof.prof.cnt[10677]++;/*I_*/if (getAdditionalInfo() != null) {
            /*_I*/{_prof.prof.cnt[10678]++;/*I_*/node.setAttribute(XMLElementAdapter.XML_NODE_ENCRYPTED_CONTENT,
                    getAdditionalInfo());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    	//	((MindMapEdgeModel)getEdge()).save(doc,node);
    
    	/*_I*/_prof.prof.cnt[10679]++;/*I_*/XMLElement edge = (getEdge()).save();
    	/*_I*/{_prof.prof.cnt[10680]++;/*I_*/if (edge != null) {
               /*_I*/{_prof.prof.cnt[10681]++;/*I_*/node.addChild(edge);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    
        /*_I*/{_prof.prof.cnt[10682]++;/*I_*/if(getCloud() != null) {
            /*_I*/_prof.prof.cnt[10683]++;/*I_*/XMLElement cloud = (getCloud()).save();
            /*_I*/{_prof.prof.cnt[10684]++;/*I_*/node.addChild(cloud);/*_I*/}/*I_*/ 
        }/*_I*/}/*I_*/
    
        /*_I*/_prof.prof.cnt[10685]++;/*I_*/Vector linkVector = registry.getAllLinksFromMe(this); /* Puh... */
        /*_I*/{_prof.prof.cnt[10686]++;/*I_*/for(int i = 0; i < linkVector.size(); ++i) {
            /*_I*/{_prof.prof.cnt[10687]++;/*I_*/if(linkVector.get(i) instanceof ArrowLinkAdapter) {
                /*_I*/_prof.prof.cnt[10688]++;/*I_*/XMLElement arrowLinkElement = ((ArrowLinkAdapter) linkVector.get(i)).save();
                /*_I*/{_prof.prof.cnt[10689]++;/*I_*/node.addChild(arrowLinkElement);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
            
    	/*_I*/{_prof.prof.cnt[10690]++;/*I_*/if (isFolded()) {
               /*_I*/{_prof.prof.cnt[10691]++;/*I_*/node.setAttribute("FOLDED","true");/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    	
        // fc, 17.12.2003: Remove the left/right bug.
        //                       VVV  save if and only if parent is root.
    	/*_I*/{_prof.prof.cnt[10692]++;/*I_*/if ((isLeft()!= null) && !(isRoot()) && (getParentNode().isRoot())) {
            /*_I*/{_prof.prof.cnt[10693]++;/*I_*/node.setAttribute("POSITION",(isLeft().getValue())?"left":"right");/*_I*/}/*I_*/ 
        }/*_I*/}/*I_*/
    	
        /*_I*/_prof.prof.cnt[10694]++;/*I_*/String label = registry.getLabel(this); /* Puh... */
    	/*_I*/{_prof.prof.cnt[10695]++;/*I_*/if (label!=null) {
               /*_I*/{_prof.prof.cnt[10696]++;/*I_*/node.setAttribute("ID",label);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    	
    	/*_I*/{_prof.prof.cnt[10697]++;/*I_*/if (color != null) {
               /*_I*/{_prof.prof.cnt[10698]++;/*I_*/node.setAttribute("COLOR", Tools.colorToXml(getColor()));/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    
    	// new background color.
    	/*_I*/{_prof.prof.cnt[10699]++;/*I_*/if (getBackgroundColor() != null) {
    		   /*_I*/{_prof.prof.cnt[10700]++;/*I_*/node.setAttribute("BACKGROUND_COLOR", Tools.colorToXml(getBackgroundColor()));/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    
    
    	/*_I*/{_prof.prof.cnt[10701]++;/*I_*/if (style != null) {
               /*_I*/{_prof.prof.cnt[10702]++;/*I_*/node.setAttribute("STYLE", this.getStyle());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    	    //  ^ Here cannot be just getStyle() without super. This is because
    	    //  getStyle's style depends on folded / unfolded. For example, when
    	    //  real style is fork and node is folded, getStyle returns
    	    //  MindMapNode.STYLE_BUBBLE, which is not what we want to save.
    
    	//layout
        /*_I*/{_prof.prof.cnt[10703]++;/*I_*/if(vGap != AUTO) {
        	/*_I*/{_prof.prof.cnt[10704]++;/*I_*/node.setAttribute("VGAP",Integer.toString(vGap));/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10705]++;/*I_*/if(hGap != HGAP) {
        	/*_I*/{_prof.prof.cnt[10706]++;/*I_*/node.setAttribute("HGAP",Integer.toString(hGap));/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10707]++;/*I_*/if(shiftY != 0) {
        	/*_I*/{_prof.prof.cnt[10708]++;/*I_*/node.setAttribute("VSHIFT",Integer.toString(shiftY));/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    	//link
    	/*_I*/{_prof.prof.cnt[10709]++;/*I_*/if (getLink() != null) {
               /*_I*/{_prof.prof.cnt[10710]++;/*I_*/node.setAttribute("LINK", getLink());/*_I*/}/*I_*/ }/*_I*/}/*I_*/


    	//history information, fc, 11.4.2005
    	/*_I*/{_prof.prof.cnt[10711]++;/*I_*/if (historyInformation != null) {
			/*_I*/{_prof.prof.cnt[10712]++;/*I_*/node.setAttribute(XMLElementAdapter.XML_NODE_HISTORY_CREATED_AT,
					Tools.dateToString(getHistoryInformation().getCreatedAt()));/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[10713]++;/*I_*/node.setAttribute(
					XMLElementAdapter.XML_NODE_HISTORY_LAST_MODIFIED_AT, Tools
							.dateToString(getHistoryInformation()
									.getLastModifiedAt()));/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    	//font
    	/*_I*/{_prof.prof.cnt[10714]++;/*I_*/if (font!=null) {
    	    /*_I*/_prof.prof.cnt[10715]++;/*I_*/XMLElement fontElement = new XMLElement();
    	    /*_I*/{_prof.prof.cnt[10716]++;/*I_*/fontElement.setName("font");/*_I*/}/*I_*/
    
    	    /*_I*/{_prof.prof.cnt[10717]++;/*I_*/if (font != null) {
                   /*_I*/{_prof.prof.cnt[10718]++;/*I_*/fontElement.setAttribute("NAME",font.getFamily());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    	    /*_I*/{_prof.prof.cnt[10719]++;/*I_*/if (font.getSize() != 0) {
                   /*_I*/{_prof.prof.cnt[10720]++;/*I_*/fontElement.setAttribute("SIZE",Integer.toString(font.getSize()));/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    	    /*_I*/{_prof.prof.cnt[10721]++;/*I_*/if (isBold()) {
                   /*_I*/{_prof.prof.cnt[10722]++;/*I_*/fontElement.setAttribute("BOLD","true");/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    	    /*_I*/{_prof.prof.cnt[10723]++;/*I_*/if (isItalic()) {
                   /*_I*/{_prof.prof.cnt[10724]++;/*I_*/fontElement.setAttribute("ITALIC","true");/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    	    /*_I*/{_prof.prof.cnt[10725]++;/*I_*/if (isUnderlined()) {
                   /*_I*/{_prof.prof.cnt[10726]++;/*I_*/fontElement.setAttribute("UNDERLINE","true");/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    	    /*_I*/{_prof.prof.cnt[10727]++;/*I_*/node.addChild(fontElement);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10728]++;/*I_*/for(int i = 0; i < getIcons().size(); ++i) {
    	    /*_I*/_prof.prof.cnt[10729]++;/*I_*/XMLElement iconElement = new XMLElement();
    	    /*_I*/{_prof.prof.cnt[10730]++;/*I_*/iconElement.setName("icon");/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[10731]++;/*I_*/iconElement.setAttribute("BUILTIN", ((MindIcon) getIcons().get(i)).getName());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[10732]++;/*I_*/node.addChild(iconElement);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    
    	/*_I*/{_prof.prof.cnt[10733]++;/*I_*/for (Iterator i = getActivatedHooks().iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[10734]++;/*I_*/XMLElement hookElement = new XMLElement();
            /*_I*/{_prof.prof.cnt[10735]++;/*I_*/hookElement.setName("hook");/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[10736]++;/*I_*/((PermanentNodeHook) i.next()).save(hookElement);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[10737]++;/*I_*/node.addChild(hookElement);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[10738]++;/*I_*/if (childrenUnfolded().hasNext()) {
            /*_I*/{_prof.prof.cnt[10739]++;/*I_*/node.writeWithoutClosingTag(writer);/*_I*/}/*I_*/
            //recursive
            /*_I*/{_prof.prof.cnt[10740]++;/*I_*/for (ListIterator e = childrenUnfolded(); e.hasNext();) {
                /*_I*/_prof.prof.cnt[10741]++;/*I_*/NodeAdapter child = (NodeAdapter) e.next();
                /*_I*/{_prof.prof.cnt[10742]++;/*I_*/child.save(writer, registry);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[10743]++;/*I_*/node.writeClosingTag(writer);/*_I*/}/*I_*/
        } else {
            /*_I*/{_prof.prof.cnt[10744]++;/*I_*/node.write(writer);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10745]++;/*I_*/return node;/*_I*/}/*I_*/
    }
    
	public int getShiftY() {/*_I*/_prof.prof.cnt[10746]++;/*I_*/
			/*_I*/{_prof.prof.cnt[10747]++;/*I_*/return shiftY ;/*_I*/}/*I_*/
	}
	
	public int calcShiftY() {/*_I*/_prof.prof.cnt[10748]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10749]++;/*I_*/try{
			/*_I*/{_prof.prof.cnt[10750]++;/*I_*/return shiftY + (getParent().getChildCount()== 1 ? SHIFT:0);/*_I*/}/*I_*/
		}
		catch(NullPointerException e){
			/*_I*/{_prof.prof.cnt[10751]++;/*I_*/return 0;/*_I*/}/*I_*/			
		}/*_I*/}/*I_*/
		
	}
	/**
	 * @param shiftY The shiftY to set.
	 */
	public void setShiftY(int shiftY) {/*_I*/_prof.prof.cnt[10752]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10753]++;/*I_*/this.shiftY = shiftY;/*_I*/}/*I_*/
	}
    /**
     *
     */

    public void setAdditionalInfo(String info) {/*_I*/_prof.prof.cnt[10754]++;/*I_*/
    }
    
    public String getAdditionalInfo(){/*_I*/_prof.prof.cnt[10755]++;/*I_*/
        /*_I*/{_prof.prof.cnt[10756]++;/*I_*/return null;/*_I*/}/*I_*/
    }
    
   
    /** This method must be synchronized as the TreeMap isn't. */
    public synchronized void setStateIcon(String key, ImageIcon icon) {/*_I*/_prof.prof.cnt[10757]++;/*I_*/
//    		logger.warning("Set state of key:"+key+", icon "+icon);
    		/*_I*/{_prof.prof.cnt[10758]++;/*I_*/createStateIcons();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10759]++;/*I_*/if (icon != null) {
			/*_I*/{_prof.prof.cnt[10760]++;/*I_*/stateIcons.put(key, icon);/*_I*/}/*I_*/
		} else /*_I*/{_prof.prof.cnt[10761]++;/*I_*/if(stateIcons.containsKey(key)) {
            /*_I*/{_prof.prof.cnt[10762]++;/*I_*/stateIcons.remove(key);/*_I*/}/*I_*/
        }/*_I*/}/*I_*//*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[10763]++;/*I_*/if(stateIcons.size()==0)
			/*_I*/{_prof.prof.cnt[10764]++;/*I_*/stateIcons = null;/*_I*/}/*I_*//*_I*/}/*I_*/
    }
    public Map getStateIcons() {/*_I*/_prof.prof.cnt[10765]++;/*I_*/
    		/*_I*/{_prof.prof.cnt[10766]++;/*I_*/if(stateIcons==null)
    			/*_I*/{_prof.prof.cnt[10767]++;/*I_*/return Collections.EMPTY_MAP;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[10768]++;/*I_*/return Collections.unmodifiableSortedMap(stateIcons);/*_I*/}/*I_*/
    }
	public HistoryInformation getHistoryInformation() {/*_I*/_prof.prof.cnt[10769]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10770]++;/*I_*/return historyInformation;/*_I*/}/*I_*/
	}
	public void setHistoryInformation(HistoryInformation historyInformation) {/*_I*/_prof.prof.cnt[10771]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10772]++;/*I_*/this.historyInformation = historyInformation;/*_I*/}/*I_*/
	}

	public int getHGap() {/*_I*/_prof.prof.cnt[10773]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10774]++;/*I_*/return hGap;/*_I*/}/*I_*/
	}
	public void setHGap(int gap) {/*_I*/_prof.prof.cnt[10775]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10776]++;/*I_*/hGap = Math.max(HGAP, gap);/*_I*/}/*I_*/
	}

	public int getVGap() {/*_I*/_prof.prof.cnt[10777]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10778]++;/*I_*/return vGap;/*_I*/}/*I_*/
}
	public int calcVGap() {/*_I*/_prof.prof.cnt[10779]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10780]++;/*I_*/if (vGap != AUTO)
			/*_I*/{_prof.prof.cnt[10781]++;/*I_*/return vGap;/*_I*/}/*I_*//*_I*/}/*I_*/
/*		
//		double delta = 8.0 / Math.pow(1.5, 1 + getNodeLevel()); // to expensive... 
		double delta = 8.0 / Math.pow(1 + getNodeLevel(), 1.5); 
        return (int ) ((1 + delta) * VGAP );			 
*/
		/*_I*/{_prof.prof.cnt[10782]++;/*I_*/return VGAP;/*_I*/}/*I_*/		    
	}
	
	public void setVGap(int gap) {/*_I*/_prof.prof.cnt[10783]++;/*I_*/
		/*_I*/{_prof.prof.cnt[10784]++;/*I_*/if (gap == AUTO) /*_I*/{_prof.prof.cnt[10785]++;/*I_*/vGap = AUTO;/*_I*/}/*I_*/
		else /*_I*/{_prof.prof.cnt[10786]++;/*I_*/vGap = Math.max(gap, 0);/*_I*/}/*I_*//*_I*/}/*I_*/
	}
}
