/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: XMLElementAdapter.java,v 1.4.14.13 2005/07/06 06:00:03 christianfoltin Exp $*/

package freemind.modes;

import java.awt.Font;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import freemind.extensions.PermanentNodeHook;
import freemind.extensions.PermanentNodeHookSubstituteUnknown;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.main.XMLElement;
import freemind.modes.mindmapmode.EncryptedMindMapNode;

public abstract class XMLElementAdapter extends XMLElement {

    
	// Logging: 
	protected static java.util.logging.Logger logger;

   private Object           userObject = null;
   private FreeMindMain     frame;
   private NodeAdapter      mapChild   = null;
   private HashMap 		  nodeAttributes = new HashMap();

   //   Font attributes

   private String fontName; 
   private int    fontStyle = 0;
   private int    fontSize = 0;

   //   Icon attributes

   private String iconName; 

    // arrow link attributes:
    protected Vector ArrowLinkAdapters;
    protected HashMap /* id -> target */  IDToTarget;
    public static final String XML_NODE_TEXT = "TEXT";
    public static final String XML_NODE = "node";
    public static final String XML_NODE_ENCRYPTED_CONTENT = "ENCRYPTED_CONTENT";
    public static final String XML_NODE_HISTORY_CREATED_AT = "CREATED";
    public static final String XML_NODE_HISTORY_LAST_MODIFIED_AT = "MODIFIED";

   //   Overhead methods

   public XMLElementAdapter(FreeMindMain frame) {/*_I*/_prof.prof.cnt[5052]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5053]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/ 
      /*_I*/{_prof.prof.cnt[5054]++;/*I_*/this.ArrowLinkAdapters = new Vector();/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[5055]++;/*I_*/this.IDToTarget = new HashMap();/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[5056]++;/*I_*/if(logger==null) {
          /*_I*/{_prof.prof.cnt[5057]++;/*I_*/logger = frame.getLogger(this.getClass().getName());/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
   }

    protected XMLElementAdapter(FreeMindMain frame, Vector ArrowLinkAdapters, HashMap IDToTarget) {/*_I*/_prof.prof.cnt[5058]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5059]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/ 
        /*_I*/{_prof.prof.cnt[5060]++;/*I_*/this.ArrowLinkAdapters = ArrowLinkAdapters;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5061]++;/*I_*/this.IDToTarget = IDToTarget;/*_I*/}/*I_*/
    }

    /** abstract method to create elements of my type (factory).*/
    abstract protected XMLElement  createAnotherElement();
    abstract protected NodeAdapter createNodeAdapter(FreeMindMain     frame, String nodeClass);
    abstract protected EdgeAdapter createEdgeAdapter(NodeAdapter node, FreeMindMain frame);
    abstract protected CloudAdapter createCloudAdapter(NodeAdapter node, FreeMindMain frame);
    abstract protected ArrowLinkAdapter createArrowLinkAdapter(NodeAdapter source, NodeAdapter target, FreeMindMain frame);
    
    protected FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[5062]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5063]++;/*I_*/return frame;/*_I*/}/*I_*/
    }

   public Object getUserObject() {/*_I*/_prof.prof.cnt[5064]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5065]++;/*I_*/return userObject;/*_I*/}/*I_*/ }

   public NodeAdapter getMapChild() {/*_I*/_prof.prof.cnt[5066]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5067]++;/*I_*/return mapChild;/*_I*/}/*I_*/ }

   //   Real parsing methods

   public void setName(String name)  {/*_I*/_prof.prof.cnt[5068]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5069]++;/*I_*/super.setName(name);/*_I*/}/*I_*/
		// Create user object based on name
		/*_I*/{_prof.prof.cnt[5070]++;/*I_*/if (name.equals(XML_NODE)) {
			/*_I*/{_prof.prof.cnt[5071]++;/*I_*/userObject = createNodeAdapter(frame, null);/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[5072]++;/*I_*/nodeAttributes.clear();/*_I*/}/*I_*/
		} else /*_I*/{_prof.prof.cnt[5073]++;/*I_*/if (name.equals("edge")) {
			/*_I*/{_prof.prof.cnt[5074]++;/*I_*/userObject = createEdgeAdapter(null, frame);/*_I*/}/*I_*/
		} else /*_I*/{_prof.prof.cnt[5075]++;/*I_*/if (name.equals("cloud")) {
			/*_I*/{_prof.prof.cnt[5076]++;/*I_*/userObject = createCloudAdapter(null, frame);/*_I*/}/*I_*/
		} else /*_I*/{_prof.prof.cnt[5077]++;/*I_*/if (name.equals("arrowlink")) {
			/*_I*/{_prof.prof.cnt[5078]++;/*I_*/userObject = createArrowLinkAdapter(null, null, frame);/*_I*/}/*I_*/
		} else /*_I*/{_prof.prof.cnt[5079]++;/*I_*/if (name.equals("font")) {
			/*_I*/{_prof.prof.cnt[5080]++;/*I_*/userObject = null;/*_I*/}/*I_*/
		} else /*_I*/{_prof.prof.cnt[5081]++;/*I_*/if (name.equals("map")) {
			/*_I*/{_prof.prof.cnt[5082]++;/*I_*/userObject = null;/*_I*/}/*I_*/
		} else /*_I*/{_prof.prof.cnt[5083]++;/*I_*/if (name.equals("icon")) {
			/*_I*/{_prof.prof.cnt[5084]++;/*I_*/userObject = null;/*_I*/}/*I_*/
		} else /*_I*/{_prof.prof.cnt[5085]++;/*I_*/if (name.equals("hook")) {
			// we gather the xml element and send it to the hook after completion.
			/*_I*/{_prof.prof.cnt[5086]++;/*I_*/userObject = new XMLElement();/*_I*/}/*I_*/
		} else {
			/*_I*/{_prof.prof.cnt[5087]++;/*I_*/userObject = new XMLElement();/*_I*/}/*I_*/ // for childs of hooks
		}/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
   }

   public void addChild(XMLElement child) {/*_I*/_prof.prof.cnt[5088]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5089]++;/*I_*/if (getName().equals("map")) {
         /*_I*/{_prof.prof.cnt[5090]++;/*I_*/mapChild = (NodeAdapter)child.getUserObject();/*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[5091]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[5092]++;/*I_*/if( userObject instanceof XMLElement ) {
      	 //((XMLElement) userObject).addChild(child);
      	 /*_I*/{_prof.prof.cnt[5093]++;/*I_*/super.addChild(child);/*_I*/}/*I_*/
      	 /*_I*/{_prof.prof.cnt[5094]++;/*I_*/return;/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[5095]++;/*I_*/if (userObject instanceof NodeAdapter) {
         /*_I*/_prof.prof.cnt[5096]++;/*I_*/NodeAdapter node = (NodeAdapter)userObject;
         /*_I*/{_prof.prof.cnt[5097]++;/*I_*/if (child.getUserObject() instanceof NodeAdapter) {
            /*_I*/{_prof.prof.cnt[5098]++;/*I_*/node.insert((NodeAdapter)child.getUserObject(),
                        -1);/*_I*/}/*I_*/ } // to the end without preferable... (PN)
                     // node.getRealChildCount()); }
         else /*_I*/{_prof.prof.cnt[5099]++;/*I_*/if (child.getUserObject() instanceof EdgeAdapter) {
            /*_I*/_prof.prof.cnt[5100]++;/*I_*/EdgeAdapter edge = (EdgeAdapter)child.getUserObject();
            /*_I*/{_prof.prof.cnt[5101]++;/*I_*/edge.setTarget(node);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5102]++;/*I_*/node.setEdge(edge);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5103]++;/*I_*/if (child.getUserObject() instanceof CloudAdapter) {
            /*_I*/_prof.prof.cnt[5104]++;/*I_*/CloudAdapter cloud = (CloudAdapter)child.getUserObject();
            /*_I*/{_prof.prof.cnt[5105]++;/*I_*/cloud.setTarget(node);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5106]++;/*I_*/node.setCloud(cloud);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5107]++;/*I_*/if (child.getUserObject() instanceof ArrowLinkAdapter) {
            /*_I*/_prof.prof.cnt[5108]++;/*I_*/ArrowLinkAdapter arrowLink = (ArrowLinkAdapter)child.getUserObject();
            /*_I*/{_prof.prof.cnt[5109]++;/*I_*/arrowLink.setSource(node);/*_I*/}/*I_*/
            // annotate this link: (later processed by caller.).
            //System.out.println("arrowLink="+arrowLink);
            /*_I*/{_prof.prof.cnt[5110]++;/*I_*/ArrowLinkAdapters.add(arrowLink);/*_I*/}/*I_*/
         }
         else /*_I*/{_prof.prof.cnt[5111]++;/*I_*/if (child.getName().equals("font")) {
            /*_I*/{_prof.prof.cnt[5112]++;/*I_*/node.setFont((Font)child.getUserObject());/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5113]++;/*I_*/if (child.getName().equals("icon")) {
             /*_I*/{_prof.prof.cnt[5114]++;/*I_*/node.addIcon((MindIcon)child.getUserObject());/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5115]++;/*I_*/if (child.getName().equals("hook")) {
         	 /*_I*/_prof.prof.cnt[5116]++;/*I_*/XMLElement xml = (XMLElement) child/*.getUserObject()*/;
             /*_I*/_prof.prof.cnt[5117]++;/*I_*/String loadName = (String)xml.getAttribute("NAME");
 			 /*_I*/_prof.prof.cnt[5118]++;/*I_*/PermanentNodeHook hook = null;
             /*_I*/{_prof.prof.cnt[5119]++;/*I_*/try {
				 //loadName=loadName.replace('/', File.separatorChar);
				 /* The next code snippet is an exception. Normally, hooks 
				  * have to be created via the ModeController. 
				  * DO NOT COPY. */
                /*_I*/{_prof.prof.cnt[5120]++;/*I_*/hook = (PermanentNodeHook) frame.getHookFactory().createNodeHook(loadName);/*_I*/}/*I_*/
                // this is a bad hack. Don't make use of this data unless
                // you know exactly what you are doing.
                /*_I*/{_prof.prof.cnt[5121]++;/*I_*/hook.setNode(node);/*_I*/}/*I_*/
             } catch(Exception e) {
                 /*_I*/{_prof.prof.cnt[5122]++;/*I_*/e.printStackTrace();/*_I*/}/*I_*/
                 /*_I*/{_prof.prof.cnt[5123]++;/*I_*/hook = new PermanentNodeHookSubstituteUnknown(loadName);/*_I*/}/*I_*/
             }/*_I*/}/*I_*/
 			 /*_I*/{_prof.prof.cnt[5124]++;/*I_*/hook.loadFrom(xml);/*_I*/}/*I_*/
 			 /*_I*/{_prof.prof.cnt[5125]++;/*I_*/node.addHook(hook);/*_I*/}/*I_*/
 		 }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
      }/*_I*/}/*I_*/
   }

public void setAttribute(String name, Object value) {/*_I*/_prof.prof.cnt[5126]++;/*I_*/
      // We take advantage of precondition that value != null.
      /*_I*/_prof.prof.cnt[5127]++;/*I_*/String sValue = value.toString();
      /*_I*/{_prof.prof.cnt[5128]++;/*I_*/if (ignoreCase) {
         /*_I*/{_prof.prof.cnt[5129]++;/*I_*/name = name.toUpperCase();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	  /*_I*/{_prof.prof.cnt[5130]++;/*I_*/if(userObject instanceof XMLElement) {
		//((XMLElement) userObject).setAttribute(name, value);
		/*_I*/{_prof.prof.cnt[5131]++;/*I_*/super.setAttribute(name, value);/*_I*/}/*I_*/ // and to myself, as I am also an xml element.
		/*_I*/{_prof.prof.cnt[5132]++;/*I_*/return;/*_I*/}/*I_*/
	  }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[5133]++;/*I_*/if (userObject instanceof NodeAdapter) {
         //
         /*_I*/_prof.prof.cnt[5134]++;/*I_*/NodeAdapter node = (NodeAdapter)userObject;
         /*_I*/{_prof.prof.cnt[5135]++;/*I_*/setNodeAttribute(name, sValue, node);/*_I*/}/*I_*/
     	/*_I*/{_prof.prof.cnt[5136]++;/*I_*/nodeAttributes.put(name, sValue);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5137]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[5138]++;/*I_*/if (userObject instanceof EdgeAdapter) {
         /*_I*/_prof.prof.cnt[5139]++;/*I_*/EdgeAdapter edge = (EdgeAdapter)userObject;
         /*_I*/{_prof.prof.cnt[5140]++;/*I_*/if (name.equals("STYLE")) {
	    /*_I*/{_prof.prof.cnt[5141]++;/*I_*/edge.setStyle(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5142]++;/*I_*/if (name.equals("COLOR")) {
	    /*_I*/{_prof.prof.cnt[5143]++;/*I_*/edge.setColor(Tools.xmlToColor(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5144]++;/*I_*/if (name.equals("WIDTH")) {
            /*_I*/{_prof.prof.cnt[5145]++;/*I_*/if (sValue.equals("thin")) {
               /*_I*/{_prof.prof.cnt[5146]++;/*I_*/edge.setWidth(EdgeAdapter.WIDTH_THIN);/*_I*/}/*I_*/ }
            else {
               /*_I*/{_prof.prof.cnt[5147]++;/*I_*/edge.setWidth(Integer.parseInt(sValue));/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[5148]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[5149]++;/*I_*/if (userObject instanceof CloudAdapter) {
         /*_I*/_prof.prof.cnt[5150]++;/*I_*/CloudAdapter cloud = (CloudAdapter)userObject;
         /*_I*/{_prof.prof.cnt[5151]++;/*I_*/if (name.equals("STYLE")) {
	    /*_I*/{_prof.prof.cnt[5152]++;/*I_*/cloud.setStyle(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5153]++;/*I_*/if (name.equals("COLOR")) {
	    /*_I*/{_prof.prof.cnt[5154]++;/*I_*/cloud.setColor(Tools.xmlToColor(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5155]++;/*I_*/if (name.equals("WIDTH")) {
               /*_I*/{_prof.prof.cnt[5156]++;/*I_*/cloud.setWidth(Integer.parseInt(sValue));/*_I*/}/*I_*/ 
         }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[5157]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[5158]++;/*I_*/if (userObject instanceof ArrowLinkAdapter) {
         /*_I*/_prof.prof.cnt[5159]++;/*I_*/ArrowLinkAdapter arrowLink = (ArrowLinkAdapter)userObject;
         /*_I*/{_prof.prof.cnt[5160]++;/*I_*/if (name.equals("STYLE")) {
             /*_I*/{_prof.prof.cnt[5161]++;/*I_*/arrowLink.setStyle(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5162]++;/*I_*/if (name.equals("ID")) {
             /*_I*/{_prof.prof.cnt[5163]++;/*I_*/arrowLink.setUniqueID(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5164]++;/*I_*/if (name.equals("COLOR")) {
             /*_I*/{_prof.prof.cnt[5165]++;/*I_*/arrowLink.setColor(Tools.xmlToColor(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5166]++;/*I_*/if (name.equals("DESTINATION")) {
             /*_I*/{_prof.prof.cnt[5167]++;/*I_*/arrowLink.setDestinationLabel(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5168]++;/*I_*/if (name.equals("REFERENCETEXT")) {
             /*_I*/{_prof.prof.cnt[5169]++;/*I_*/arrowLink.setReferenceText((sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5170]++;/*I_*/if (name.equals("STARTINCLINATION")) {
             /*_I*/{_prof.prof.cnt[5171]++;/*I_*/arrowLink.setStartInclination(Tools.xmlToPoint(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5172]++;/*I_*/if (name.equals("ENDINCLINATION")) {
             /*_I*/{_prof.prof.cnt[5173]++;/*I_*/arrowLink.setEndInclination(Tools.xmlToPoint(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5174]++;/*I_*/if (name.equals("STARTARROW")) {
             /*_I*/{_prof.prof.cnt[5175]++;/*I_*/arrowLink.setStartArrow(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5176]++;/*I_*/if (name.equals("ENDARROW")) {
             /*_I*/{_prof.prof.cnt[5177]++;/*I_*/arrowLink.setEndArrow(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5178]++;/*I_*/if (name.equals("WIDTH")) {
             /*_I*/{_prof.prof.cnt[5179]++;/*I_*/arrowLink.setWidth(Integer.parseInt(sValue));/*_I*/}/*I_*/ 
         }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[5180]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[5181]++;/*I_*/if (getName().equals("font")) {
         /*_I*/{_prof.prof.cnt[5182]++;/*I_*/if (name.equals("SIZE")) {
            /*_I*/{_prof.prof.cnt[5183]++;/*I_*/fontSize = Integer.parseInt(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[5184]++;/*I_*/if (name.equals("NAME")) {
            /*_I*/{_prof.prof.cnt[5185]++;/*I_*/fontName = sValue;/*_I*/}/*I_*/ }             

         // Styling
         else /*_I*/{_prof.prof.cnt[5186]++;/*I_*/if (sValue.equals("true")) {
            /*_I*/{_prof.prof.cnt[5187]++;/*I_*/if (name.equals("BOLD")) {
               /*_I*/{_prof.prof.cnt[5188]++;/*I_*/fontStyle+=Font.BOLD;/*_I*/}/*I_*/ }
            else /*_I*/{_prof.prof.cnt[5189]++;/*I_*/if (name.equals("ITALIC")) {
               /*_I*/{_prof.prof.cnt[5190]++;/*I_*/fontStyle+=Font.ITALIC;/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*/}/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/}/*_I*/}/*I_*/
      /* icons */
      /*_I*/{_prof.prof.cnt[5191]++;/*I_*/if (getName().equals("icon")) {
         /*_I*/{_prof.prof.cnt[5192]++;/*I_*/if (name.equals("BUILTIN")) {
            /*_I*/{_prof.prof.cnt[5193]++;/*I_*/iconName = sValue;/*_I*/}/*I_*/ }/*_I*/}/*I_*/ 
      }/*_I*/}/*I_*/
  }

   private void setNodeAttribute(String name, String sValue, NodeAdapter node) {/*_I*/_prof.prof.cnt[5194]++;/*I_*/
     /*_I*/{_prof.prof.cnt[5195]++;/*I_*/if (name.equals(XML_NODE_TEXT)) {
	    /*_I*/{_prof.prof.cnt[5196]++;/*I_*/node.setUserObject(sValue);/*_I*/}/*I_*/ }
	 else /*_I*/{_prof.prof.cnt[5197]++;/*I_*/if (name.equals(XML_NODE_ENCRYPTED_CONTENT)) {
	     // we change the node implementation to EncryptedMindMapNode.
	     /*_I*/{_prof.prof.cnt[5198]++;/*I_*/node = createNodeGivenClassName(EncryptedMindMapNode.class.getName());/*_I*/}/*I_*/
	     /*_I*/{_prof.prof.cnt[5199]++;/*I_*/node.setAdditionalInfo(sValue);/*_I*/}/*I_*/ 
	 } else /*_I*/{_prof.prof.cnt[5200]++;/*I_*/if (name.equals(XML_NODE_HISTORY_CREATED_AT)) {
	     /*_I*/{_prof.prof.cnt[5201]++;/*I_*/if(node.getHistoryInformation()==null) {
	     	/*_I*/{_prof.prof.cnt[5202]++;/*I_*/node.setHistoryInformation(new HistoryInformation());/*_I*/}/*I_*/
	     }/*_I*/}/*I_*/
	     /*_I*/{_prof.prof.cnt[5203]++;/*I_*/node.getHistoryInformation().setCreatedAt(Tools.xmlToDate(sValue));/*_I*/}/*I_*/
	 }
	 else /*_I*/{_prof.prof.cnt[5204]++;/*I_*/if (name.equals(XML_NODE_HISTORY_LAST_MODIFIED_AT)) {
	     /*_I*/{_prof.prof.cnt[5205]++;/*I_*/if(node.getHistoryInformation()==null) {
	     	/*_I*/{_prof.prof.cnt[5206]++;/*I_*/node.setHistoryInformation(new HistoryInformation());/*_I*/}/*I_*/
	     }/*_I*/}/*I_*/
	     /*_I*/{_prof.prof.cnt[5207]++;/*I_*/node.getHistoryInformation().setLastModifiedAt(Tools.xmlToDate(sValue));/*_I*/}/*I_*/
	 }
	 else /*_I*/{_prof.prof.cnt[5208]++;/*I_*/if (name.equals("FOLDED")) {
	    /*_I*/{_prof.prof.cnt[5209]++;/*I_*/if (sValue.equals("true")) {
	       /*_I*/{_prof.prof.cnt[5210]++;/*I_*/node.setFolded(true);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
	 else /*_I*/{_prof.prof.cnt[5211]++;/*I_*/if (name.equals("POSITION")) {
	     // fc, 17.12.2003: Remove the left/right bug.
	     /*_I*/{_prof.prof.cnt[5212]++;/*I_*/node.setLeft(sValue.equals("left"));/*_I*/}/*I_*/ }
	 else /*_I*/{_prof.prof.cnt[5213]++;/*I_*/if (name.equals("COLOR")) {
	    /*_I*/{_prof.prof.cnt[5214]++;/*I_*/if (sValue.length() == 7) {
	       /*_I*/{_prof.prof.cnt[5215]++;/*I_*/node.setColor(Tools.xmlToColor(sValue));/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
	 else /*_I*/{_prof.prof.cnt[5216]++;/*I_*/if (name.equals("BACKGROUND_COLOR")) {
	    /*_I*/{_prof.prof.cnt[5217]++;/*I_*/if (sValue.length() == 7) {
	       /*_I*/{_prof.prof.cnt[5218]++;/*I_*/node.setBackgroundColor(Tools.xmlToColor(sValue));/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
	 else /*_I*/{_prof.prof.cnt[5219]++;/*I_*/if (name.equals("LINK")) {
	    /*_I*/{_prof.prof.cnt[5220]++;/*I_*/node.setLink(sValue);/*_I*/}/*I_*/ }
	 else /*_I*/{_prof.prof.cnt[5221]++;/*I_*/if (name.equals("STYLE")) {
	    /*_I*/{_prof.prof.cnt[5222]++;/*I_*/node.setStyle(sValue);/*_I*/}/*I_*/ }
	 else /*_I*/{_prof.prof.cnt[5223]++;/*I_*/if (name.equals("ID")) {
	     // do not set label but annotate in list:
	     //System.out.println("(sValue, node) = " + sValue + ", "+  node);
	     /*_I*/{_prof.prof.cnt[5224]++;/*I_*/IDToTarget.put(sValue, node);/*_I*/}/*I_*/
	 }
	 else /*_I*/{_prof.prof.cnt[5225]++;/*I_*/if (name.equals("VSHIFT")) {
	 	/*_I*/{_prof.prof.cnt[5226]++;/*I_*/node.setShiftY(Integer.parseInt(sValue));/*_I*/}/*I_*/
	 }
	 else /*_I*/{_prof.prof.cnt[5227]++;/*I_*/if (name.equals("VGAP")) {
	   	/*_I*/{_prof.prof.cnt[5228]++;/*I_*/node.setVGap(Integer.parseInt(sValue));/*_I*/}/*I_*/
	 }
	 else /*_I*/{_prof.prof.cnt[5229]++;/*I_*/if (name.equals("HGAP")) {
	   	/*_I*/{_prof.prof.cnt[5230]++;/*I_*/node.setHGap(Integer.parseInt(sValue));/*_I*/}/*I_*/
	 }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
}

    /**
     * @param className
     */
    private NodeAdapter createNodeGivenClassName(String className) {/*_I*/_prof.prof.cnt[5231]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5232]++;/*I_*/userObject = createNodeAdapter(frame, className);/*_I*/}/*I_*/
        // reactivate all settings from nodeAttributes:
        /*_I*/{_prof.prof.cnt[5233]++;/*I_*/for (Iterator i = nodeAttributes.keySet().iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[5234]++;/*I_*/String key = (String) i.next();
            //to avoid self reference:
            /*_I*/{_prof.prof.cnt[5235]++;/*I_*/setNodeAttribute(key, (String) nodeAttributes.get(key),
                    (NodeAdapter) userObject);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5236]++;/*I_*/return (NodeAdapter) userObject;/*_I*/}/*I_*/
    }

    protected void completeElement() {/*_I*/_prof.prof.cnt[5237]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5238]++;/*I_*/if (getName().equals("font")) {
         /*_I*/{_prof.prof.cnt[5239]++;/*I_*/userObject =  frame.getController().getFontThroughMap
            (new Font(fontName, fontStyle, fontSize));/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /* icons */
            /*_I*/{_prof.prof.cnt[5240]++;/*I_*/if (getName().equals("icon")) {
         /*_I*/{_prof.prof.cnt[5241]++;/*I_*/userObject =  MindIcon.factory(iconName);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
   }

    /** Completes the links within the map. They are registered in the registry.*/
    public void processUnfinishedLinks(MindMapLinkRegistry registry) {/*_I*/_prof.prof.cnt[5242]++;/*I_*/
        // add labels to the nodes:
        /*_I*/{_prof.prof.cnt[5243]++;/*I_*/setIDs(IDToTarget, registry);/*_I*/}/*I_*/
        // complete arrow links with right labels:
        /*_I*/{_prof.prof.cnt[5244]++;/*I_*/for(int i = 0; i < ArrowLinkAdapters.size(); ++i) {
            /*_I*/_prof.prof.cnt[5245]++;/*I_*/ArrowLinkAdapter arrowLink = (ArrowLinkAdapter) ArrowLinkAdapters.get(i);
            /*_I*/_prof.prof.cnt[5246]++;/*I_*/String oldID = arrowLink.getDestinationLabel();
            /*_I*/_prof.prof.cnt[5247]++;/*I_*/NodeAdapter target = null;
            /*_I*/_prof.prof.cnt[5248]++;/*I_*/String newID = null;
            // find oldID in target list:
            /*_I*/{_prof.prof.cnt[5249]++;/*I_*/if(IDToTarget.containsKey(oldID)) {
                // link present in the xml text
                /*_I*/{_prof.prof.cnt[5250]++;/*I_*/target = (NodeAdapter) IDToTarget.get(oldID);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5251]++;/*I_*/newID = registry.getLabel(target);/*_I*/}/*I_*/
            } else /*_I*/{_prof.prof.cnt[5252]++;/*I_*/if(registry.getTargetForID(oldID) != null) {
                // link is already present in the map (paste).
                /*_I*/{_prof.prof.cnt[5253]++;/*I_*/target = (NodeAdapter) registry.getTargetForID(oldID);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5254]++;/*I_*/if(target == null) {
                    // link target is in nowhere-land
                    /*_I*/{_prof.prof.cnt[5255]++;/*I_*/System.err.println("Cannot find the label " + oldID + " in the map. The link "+arrowLink+" is not restored.");/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[5256]++;/*I_*/continue;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5257]++;/*I_*/newID = registry.getLabel(target);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5258]++;/*I_*/if( ! newID.equals(oldID) ) {
                    /*_I*/{_prof.prof.cnt[5259]++;/*I_*/System.err.println("Servere internal error. Looked for id " + oldID + " but found "+newID + " in the node " + target+".");/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[5260]++;/*I_*/continue;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            } else {                
                // link target is in nowhere-land
                /*_I*/{_prof.prof.cnt[5261]++;/*I_*/System.err.println("Cannot find the label " + oldID + " in the map. The link "+arrowLink+" is not restored.");/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5262]++;/*I_*/continue;/*_I*/}/*I_*/
            }/*_I*/}/*I_*//*_I*/}/*I_*/
            // set the new ID:
            /*_I*/{_prof.prof.cnt[5263]++;/*I_*/arrowLink.setDestinationLabel(newID);/*_I*/}/*I_*/
            // set the target:
            /*_I*/{_prof.prof.cnt[5264]++;/*I_*/arrowLink.setTarget(target);/*_I*/}/*I_*/
            // add the arrowLink:
            //System.out.println("Node = " + target+ ", oldID="+oldID+", newID="+newID);
            /*_I*/{_prof.prof.cnt[5265]++;/*I_*/registry.registerLink(arrowLink);/*_I*/}/*I_*/

        }/*_I*/}/*I_*/
    }


    /**Recursive method to set the ids of the nodes.*/
    private void setIDs(HashMap IDToTarget, MindMapLinkRegistry registry) {/*_I*/_prof.prof.cnt[5266]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5267]++;/*I_*/for(Iterator i = IDToTarget.keySet().iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[5268]++;/*I_*/String key = (String) i.next();
            /*_I*/_prof.prof.cnt[5269]++;/*I_*/NodeAdapter target = (NodeAdapter) IDToTarget.get(key);
            /*_I*/_prof.prof.cnt[5270]++;/*I_*/MindMapLinkRegistry.ID_Registered newState = registry.registerLinkTarget(target, key /* Proposed name for the target, is changed by the registry, if already present.*/);
            /*_I*/_prof.prof.cnt[5271]++;/*I_*/String newId = newState.getID();
            // and in the cutted case: 
            // search for links to this ids that have been cutted earlier:
            /*_I*/_prof.prof.cnt[5272]++;/*I_*/Vector cuttedLinks = registry.getCuttedNode(key /* old target id*/);
            /*_I*/{_prof.prof.cnt[5273]++;/*I_*/for(int j=0; j < cuttedLinks.size(); ++j) {
                /*_I*/_prof.prof.cnt[5274]++;/*I_*/ArrowLinkAdapter link = (ArrowLinkAdapter) cuttedLinks.get(j);
                // repair link
                /*_I*/{_prof.prof.cnt[5275]++;/*I_*/link.setTarget(target);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5276]++;/*I_*/link.setDestinationLabel(newId);/*_I*/}/*I_*/
                // and set it:
                /*_I*/{_prof.prof.cnt[5277]++;/*I_*/registry.registerLink(link);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }


}
