/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: MindMapController.java,v 1.35.14.13 2005/06/16 19:27:05 christianfoltin Exp $*/

package freemind.modes.mindmapmode;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileFilter;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import freemind.common.JaxbTools;
import freemind.controller.MenuBar;
import freemind.controller.StructuredMenuHolder;
import freemind.controller.actions.generated.instance.MenuActionBase;
import freemind.controller.actions.generated.instance.MenuCategoryBase;
import freemind.controller.actions.generated.instance.MenuCheckedAction;
import freemind.controller.actions.generated.instance.MenuSeparator;
import freemind.controller.actions.generated.instance.MenuStructure;
import freemind.controller.actions.generated.instance.MenuSubmenu;
import freemind.extensions.HookFactory;
import freemind.extensions.HookRegistration;
import freemind.extensions.HookFactory.RegistrationContainer;
import freemind.main.Tools;
import freemind.modes.ControllerAdapter;
import freemind.modes.MapAdapter;
import freemind.modes.MindIcon;
import freemind.modes.MindMap;
import freemind.modes.MindMapNode;
import freemind.modes.Mode;
import freemind.modes.ModeController;
import freemind.modes.NodeAdapter;
import freemind.modes.actions.ChangeArrowsInArrowLinkAction;
import freemind.modes.actions.ColorArrowLinkAction;
import freemind.modes.actions.FindAction;
import freemind.modes.actions.GotoLinkNodeAction;
import freemind.modes.actions.IconAction;
import freemind.modes.actions.ImportExplorerFavoritesAction;
import freemind.modes.actions.ImportFolderStructureAction;
import freemind.modes.actions.NewMapAction;
import freemind.modes.actions.NewPreviousSiblingAction;
import freemind.modes.actions.NewSiblingAction;
import freemind.modes.actions.NodeGeneralAction;
import freemind.modes.actions.NodeHookAction;
import freemind.modes.actions.RemoveArrowLinkAction;
import freemind.modes.actions.SingleNodeOperation;





public class MindMapController extends ControllerAdapter {

	private static Logger logger;
	private Vector hookActions;
	/** Stores the menu items belonging to the given action. */
	private HashMap actionToMenuPositions;
	//    Mode mode;
    private MindMapPopupMenu popupmenu;
    //private JToolBar toolbar;
    private MindMapToolBar toolbar;
    private boolean addAsChildMode = false;
   public Action newMap = new NewMapAction(this, this);
   public Action open = new OpenAction(this);
   public Action save = new SaveAction(this);
   public Action saveAs = new SaveAsAction(this);
   public Action exportToHTML = new ExportToHTMLAction(this);
   public Action exportBranchToHTML = new ExportBranchToHTMLAction(this);

   public Action editLong = new EditLongAction();
   public Action newSibling = new NewSiblingAction(this);
   public Action newPreviousSibling = new NewPreviousSiblingAction(this);
   public Action setLinkByFileChooser = new SetLinkByFileChooserAction();
   public Action setImageByFileChooser = new SetImageByFileChooserAction();
   public Action followLink = new FollowLinkAction();
   public Action exportBranch = new ExportBranchAction();
   public Action importBranch = new ImportBranchAction();
   public Action importLinkedBranch = new ImportLinkedBranchAction();
   public Action importLinkedBranchWithoutRoot = new ImportLinkedBranchWithoutRootAction();


    public Action increaseNodeFont = new NodeGeneralAction (this, "increase_node_font_size", null,
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[8506]++;/*I_*/
           /*_I*/{_prof.prof.cnt[8507]++;/*I_*/increaseFontSize(node, 1);/*_I*/}/*I_*/
    }});
    public Action decreaseNodeFont = new NodeGeneralAction (this, "decrease_node_font_size", null,
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[8508]++;/*I_*/
           /*_I*/{_prof.prof.cnt[8509]++;/*I_*/increaseFontSize(node, -1);/*_I*/}/*I_*/
    }});

    // Extension Actions
    public Vector iconActions = new Vector(); //fc

    FileFilter filefilter = new MindMapFilter();

    private MenuStructure mMenuStructure;
    private List pRegistrations;

    public MindMapController(Mode mode) {
	super(mode);/*_I*/_prof.prof.cnt[8510]++;/*I_*//*_I*/_prof.prof.cnt[8511]++;/*I_*/
	/*_I*/{_prof.prof.cnt[8512]++;/*I_*/if(logger == null) {
		/*_I*/{_prof.prof.cnt[8513]++;/*I_*/logger = getFrame().getLogger(this.getClass().getName());/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[8514]++;/*I_*/logger.info("createIconActions");/*_I*/}/*I_*/
        // icon actions:
        /*_I*/{_prof.prof.cnt[8515]++;/*I_*/createIconActions();/*_I*/}/*I_*/
    	/*_I*/{_prof.prof.cnt[8516]++;/*I_*/logger.info("createNodeHookActions");/*_I*/}/*I_*/
        //node hook actions:
        /*_I*/{_prof.prof.cnt[8517]++;/*I_*/createNodeHookActions();/*_I*/}/*I_*/

    	/*_I*/{_prof.prof.cnt[8518]++;/*I_*/logger.info("mindmap_menus");/*_I*/}/*I_*/
        // load menus:
        /*_I*/{_prof.prof.cnt[8519]++;/*I_*/try {
            /*_I*/_prof.prof.cnt[8520]++;/*I_*/InputStream in;
            /*_I*/{_prof.prof.cnt[8521]++;/*I_*/in = this.getFrame().getResource("mindmap_menus.xml").openStream();/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8522]++;/*I_*/mMenuStructure = updateMenusFromXml(in);/*_I*/}/*I_*/
        } catch (IOException e) {
            // TODO Auto-generated catch block
            /*_I*/{_prof.prof.cnt[8523]++;/*I_*/e.printStackTrace();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/

    	/*_I*/{_prof.prof.cnt[8524]++;/*I_*/logger.info("MindMapPopupMenu");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8525]++;/*I_*/popupmenu = new MindMapPopupMenu(this);/*_I*/}/*I_*/
    	/*_I*/{_prof.prof.cnt[8526]++;/*I_*/logger.info("MindMapToolBar");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8527]++;/*I_*/toolbar = new MindMapToolBar(this);/*_I*/}/*I_*/
    	/*_I*/{_prof.prof.cnt[8528]++;/*I_*/logger.info("setAllActions");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8529]++;/*I_*/setAllActions(false);/*_I*/}/*I_*/

        // addAsChildMode (use old model of handling CtrN) (PN)
        /*_I*/{_prof.prof.cnt[8530]++;/*I_*/addAsChildMode =
            Tools.safeEquals(getFrame().getProperty("add_as_child"), "true");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8531]++;/*I_*/pRegistrations = new Vector();/*_I*/}/*I_*/
    }

    /** This method is called after and before a change of the map module.
     * Use it to perform the actions that cannot be performed at creation time.
     * 
     */
    public void startupController() {/*_I*/_prof.prof.cnt[8532]++;/*I_*/
		/*_I*/{_prof.prof.cnt[8533]++;/*I_*/super.startupController();/*_I*/}/*I_*/
		/*_I*/_prof.prof.cnt[8534]++;/*I_*/HookFactory hookFactory = getFrame().getHookFactory();
		/*_I*/_prof.prof.cnt[8535]++;/*I_*/List pluginRegistratios = hookFactory.getRegistrations(this.getClass());
		/*_I*/{_prof.prof.cnt[8536]++;/*I_*/logger.info("mScheduledActions are executed: "
				+ pluginRegistratios.size());/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[8537]++;/*I_*/for (Iterator i = pluginRegistratios.iterator(); i.hasNext();) {
			// call constructor:
			/*_I*/{_prof.prof.cnt[8538]++;/*I_*/try {
				/*_I*/_prof.prof.cnt[8539]++;/*I_*/HookFactory.RegistrationContainer container = (RegistrationContainer) i
						.next();
				/*_I*/_prof.prof.cnt[8540]++;/*I_*/Class registrationClass = container.hookRegistrationClass;
				/*_I*/_prof.prof.cnt[8541]++;/*I_*/Constructor hookConstructor = registrationClass
						.getConstructor(new Class[] { ModeController.class,
								MindMap.class });
				/*_I*/_prof.prof.cnt[8542]++;/*I_*/HookRegistration registrationInstance = (HookRegistration) hookConstructor
						.newInstance(new Object[] { this, getMap() });
				// register the instance to enable basePlugins.
				/*_I*/{_prof.prof.cnt[8543]++;/*I_*/hookFactory.registerRegistrationContainer(container,
						registrationInstance);/*_I*/}/*I_*/
				/*_I*/{_prof.prof.cnt[8544]++;/*I_*/registrationInstance.register();/*_I*/}/*I_*/
				/*_I*/{_prof.prof.cnt[8545]++;/*I_*/pRegistrations.add(registrationInstance);/*_I*/}/*I_*/
			} catch (Exception e) {
				/*_I*/{_prof.prof.cnt[8546]++;/*I_*/e.printStackTrace();/*_I*/}/*I_*/
			}/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}


    public void shutdownController() {/*_I*/_prof.prof.cnt[8547]++;/*I_*/
        /*_I*/{_prof.prof.cnt[8548]++;/*I_*/super.shutdownController();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8549]++;/*I_*/for (Iterator i = pRegistrations.iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[8550]++;/*I_*/HookRegistration registrationInstance = (HookRegistration) i.next();
            /*_I*/{_prof.prof.cnt[8551]++;/*I_*/registrationInstance.deRegister();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }
    
	public MapAdapter newModel() {/*_I*/_prof.prof.cnt[8552]++;/*I_*/
       /*_I*/{_prof.prof.cnt[8553]++;/*I_*/return new MindMapMapModel(getFrame());/*_I*/}/*I_*/ }

    private void createIconActions() {/*_I*/_prof.prof.cnt[8554]++;/*I_*/
        /*_I*/_prof.prof.cnt[8555]++;/*I_*/Vector iconNames = MindIcon.getAllIconNames();
        /*_I*/{_prof.prof.cnt[8556]++;/*I_*/for ( int i = 0 ; i < iconNames.size(); ++i ) {
            /*_I*/_prof.prof.cnt[8557]++;/*I_*/String iconName = ((String) iconNames.get(i));
            /*_I*/_prof.prof.cnt[8558]++;/*I_*/MindIcon myIcon     = MindIcon.factory(iconName);
            /*_I*/_prof.prof.cnt[8559]++;/*I_*/IconAction myAction = new IconAction(this, myIcon,removeLastIconAction);
            /*_I*/{_prof.prof.cnt[8560]++;/*I_*/iconActions.add(myAction);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

	/**
	 * 
	 */
	private void createNodeHookActions() {/*_I*/_prof.prof.cnt[8561]++;/*I_*/
		/*_I*/{_prof.prof.cnt[8562]++;/*I_*/actionToMenuPositions = new HashMap();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8563]++;/*I_*/if (hookActions == null) {
            /*_I*/{_prof.prof.cnt[8564]++;/*I_*/hookActions = new Vector();/*_I*/}/*I_*/
            // HOOK TEST
            /*_I*/_prof.prof.cnt[8565]++;/*I_*/HookFactory factory = getFrame().getHookFactory();
            /*_I*/_prof.prof.cnt[8566]++;/*I_*/List list = factory.getPossibleNodeHooks(this.getClass());
            /*_I*/{_prof.prof.cnt[8567]++;/*I_*/for (Iterator i = list.iterator(); i.hasNext();) {
                /*_I*/_prof.prof.cnt[8568]++;/*I_*/String desc = (String) i.next();
                // create hook action. 
                /*_I*/_prof.prof.cnt[8569]++;/*I_*/NodeHookAction action = new NodeHookAction(desc, this);
                /*_I*/{_prof.prof.cnt[8570]++;/*I_*/factory.decorateAction(desc, action);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[8571]++;/*I_*/actionToMenuPositions.put(action, factory.getHookMenuPositions(desc));/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[8572]++;/*I_*/hookActions.add(action);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[8573]++;/*I_*/List hooks =
                factory.getPossibleModeControllerHooks(this.getClass());
            /*_I*/{_prof.prof.cnt[8574]++;/*I_*/for (Iterator i = hooks.iterator(); i.hasNext();) {
                /*_I*/_prof.prof.cnt[8575]++;/*I_*/String desc = (String) i.next();
                /*_I*/_prof.prof.cnt[8576]++;/*I_*/ModeControllerHookAction action =
                    new ModeControllerHookAction(desc, this);
                /*_I*/{_prof.prof.cnt[8577]++;/*I_*/factory.decorateAction(desc, action);/*_I*/}/*I_*/
			   /*_I*/{_prof.prof.cnt[8578]++;/*I_*/actionToMenuPositions.put(action, factory.getHookMenuPositions(desc));/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[8579]++;/*I_*/hookActions.add(action);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            //HOOK TEST END       
	    }/*_I*/}/*I_*/
	}


    public FileFilter getFileFilter() {/*_I*/_prof.prof.cnt[8580]++;/*I_*/
       /*_I*/{_prof.prof.cnt[8581]++;/*I_*/return filefilter;/*_I*/}/*I_*/ }

    public void nodeChanged(MindMapNode n) {/*_I*/_prof.prof.cnt[8582]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[8583]++;/*I_*/super.nodeChanged(n);/*_I*/}/*I_*/
    	// only for the selected node (fc, 2.5.2004)
		/*_I*/{_prof.prof.cnt[8584]++;/*I_*/if (n == getSelected()) {
			/*_I*/{_prof.prof.cnt[8585]++;/*I_*/toolbar.selectFontSize(n.getFontSize());/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[8586]++;/*I_*/toolbar.selectFontName(n.getFontFamilyName());/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	 }

    public void anotherNodeSelected(MindMapNode n) {/*_I*/_prof.prof.cnt[8587]++;/*I_*/
       /*_I*/{_prof.prof.cnt[8588]++;/*I_*/super.anotherNodeSelected(n);/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[8589]++;/*I_*/toolbar.selectFontSize(((NodeAdapter)n).getFontSize());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[8590]++;/*I_*/toolbar.selectFontName(((NodeAdapter)n).getFontFamilyName());/*_I*/}/*I_*/ }

    // fc, 14.12.2004: changes, such that different models can be used:
    private NewNodeCreator myNewNodeCreator = null;
    
    public interface NewNodeCreator {
        MindMapNode createNode(Object userObject);
    }

    public class DefaultMindMapNodeCreator implements NewNodeCreator {

        public MindMapNode createNode(Object userObject) {/*_I*/_prof.prof.cnt[8591]++;/*I_*/
            /*_I*/{_prof.prof.cnt[8592]++;/*I_*/return new MindMapNodeModel(userObject, getFrame());/*_I*/}/*I_*/
        }
        
    }
    
    public void setNewNodeCreator(NewNodeCreator creator) {/*_I*/_prof.prof.cnt[8593]++;/*I_*/
        /*_I*/{_prof.prof.cnt[8594]++;/*I_*/myNewNodeCreator = creator;/*_I*/}/*I_*/
    }
    
    public MindMapNode newNode(Object userObject) {/*_I*/_prof.prof.cnt[8595]++;/*I_*/
        // singleton default:
        /*_I*/{_prof.prof.cnt[8596]++;/*I_*/if (myNewNodeCreator == null) {
            /*_I*/{_prof.prof.cnt[8597]++;/*I_*/myNewNodeCreator = new DefaultMindMapNodeCreator();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8598]++;/*I_*/return myNewNodeCreator.createNode(userObject);/*_I*/}/*I_*/
    }

    // fc, 14.12.2004: end "different models" change
    
	    //get/set methods



	/**
	 * @param holder
	 */
    public void updateMenus(StructuredMenuHolder holder) {/*_I*/_prof.prof.cnt[8599]++;/*I_*/

		/*_I*/{_prof.prof.cnt[8600]++;/*I_*/processMenuCategory(holder, mMenuStructure.getMenuCategory(), "");/*_I*/}/*I_*/ /*MenuBar.MENU_BAR_PREFIX*/
		// add hook actions to this holder.
		// hooks, fc, 1.3.2004:
		/*_I*/{_prof.prof.cnt[8601]++;/*I_*/for (int i = 0; i < hookActions.size(); ++i) {
			/*_I*/_prof.prof.cnt[8602]++;/*I_*/Action hookAction = (Action) hookActions.get(i);
			/*_I*/_prof.prof.cnt[8603]++;/*I_*/List positions = (List) actionToMenuPositions.get(hookAction);
			/*_I*/{_prof.prof.cnt[8604]++;/*I_*/for (Iterator j = positions.iterator(); j.hasNext();) {
                /*_I*/_prof.prof.cnt[8605]++;/*I_*/String pos = (String) j.next();
                /*_I*/{_prof.prof.cnt[8606]++;/*I_*/holder.addAction(hookAction, pos);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		// update popup and toolbar:
		/*_I*/{_prof.prof.cnt[8607]++;/*I_*/popupmenu.update(holder);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[8608]++;/*I_*/toolbar.update(holder);/*_I*/}/*I_*/
        
//		editMenu.add(getExtensionMenu());
		/*_I*/_prof.prof.cnt[8609]++;/*I_*/String formatMenuString = MenuBar.FORMAT_MENU;
        /*_I*/{_prof.prof.cnt[8610]++;/*I_*/createPatternSubMenu(holder, formatMenuString);/*_I*/}/*I_*/


//        editMenu.add(getIconMenu());
		/*_I*/{_prof.prof.cnt[8611]++;/*I_*/addIconsToMenu(holder, MenuBar.INSERT_MENU + "icons");/*_I*/}/*I_*/

    }

    public void addIconsToMenu(StructuredMenuHolder holder, String iconMenuString) {/*_I*/_prof.prof.cnt[8612]++;/*I_*/
		/*_I*/_prof.prof.cnt[8613]++;/*I_*/JMenu iconMenu = holder.addMenu(new JMenu(getText("icon_menu")), iconMenuString+"/.") ;
		/*_I*/{_prof.prof.cnt[8614]++;/*I_*/holder.addAction(removeLastIconAction, iconMenuString+"/removeLastIcon");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[8615]++;/*I_*/holder.addAction(removeAllIconsAction, iconMenuString+"/removeAllIcons");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[8616]++;/*I_*/holder.addSeparator(iconMenuString);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[8617]++;/*I_*/for (int i=0; i<iconActions.size(); ++i) {          
			   /*_I*/_prof.prof.cnt[8618]++;/*I_*/JMenuItem item = holder.addAction((Action) iconActions.get(i), iconMenuString+"/"+i);
		}/*_I*/}/*I_*/
	}

	/**
     * @param holder
     * @param formatMenuString
     */
    public JMenu createPatternSubMenu(StructuredMenuHolder holder, String formatMenuString) {/*_I*/_prof.prof.cnt[8619]++;/*I_*/
        /*_I*/_prof.prof.cnt[8620]++;/*I_*/JMenu extensionMenu = holder.addMenu(new JMenu(getText("extension_menu")), formatMenuString+"patterns/.");
        /*_I*/{_prof.prof.cnt[8621]++;/*I_*/for (int i = 0; i < patterns.length; ++i) {
            /*_I*/_prof.prof.cnt[8622]++;/*I_*/JMenuItem item =
                holder.addAction(
                    patterns[i],
                    formatMenuString + "patterns/" + i);
            /*_I*/{_prof.prof.cnt[8623]++;/*I_*/item.setAccelerator(
                KeyStroke.getKeyStroke(
                    getFrame().getProperty(
                        "keystroke_apply_pattern_" + (i + 1))));/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8624]++;/*I_*/return extensionMenu;/*_I*/}/*I_*/
    }

    public MenuStructure updateMenusFromXml(InputStream in) {/*_I*/_prof.prof.cnt[8625]++;/*I_*/
        // get from resources:
        /*_I*/{_prof.prof.cnt[8626]++;/*I_*/try {
        	/*_I*/_prof.prof.cnt[8627]++;/*I_*/Unmarshaller unmarshaller = JaxbTools.getInstance().createUnmarshaller();
        	/*_I*/{_prof.prof.cnt[8628]++;/*I_*/unmarshaller.setValidating(true);/*_I*/}/*I_*/
        	/*_I*/_prof.prof.cnt[8629]++;/*I_*/MenuStructure menus = (MenuStructure) unmarshaller.unmarshal(in);
        	/*_I*/{_prof.prof.cnt[8630]++;/*I_*/return menus;/*_I*/}/*I_*/
		} catch (JAXBException e) {
        	/*_I*/{_prof.prof.cnt[8631]++;/*I_*/logger.severe(e.getCause() + e.getLocalizedMessage() + e.getMessage());/*_I*/}/*I_*/
        	/*_I*/{_prof.prof.cnt[8632]++;/*I_*/e.printStackTrace();/*_I*/}/*I_*/
        	/*_I*/{_prof.prof.cnt[8633]++;/*I_*/throw new IllegalArgumentException("Menu structure could not be read.");/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }


    /**
     * @param type
     */
    public void processMenuCategory(StructuredMenuHolder holder, List list, String category) {/*_I*/_prof.prof.cnt[8634]++;/*I_*/
		/*_I*/_prof.prof.cnt[8635]++;/*I_*/String categoryCopy = category;
    	/*_I*/{_prof.prof.cnt[8636]++;/*I_*/for (Iterator i = list.iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[8637]++;/*I_*/Object obj = (Object) i.next();
            /*_I*/{_prof.prof.cnt[8638]++;/*I_*/if(obj instanceof MenuCategoryBase) {
				/*_I*/_prof.prof.cnt[8639]++;/*I_*/MenuCategoryBase cat = (MenuCategoryBase) obj;
            	/*_I*/_prof.prof.cnt[8640]++;/*I_*/String newCategory = categoryCopy+"/"+cat.getName();
				/*_I*/{_prof.prof.cnt[8641]++;/*I_*/holder.addCategory(newCategory);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[8642]++;/*I_*/if(cat instanceof MenuSubmenu) {
                	/*_I*/_prof.prof.cnt[8643]++;/*I_*/MenuSubmenu submenu = (MenuSubmenu) cat;
                	/*_I*/{_prof.prof.cnt[8644]++;/*I_*/holder.addMenu(new JMenu(getText(submenu.getNameRef())), newCategory+"/.");/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            	/*_I*/{_prof.prof.cnt[8645]++;/*I_*/processMenuCategory(holder, cat.getMenuCategoryOrMenuSubmenuOrMenuAction(), newCategory);/*_I*/}/*I_*/
            } else /*_I*/{_prof.prof.cnt[8646]++;/*I_*/if( obj instanceof MenuActionBase ) {
				/*_I*/_prof.prof.cnt[8647]++;/*I_*/MenuActionBase action = (MenuActionBase) obj;            	
				/*_I*/_prof.prof.cnt[8648]++;/*I_*/String field = action.getField();
				/*_I*/_prof.prof.cnt[8649]++;/*I_*/String name = action.getName();
				/*_I*/{_prof.prof.cnt[8650]++;/*I_*/if(name == null) {
					/*_I*/{_prof.prof.cnt[8651]++;/*I_*/name = field;/*_I*/}/*I_*/
				}/*_I*/}/*I_*/
				/*_I*/_prof.prof.cnt[8652]++;/*I_*/String keystroke = action.getKeyRef();
				/*_I*/{_prof.prof.cnt[8653]++;/*I_*/try {
					/*_I*/_prof.prof.cnt[8654]++;/*I_*/Action theAction = (Action) this.getClass().getField(field).get(this);
					/*_I*/_prof.prof.cnt[8655]++;/*I_*/String theCategory = categoryCopy+"/"+name;
					/*_I*/{_prof.prof.cnt[8656]++;/*I_*/if (obj instanceof MenuCheckedAction) {
						/*_I*/{_prof.prof.cnt[8657]++;/*I_*/addCheckBox(holder, theCategory, theAction, keystroke);/*_I*/}/*I_*/
					} else {
						/*_I*/{_prof.prof.cnt[8658]++;/*I_*/add(holder, theCategory, theAction, keystroke);/*_I*/}/*I_*/
					}/*_I*/}/*I_*/
				} catch (Exception e1) {
					/*_I*/{_prof.prof.cnt[8659]++;/*I_*/e1.printStackTrace();/*_I*/}/*I_*/
				}/*_I*/}/*I_*/
            } else /*_I*/{_prof.prof.cnt[8660]++;/*I_*/if (obj instanceof MenuSeparator) {
            	/*_I*/{_prof.prof.cnt[8661]++;/*I_*/holder.addSeparator(categoryCopy);/*_I*/}/*I_*/
            }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/ /* else exception */
         }/*_I*/}/*I_*/
    }

    public JPopupMenu getPopupMenu() {/*_I*/_prof.prof.cnt[8662]++;/*I_*/
        /*_I*/{_prof.prof.cnt[8663]++;/*I_*/return popupmenu;/*_I*/}/*I_*/
    }

    /** Link implementation: If this is a link, we want to make a popup with at least removelink available.*/
    public JPopupMenu getPopupForModel(java.lang.Object obj) {/*_I*/_prof.prof.cnt[8664]++;/*I_*/
        /*_I*/{_prof.prof.cnt[8665]++;/*I_*/if( obj instanceof MindMapArrowLinkModel) {
            // yes, this is a link.
            /*_I*/_prof.prof.cnt[8666]++;/*I_*/MindMapArrowLinkModel link = (MindMapArrowLinkModel) obj;
            /*_I*/_prof.prof.cnt[8667]++;/*I_*/JPopupMenu arrowLinkPopup = new JPopupMenu();
            // block the screen while showing popup.
            /*_I*/{_prof.prof.cnt[8668]++;/*I_*/arrowLinkPopup.addPopupMenuListener( this.popupListenerSingleton );/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8669]++;/*I_*/removeArrowLinkAction.setArrowLink(link);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8670]++;/*I_*/arrowLinkPopup.add(new RemoveArrowLinkAction(this, link));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8671]++;/*I_*/arrowLinkPopup.add(new ColorArrowLinkAction(this, link));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8672]++;/*I_*/arrowLinkPopup.addSeparator();/*_I*/}/*I_*/
            /* The arrow state as radio buttons: */
            /*_I*/_prof.prof.cnt[8673]++;/*I_*/JRadioButtonMenuItem itemnn = new JRadioButtonMenuItem( new ChangeArrowsInArrowLinkAction(this, "none", "images/arrow-mode-none.gif",link, false, false) );
            /*_I*/_prof.prof.cnt[8674]++;/*I_*/JRadioButtonMenuItem itemnt = new JRadioButtonMenuItem( new ChangeArrowsInArrowLinkAction(this, "forward", "images/arrow-mode-forward.gif",link, false, true) );
            /*_I*/_prof.prof.cnt[8675]++;/*I_*/JRadioButtonMenuItem itemtn = new JRadioButtonMenuItem( new ChangeArrowsInArrowLinkAction(this, "backward", "images/arrow-mode-backward.gif",link, true, false) );
            /*_I*/_prof.prof.cnt[8676]++;/*I_*/JRadioButtonMenuItem itemtt = new JRadioButtonMenuItem( new ChangeArrowsInArrowLinkAction(this, "both", "images/arrow-mode-both.gif",link, true, true) );
            /*_I*/{_prof.prof.cnt[8677]++;/*I_*/itemnn.setText(null);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8678]++;/*I_*/itemnt.setText(null);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8679]++;/*I_*/itemtn.setText(null);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8680]++;/*I_*/itemtt.setText(null);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8681]++;/*I_*/arrowLinkPopup.add( itemnn );/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8682]++;/*I_*/arrowLinkPopup.add( itemnt );/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8683]++;/*I_*/arrowLinkPopup.add( itemtn );/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8684]++;/*I_*/arrowLinkPopup.add( itemtt );/*_I*/}/*I_*/
            // select the right one:
            /*_I*/_prof.prof.cnt[8685]++;/*I_*/boolean a = !link.getStartArrow().equals("None");
            /*_I*/_prof.prof.cnt[8686]++;/*I_*/boolean b = !link.getEndArrow().equals("None");
            /*_I*/{_prof.prof.cnt[8687]++;/*I_*/itemtt.setSelected(a&&b);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8688]++;/*I_*/itemnt.setSelected(!a&&b);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8689]++;/*I_*/itemtn.setSelected(a&&!b);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8690]++;/*I_*/itemnn.setSelected(!a&&!b);/*_I*/}/*I_*/

            /*_I*/{_prof.prof.cnt[8691]++;/*I_*/arrowLinkPopup.addSeparator();/*_I*/}/*I_*/
            
            /*_I*/{_prof.prof.cnt[8692]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(this, link.getSource()));/*_I*/}/*I_*/ 
            /*_I*/{_prof.prof.cnt[8693]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(this, link.getTarget()));/*_I*/}/*I_*/ 

            /*_I*/{_prof.prof.cnt[8694]++;/*I_*/arrowLinkPopup.addSeparator();/*_I*/}/*I_*/
            // add all links from target and from source:
            /*_I*/_prof.prof.cnt[8695]++;/*I_*/HashSet NodeAlreadyVisited = new HashSet();
            /*_I*/{_prof.prof.cnt[8696]++;/*I_*/NodeAlreadyVisited.add(link.getSource());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8697]++;/*I_*/NodeAlreadyVisited.add(link.getTarget());/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[8698]++;/*I_*/Vector links = getMindMapMapModel().getLinkRegistry().getAllLinks(link.getSource());
            /*_I*/{_prof.prof.cnt[8699]++;/*I_*/links.addAll(getMindMapMapModel().getLinkRegistry().getAllLinks(link.getTarget()));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8700]++;/*I_*/for(int i = 0; i < links.size(); ++i) {
                /*_I*/_prof.prof.cnt[8701]++;/*I_*/MindMapArrowLinkModel foreign_link = (MindMapArrowLinkModel) links.get(i);
                /*_I*/{_prof.prof.cnt[8702]++;/*I_*/if(NodeAlreadyVisited.add(foreign_link.getTarget())) {
                    /*_I*/{_prof.prof.cnt[8703]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(this, foreign_link.getTarget()));/*_I*/}/*I_*/ 
                }/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[8704]++;/*I_*/if(NodeAlreadyVisited.add(foreign_link.getSource())) {
                    /*_I*/{_prof.prof.cnt[8705]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(this, foreign_link.getSource()));/*_I*/}/*I_*/ 
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8706]++;/*I_*/return arrowLinkPopup;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8707]++;/*I_*/return null;/*_I*/}/*I_*/
    }


    //convenience methods
    private MindMapMapModel getMindMapMapModel() {/*_I*/_prof.prof.cnt[8708]++;/*I_*/
	/*_I*/{_prof.prof.cnt[8709]++;/*I_*/return (MindMapMapModel)getController().getModel();/*_I*/}/*I_*/
    }

    MindMapToolBar getToolBar() {/*_I*/_prof.prof.cnt[8710]++;/*I_*/
	/*_I*/{_prof.prof.cnt[8711]++;/*I_*/return (MindMapToolBar)toolbar;/*_I*/}/*I_*/
    }

    JToolBar getLeftToolBar() {/*_I*/_prof.prof.cnt[8712]++;/*I_*/
	/*_I*/{_prof.prof.cnt[8713]++;/*I_*/return ((MindMapToolBar)toolbar).getLeftToolBar();/*_I*/}/*I_*/
    }

    /**
     * Enabled/Disabled all actions that are dependent on
     * whether there is a map open or not.
     */
    protected void setAllActions(boolean enabled) {/*_I*/_prof.prof.cnt[8714]++;/*I_*/
        /*_I*/{_prof.prof.cnt[8715]++;/*I_*/super.setAllActions(enabled);/*_I*/}/*I_*/
        // own actions
        /*_I*/{_prof.prof.cnt[8716]++;/*I_*/increaseNodeFont.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8717]++;/*I_*/decreaseNodeFont.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8718]++;/*I_*/exportBranch.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8719]++;/*I_*/exportBranchToHTML.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8720]++;/*I_*/editLong.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8721]++;/*I_*/newSibling.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8722]++;/*I_*/newPreviousSibling.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8723]++;/*I_*/setLinkByFileChooser.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8724]++;/*I_*/setImageByFileChooser.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8725]++;/*I_*/followLink.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8726]++;/*I_*/for (int i=0; i<iconActions.size(); ++i) {          
            /*_I*/{_prof.prof.cnt[8727]++;/*I_*/((Action) iconActions.get(i)).setEnabled(enabled);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8728]++;/*I_*/save.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8729]++;/*I_*/saveAs.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8730]++;/*I_*/getToolBar().setAllActions(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8731]++;/*I_*/exportBranch.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8732]++;/*I_*/exportToHTML.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8733]++;/*I_*/importBranch.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8734]++;/*I_*/importLinkedBranch.setEnabled(enabled);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[8735]++;/*I_*/importLinkedBranchWithoutRoot.setEnabled(enabled);/*_I*/}/*I_*/
        // hooks:
		/*_I*/{_prof.prof.cnt[8736]++;/*I_*/for (int i=0; i<hookActions.size(); ++i) {          
			/*_I*/{_prof.prof.cnt[8737]++;/*I_*/((Action) hookActions.get(i)).setEnabled(enabled);/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
    }




    //
    //      Actions
    //_______________________________________________________________________________



    // Export and Import
    // _________________


    // This may later be moved to ControllerAdapter. So far there is no reason for it.
    protected class ExportToHTMLAction extends AbstractAction {
	MindMapController c;
	public ExportToHTMLAction(MindMapController controller) {
	    super(getText("export_to_html"));/*_I*/_prof.prof.cnt[8738]++;/*I_*//*_I*/_prof.prof.cnt[8739]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[8740]++;/*I_*/c = controller;/*_I*/}/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[8741]++;/*I_*/
           /*_I*/_prof.prof.cnt[8742]++;/*I_*/File file = new File(c.getMindMapMapModel().getFile()+".html");
           /*_I*/{_prof.prof.cnt[8743]++;/*I_*/if (c.getMindMapMapModel().saveHTML((MindMapNodeModel)c.getMindMapMapModel().getRoot(),file)) {
              /*_I*/{_prof.prof.cnt[8744]++;/*I_*/loadURL(file.toString());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    protected class ExportBranchToHTMLAction extends AbstractAction {
	MindMapController c;
	public ExportBranchToHTMLAction(MindMapController controller) {
	    super(getText("export_branch_to_html"));/*_I*/_prof.prof.cnt[8745]++;/*I_*//*_I*/_prof.prof.cnt[8746]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[8747]++;/*I_*/c = controller;/*_I*/}/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[8748]++;/*I_*/
           /*_I*/{_prof.prof.cnt[8749]++;/*I_*/try {
              /*_I*/_prof.prof.cnt[8750]++;/*I_*/File file = File.createTempFile("tmm", ".html");
              /*_I*/{_prof.prof.cnt[8751]++;/*I_*/if (c.getMindMapMapModel().saveHTML((MindMapNodeModel)getSelected(),file)) {
                 /*_I*/{_prof.prof.cnt[8752]++;/*I_*/loadURL(file.toString());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
           catch (IOException ex) {}/*_I*/}/*I_*/}}

    private class ExportBranchAction extends AbstractAction {
	ExportBranchAction() {
	    super(getText("export_branch"));/*_I*/_prof.prof.cnt[8753]++;/*I_*//*_I*/_prof.prof.cnt[8754]++;/*I_*/
	}
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[8755]++;/*I_*/
            /*_I*/_prof.prof.cnt[8756]++;/*I_*/MindMapNodeModel node = (MindMapNodeModel) getSelected();

            //if something is wrong, abort.
            /*_I*/{_prof.prof.cnt[8757]++;/*I_*/if (getMap() == null || node == null || node.isRoot()) {
                /*_I*/{_prof.prof.cnt[8758]++;/*I_*/getFrame().err("Could not export branch.");/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[8759]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            //If the current map is not saved yet, save it first.
            /*_I*/{_prof.prof.cnt[8760]++;/*I_*/if (getMap().getFile() == null) {
                /*_I*/{_prof.prof.cnt[8761]++;/*I_*/getFrame().out("You must save the current map first!");/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[8762]++;/*I_*/save();/*_I*/}/*I_*/
            }/*_I*/}/*I_*/

            //Open FileChooser to choose in which file the exported
            //branch should be stored
	    /*_I*/_prof.prof.cnt[8763]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
            /*_I*/_prof.prof.cnt[8764]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir); 
//             if (getMap().getFile().getParentFile() != null) {
//                 chooser = new JFileChooser(getMap().getFile().getParentFile());
//             } else {
//                 chooser = new JFileChooser();
//             }
            //chooser.setLocale(currentLocale);
            /*_I*/{_prof.prof.cnt[8765]++;/*I_*/if (getFileFilter() != null) {
                /*_I*/{_prof.prof.cnt[8766]++;/*I_*/chooser.addChoosableFileFilter(getFileFilter());/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[8767]++;/*I_*/int returnVal = chooser.showSaveDialog(getSelected().getViewer());
            /*_I*/{_prof.prof.cnt[8768]++;/*I_*/if (returnVal == JFileChooser.APPROVE_OPTION) {
                /*_I*/_prof.prof.cnt[8769]++;/*I_*/File chosenFile = chooser.getSelectedFile();
                /*_I*/_prof.prof.cnt[8770]++;/*I_*/URL link;
                //Force the extension to be .mm
                /*_I*/_prof.prof.cnt[8771]++;/*I_*/String ext = Tools.getExtension(chosenFile.getName());
                /*_I*/{_prof.prof.cnt[8772]++;/*I_*/if (!ext.equals("mm")) {
                    /*_I*/{_prof.prof.cnt[8773]++;/*I_*/chosenFile = new File(chosenFile.getParent(), chosenFile.getName() + ".mm");/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[8774]++;/*I_*/try {
                    /*_I*/{_prof.prof.cnt[8775]++;/*I_*/link = chosenFile.toURL();/*_I*/}/*I_*/
                } catch (MalformedURLException ex) {
                    /*_I*/{_prof.prof.cnt[8776]++;/*I_*/JOptionPane.showMessageDialog(getView(),
                            "couldn't create valid URL!");/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[8777]++;/*I_*/return;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/

                //Now make a copy from the node, remove the node from the map
                // and create a new
                //Map with the node as root, store the new Map, add the copy of
                // the node to the parent,
                //and set a link from the copy to the new Map.

                /*_I*/_prof.prof.cnt[8778]++;/*I_*/MindMapNodeModel parent = (MindMapNodeModel) node
                        .getParentNode();
                /*_I*/{_prof.prof.cnt[8779]++;/*I_*/if (getMindMapMapModel().getFile() != null) {
                    /*_I*/{_prof.prof.cnt[8780]++;/*I_*/try {
                        //set a link from the new root to the old map
                        /*_I*/_prof.prof.cnt[8781]++;/*I_*/String linkToNewMapString = Tools.toRelativeURL(chosenFile.toURL(),
                                getMindMapMapModel().getFile().toURL());
                        /*_I*/{_prof.prof.cnt[8782]++;/*I_*/setLink(node, linkToNewMapString);/*_I*/}/*I_*/
                    } catch (MalformedURLException ex) {
                    }/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                /*_I*/_prof.prof.cnt[8783]++;/*I_*/int nodePosition = parent.getChildPosition(node);
                /*_I*/{_prof.prof.cnt[8784]++;/*I_*/deleteNode(node);/*_I*/}/*I_*/
                // save node:
                /*_I*/{_prof.prof.cnt[8785]++;/*I_*/node.setParent(null);/*_I*/}/*I_*/
                /*_I*/_prof.prof.cnt[8786]++;/*I_*/MindMapMapModel map = new MindMapMapModel(node, getFrame());
                /*_I*/{_prof.prof.cnt[8787]++;/*I_*/map.save(chosenFile);/*_I*/}/*I_*/
                // new node instead:
                /*_I*/_prof.prof.cnt[8788]++;/*I_*/MindMapNode newNode = addNewNode(parent, nodePosition, node.isLeft());
                /*_I*/{_prof.prof.cnt[8789]++;/*I_*/setNodeText(newNode, node.getText());/*_I*/}/*I_*/

                /*_I*/{_prof.prof.cnt[8790]++;/*I_*/try {
                    /*_I*/_prof.prof.cnt[8791]++;/*I_*/String linkString = Tools.toRelativeURL(
                            getMindMapMapModel().getFile().toURL(), chosenFile.toURL());
                    /*_I*/{_prof.prof.cnt[8792]++;/*I_*/setLink(newNode, linkString);/*_I*/}/*I_*/
                } catch (MalformedURLException ex) {
                }/*_I*/}/*I_*/
                // map should not be save automatically!!
                //getMindMapMapModel().save(getMindMapMapModel().getFile());
            }/*_I*/}/*I_*/
        }
    }

    private class ImportBranchAction extends AbstractAction {
	ImportBranchAction() {
           super(getText("import_branch"));/*_I*/_prof.prof.cnt[8793]++;/*I_*//*_I*/_prof.prof.cnt[8794]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[8795]++;/*I_*/
	    /*_I*/_prof.prof.cnt[8796]++;/*I_*/MindMapNodeModel parent = (MindMapNodeModel)getSelected();
	    /*_I*/{_prof.prof.cnt[8797]++;/*I_*/if (parent == null) {
               /*_I*/{_prof.prof.cnt[8798]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	    /*_I*/_prof.prof.cnt[8799]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
            /*_I*/_prof.prof.cnt[8800]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir);
            //chooser.setLocale(currentLocale);
            /*_I*/{_prof.prof.cnt[8801]++;/*I_*/if (getFileFilter() != null) {
               /*_I*/{_prof.prof.cnt[8802]++;/*I_*/chooser.addChoosableFileFilter(getFileFilter());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[8803]++;/*I_*/int returnVal = chooser.showOpenDialog(getFrame().getContentPane());
            /*_I*/{_prof.prof.cnt[8804]++;/*I_*/if (returnVal==JFileChooser.APPROVE_OPTION) {
               /*_I*/{_prof.prof.cnt[8805]++;/*I_*/try {
                  /*_I*/_prof.prof.cnt[8806]++;/*I_*/MindMapNodeModel node = getMindMapMapModel().loadTree(chooser.getSelectedFile());
                  /*_I*/{_prof.prof.cnt[8807]++;/*I_*/paste(node, parent);/*_I*/}/*I_*/
 				  /*_I*/{_prof.prof.cnt[8808]++;/*I_*/invokeHooksRecursively(node, getMindMapMapModel());/*_I*/}/*I_*/
               }
               catch (Exception ex) {
                  /*_I*/{_prof.prof.cnt[8809]++;/*I_*/handleLoadingException(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/}}

    private class ImportLinkedBranchAction extends AbstractAction {
	ImportLinkedBranchAction() {
           super(getText("import_linked_branch"));/*_I*/_prof.prof.cnt[8810]++;/*I_*//*_I*/_prof.prof.cnt[8811]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[8812]++;/*I_*/
	    /*_I*/_prof.prof.cnt[8813]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)getSelected();
	    /*_I*/{_prof.prof.cnt[8814]++;/*I_*/if (selected == null || selected.getLink() == null) {
	        /*_I*/{_prof.prof.cnt[8815]++;/*I_*/JOptionPane.showMessageDialog(getView(),getText("import_linked_branch_no_link"));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8816]++;/*I_*/return;/*_I*/}/*I_*/ 
        }/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[8817]++;/*I_*/URL absolute = null;
            /*_I*/{_prof.prof.cnt[8818]++;/*I_*/try {
               /*_I*/_prof.prof.cnt[8819]++;/*I_*/String relative = selected.getLink();
               /*_I*/{_prof.prof.cnt[8820]++;/*I_*/absolute = Tools.isAbsolutePath(relative) ? new File(relative).toURL() :
                  new URL(getMap().getFile().toURL(), relative);/*_I*/}/*I_*/ }
            catch (MalformedURLException ex) {
               /*_I*/{_prof.prof.cnt[8821]++;/*I_*/JOptionPane.showMessageDialog(getView(),"Couldn't create valid URL for:"+getMap().getFile());/*_I*/}/*I_*/
               /*_I*/{_prof.prof.cnt[8822]++;/*I_*/ex.printStackTrace();/*_I*/}/*I_*/
               /*_I*/{_prof.prof.cnt[8823]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8824]++;/*I_*/try {
               /*_I*/_prof.prof.cnt[8825]++;/*I_*/MindMapNodeModel node = getMindMapMapModel().loadTree(new File(absolute.getFile()));
               /*_I*/{_prof.prof.cnt[8826]++;/*I_*/paste(node, selected);/*_I*/}/*I_*/ 
			   /*_I*/{_prof.prof.cnt[8827]++;/*I_*/invokeHooksRecursively(node, getMindMapMapModel());/*_I*/}/*I_*/
            }
            catch (Exception ex) {
               /*_I*/{_prof.prof.cnt[8828]++;/*I_*/handleLoadingException(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    /**
     * This is exactly the opposite of exportBranch.
     */
    private class ImportLinkedBranchWithoutRootAction extends AbstractAction {
	ImportLinkedBranchWithoutRootAction() {
           super(getText("import_linked_branch_without_root"));/*_I*/_prof.prof.cnt[8829]++;/*I_*//*_I*/_prof.prof.cnt[8830]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[8831]++;/*I_*/
	    /*_I*/_prof.prof.cnt[8832]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)getSelected();
	    /*_I*/{_prof.prof.cnt[8833]++;/*I_*/if (selected == null || selected.getLink() == null) {
	        /*_I*/{_prof.prof.cnt[8834]++;/*I_*/JOptionPane.showMessageDialog(getView(),getText("import_linked_branch_no_link"));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8835]++;/*I_*/return;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[8836]++;/*I_*/URL absolute = null;
            /*_I*/{_prof.prof.cnt[8837]++;/*I_*/try {
               /*_I*/_prof.prof.cnt[8838]++;/*I_*/String relative = selected.getLink();
               /*_I*/{_prof.prof.cnt[8839]++;/*I_*/absolute = Tools.isAbsolutePath(relative) ? new File(relative).toURL() :
                  new URL(getMap().getFile().toURL(), relative);/*_I*/}/*I_*/ }
            catch (MalformedURLException ex) {
               /*_I*/{_prof.prof.cnt[8840]++;/*I_*/JOptionPane.showMessageDialog(getView(),"Couldn't create valid URL.");/*_I*/}/*I_*/
               /*_I*/{_prof.prof.cnt[8841]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[8842]++;/*I_*/try {
               /*_I*/_prof.prof.cnt[8843]++;/*I_*/MindMapNodeModel node = getMindMapMapModel().loadTree(new File(absolute.getFile()));
               /*_I*/{_prof.prof.cnt[8844]++;/*I_*/for (ListIterator i = node.childrenUnfolded();i.hasNext();) {
                  	/*_I*/_prof.prof.cnt[8845]++;/*I_*/MindMapNodeModel importNode = (MindMapNodeModel)i.next();
					/*_I*/{_prof.prof.cnt[8846]++;/*I_*/paste(importNode, selected);/*_I*/}/*I_*/
					/*_I*/{_prof.prof.cnt[8847]++;/*I_*/invokeHooksRecursively(importNode, getMindMapMapModel());/*_I*/}/*I_*/
                  }/*_I*/}/*I_*/
               }
               //getModel().setLink(parent, null); }
            catch (Exception ex) {
               /*_I*/{_prof.prof.cnt[8848]++;/*I_*/handleLoadingException(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    private class FollowLinkAction extends AbstractAction {
	FollowLinkAction() { super(getText("follow_link"));/*_I*/_prof.prof.cnt[8849]++;/*I_*//*_I*/_prof.prof.cnt[8850]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[8851]++;/*I_*/
           /*_I*/{_prof.prof.cnt[8852]++;/*I_*/loadURL();/*_I*/}/*I_*/ }}

/*
 MindMapController.java
    private class NodeViewStyleAction extends AbstractAction {
       NodeViewStyleAction(final String style) { super(getText(style)); m_style = style; }
       public void actionPerformed(ActionEvent e) {
          for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
             MindMapNodeModel selected = (MindMapNodeModel)it.next();
             getModel().setNodeStyle(selected, m_style); }}
       private String m_style;}

    private class EdgeStyleAction extends AbstractAction {
	String style;
	EdgeStyleAction(String style) {
	    super(getText(style));
            this.style = style; }	
       public void actionPerformed(ActionEvent e) {          
          for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
             MindMapNodeModel selected = (MindMapNodeModel)it.next();
             getModel().setEdgeStyle(selected, style); }}}

    private class ApplyPatternAction extends AbstractAction {
        StylePattern pattern;
	ApplyPatternAction(StylePattern pattern) {
	    super(pattern.getName());
	    this.pattern=pattern; }
	public void actionPerformed(ActionEvent e) {
	    for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
		MindMapNodeModel selected = (MindMapNodeModel)it.next();
                ((MindMapMapModel)getModel()).applyPattern(selected, pattern); }}}




    // Nonaction classes
    // ________________________________________________________________________

=======
*/
    private class MindMapFilter extends FileFilter {
      public boolean accept(File f) {/*_I*/_prof.prof.cnt[8853]++;/*I_*/
	      /*_I*/{_prof.prof.cnt[8854]++;/*I_*/if (f.isDirectory()) /*_I*/{_prof.prof.cnt[8855]++;/*I_*/return true;/*_I*/}/*I_*//*_I*/}/*I_*/
	      /*_I*/_prof.prof.cnt[8856]++;/*I_*/String extension = Tools.getExtension(f.getName());
	      /*_I*/{_prof.prof.cnt[8857]++;/*I_*/if (extension != null) {
		  /*_I*/{_prof.prof.cnt[8858]++;/*I_*/if (extension.equals("mm")) {
		     /*_I*/{_prof.prof.cnt[8859]++;/*I_*/return true;/*_I*/}/*I_*/
		  } else {
		     /*_I*/{_prof.prof.cnt[8860]++;/*I_*/return false;/*_I*/}/*I_*/
		  }/*_I*/}/*I_*/
	      }/*_I*/}/*I_*/
	      /*_I*/{_prof.prof.cnt[8861]++;/*I_*/return false;/*_I*/}/*I_*/
       }
	
	   public String getDescription() {/*_I*/_prof.prof.cnt[8862]++;/*I_*/
	      /*_I*/{_prof.prof.cnt[8863]++;/*I_*/return getText("mindmaps_desc");/*_I*/}/*I_*/
	   }
    }


}
