/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: EdgeView.java,v 1.13.14.2 2005/06/16 19:54:36 christianfoltin Exp $*/

package freemind.view.mindmapview;

import freemind.modes.MindMapEdge;
import java.awt.*;
import javax.swing.JLabel;

/**
 * This class represents a single Edge of a MindMap.
 */
public abstract class EdgeView {
    protected NodeView source,target;
    private JLabel label = new JLabel();
    protected Point start, end;
    private static int i;

    static final Stroke DEF_STROKE = new BasicStroke();
	
    protected EdgeView(NodeView source, NodeView target) {/*_I*/_prof.prof.cnt[4737]++;/*I_*/
	/*_I*/{_prof.prof.cnt[4738]++;/*I_*/this.source = source;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[4739]++;/*I_*/this.target = target;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[4740]++;/*I_*/label.setFont(getMap().getController().getFontThroughMap
                      (new Font("Sans Serif",0,10)));/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[4741]++;/*I_*/label.setText(getModel().toString());/*_I*/}/*I_*///Calling update() crashes BezierEdgeView

	/*_I*/{_prof.prof.cnt[4742]++;/*I_*/getMap().add(label);/*_I*/}/*I_*/
    }


    /**
     * This should be a task of MindMapLayout
     * start,end must be initialized...
     */
    public void update() {/*_I*/_prof.prof.cnt[4743]++;/*I_*/
	/*_I*/{_prof.prof.cnt[4744]++;/*I_*/label.setText(getModel().toString());/*_I*/}/*I_*/	
    }

    public void paint(Graphics2D g) {/*_I*/_prof.prof.cnt[4745]++;/*I_*/
	//	label.repaint();
		/*_I*/{_prof.prof.cnt[4746]++;/*I_*/target.paintFoldingMark(g);/*_I*/}/*I_*/
        // if node is folded, then add a plus sign:
        // feature not approved by Daniel. Disabled, fc, 10.1.2004.
//         if(target.getModel().isFolded()) {
//             int height = target.getSize().height/2;
//             // implement a maximum:
//             final int MAX_HEIGHT = 50;
//             if(height > MAX_HEIGHT)
//                 height = MAX_HEIGHT;
//             g.drawArc( end.x - height/2 , end.y - height/2, height,   height,(target.isLeft())?270:90,180);
//         }
        
    }

    public JLabel getLabel() {/*_I*/_prof.prof.cnt[4747]++;/*I_*/
	/*_I*/{_prof.prof.cnt[4748]++;/*I_*/return label;/*_I*/}/*I_*/
    }

    void remove() {/*_I*/_prof.prof.cnt[4749]++;/*I_*/
	/*_I*/{_prof.prof.cnt[4750]++;/*I_*/getMap().remove(label);/*_I*/}/*I_*/
    }

    public abstract Color getColor();

    public Stroke getStroke() {/*_I*/_prof.prof.cnt[4751]++;/*I_*/
       /*_I*/_prof.prof.cnt[4752]++;/*I_*/Stroke result = getModel().getStroke();
       /*_I*/{_prof.prof.cnt[4753]++;/*I_*/if (result==null)
          /*_I*/{_prof.prof.cnt[4754]++;/*I_*/return DEF_STROKE;/*_I*/}/*I_*//*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[4755]++;/*I_*/return result;/*_I*/}/*I_*/ }

    public int getWidth() {/*_I*/_prof.prof.cnt[4756]++;/*I_*/
       /*_I*/{_prof.prof.cnt[4757]++;/*I_*/return getModel().getWidth();/*_I*/}/*I_*/ }

    /**
     * Get the width in pixels rather than in width constant (like -1)
     */
    public int getRealWidth() {/*_I*/_prof.prof.cnt[4758]++;/*I_*/
       /*_I*/_prof.prof.cnt[4759]++;/*I_*/int width = getWidth();
       /*_I*/{_prof.prof.cnt[4760]++;/*I_*/return (width < 1) ? 1 : width;/*_I*/}/*I_*/ }

    protected MindMapEdge getModel() {/*_I*/_prof.prof.cnt[4761]++;/*I_*/
       /*_I*/{_prof.prof.cnt[4762]++;/*I_*/return target.getModel().getEdge();/*_I*/}/*I_*/ }

    protected MapView getMap() {/*_I*/_prof.prof.cnt[4763]++;/*I_*/
       /*_I*/{_prof.prof.cnt[4764]++;/*I_*/return source.getMap();/*_I*/}/*I_*/ }

	
   /**
    *  Get the vertical shift due to alignment of node connexion and edge width.
    *  Bold edges are centered by Graphic. Applies this shift to change this.
    */
   protected int getNodeShift(NodeView node) {/*_I*/_prof.prof.cnt[4765]++;/*I_*/
      /*_I*/{_prof.prof.cnt[4766]++;/*I_*/if (node.getAlignment()==NodeView.ALIGN_CENTER) /*_I*/{_prof.prof.cnt[4767]++;/*I_*/return 0;/*_I*/}/*I_*//*_I*/}/*I_*/
      // ALIGN_BOTTOM is the case of fork style nodes.
      /*_I*/{_prof.prof.cnt[4768]++;/*I_*/if (node.getAlignment()==NodeView.ALIGN_BOTTOM) /*_I*/{_prof.prof.cnt[4769]++;/*I_*/return -getRealWidth()/2+1;/*_I*/}/*I_*//*_I*/}/*I_*/
      //if(node.getAlignment()==NodeView.ALIGN_TOP) return w/2; // Daniel: probably never used
      /*_I*/{_prof.prof.cnt[4770]++;/*I_*/return 0;/*_I*/}/*I_*/
   }
	
   protected int getTargetShift() {/*_I*/_prof.prof.cnt[4771]++;/*I_*/
      /*_I*/{_prof.prof.cnt[4772]++;/*I_*/return getNodeShift(target);/*_I*/}/*I_*/
   }
	
   protected int getSourceShift() {/*_I*/_prof.prof.cnt[4773]++;/*I_*/
      /*_I*/{_prof.prof.cnt[4774]++;/*I_*/return getNodeShift(source);/*_I*/}/*I_*/
   }

   protected void setRendering(Graphics2D g) {/*_I*/_prof.prof.cnt[4775]++;/*I_*/
      /*_I*/{_prof.prof.cnt[4776]++;/*I_*/if (getMap().getController().getAntialiasEdges() || getMap().getController().getAntialiasAll()) {
         /*_I*/{_prof.prof.cnt[4777]++;/*I_*/g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

}
