/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2004  Joerg Mueller, Daniel Polansky, Christian Foltin and others.
 *
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Created on 02.05.2004
 */
/*$Id: EditNodeTextField.java,v 1.1.4.3 2005/04/27 21:45:30 christianfoltin Exp $*/

package freemind.view.mindmapview;

import java.awt.Point;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import freemind.main.Tools;
import freemind.modes.ModeController;

/**
 * @author foltin
 *
 */
public class EditNodeTextField extends EditNodeBase {

    private KeyEvent firstEvent;

    private JTextField textfield;

    public EditNodeTextField(
        final NodeView node,
        final String text,
        final KeyEvent firstEvent,
        ModeController controller,
        EditControl editControl) {
        super(node, text, controller, editControl);/*_I*/_prof.prof.cnt[3381]++;/*I_*//*_I*/_prof.prof.cnt[3382]++;/*I_*/
        /*_I*/{_prof.prof.cnt[3383]++;/*I_*/this.firstEvent = firstEvent;/*_I*/}/*I_*/
    }

    public void show() {/*_I*/_prof.prof.cnt[3384]++;/*I_*/
            /*_I*/{_prof.prof.cnt[3385]++;/*I_*/textfield =
                (getText().length() < 8)
                    ? new JTextField(
                        getText(),
                        8) //Make fields for short texts editable
    : new JTextField(getText());/*_I*/}/*I_*/

        // Set textFields's properties

        /* fc, 12.10.2003: the following method is not correct. Even more with the zoom factors!*/
        /*_I*/_prof.prof.cnt[3386]++;/*I_*/int linkIconWidth = 16;
        /*_I*/_prof.prof.cnt[3387]++;/*I_*/int textFieldBorderWidth = 2;
        /*_I*/_prof.prof.cnt[3388]++;/*I_*/int cursorWidth = 1;
        /*_I*/_prof.prof.cnt[3389]++;/*I_*/int xOffset =
            -1 * textFieldBorderWidth  - 1;
        /*_I*/_prof.prof.cnt[3390]++;/*I_*/int yOffset = -1; // Optimized for Windows style; basically ad hoc
        /*_I*/_prof.prof.cnt[3391]++;/*I_*/int widthAddition =
            2 * textFieldBorderWidth
                + cursorWidth
                + 2;
        /*_I*/_prof.prof.cnt[3392]++;/*I_*/int heightAddition = 2;
        /*_I*/{_prof.prof.cnt[3393]++;/*I_*/if (getNode().getModel().getLink() != null) {
            /*_I*/{_prof.prof.cnt[3394]++;/*I_*/xOffset += linkIconWidth;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3395]++;/*I_*/widthAddition -= linkIconWidth;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3396]++;/*I_*/if (getNode().getModel().getIcons().size() != 0) {
            // fc, 24.9.2003 full ok for the moment, that an icon has the same size as the link icon.
            /*_I*/{_prof.prof.cnt[3397]++;/*I_*/xOffset += linkIconWidth * getNode().getModel().getIcons().size();/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3398]++;/*I_*/widthAddition -= linkIconWidth;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /* fc, 12.10.2003: end buggy method*/

        // minimal width for input field of leaf or folded node (PN)
        /*_I*/_prof.prof.cnt[3399]++;/*I_*/final int MINIMAL_LEAF_WIDTH = 150;
        /*_I*/_prof.prof.cnt[3400]++;/*I_*/final int MINIMAL_WIDTH = 50;

        /*_I*/_prof.prof.cnt[3401]++;/*I_*/int xSize = getNode().getWidth() + widthAddition;
        /*_I*/_prof.prof.cnt[3402]++;/*I_*/int xExtraWidth = 0;
        /*_I*/{_prof.prof.cnt[3403]++;/*I_*/if (MINIMAL_LEAF_WIDTH > xSize
            && (getNode().getModel().isFolded()
                || !getNode().getModel().hasChildren())) {
            // leaf or folded node with small size
            /*_I*/{_prof.prof.cnt[3404]++;/*I_*/xExtraWidth = MINIMAL_LEAF_WIDTH - xSize;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3405]++;/*I_*/xSize = MINIMAL_LEAF_WIDTH;/*_I*/}/*I_*/ // increase minimum size
            /*_I*/{_prof.prof.cnt[3406]++;/*I_*/if (getNode().isLeft()) { // left leaf
                /*_I*/{_prof.prof.cnt[3407]++;/*I_*/xExtraWidth = -xExtraWidth;/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[3408]++;/*I_*/textfield.setHorizontalAlignment(JTextField.RIGHT);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        } else /*_I*/{_prof.prof.cnt[3409]++;/*I_*/if (MINIMAL_WIDTH > xSize) {
            // opened node with small size
            /*_I*/{_prof.prof.cnt[3410]++;/*I_*/xExtraWidth = MINIMAL_WIDTH - xSize;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3411]++;/*I_*/xSize = MINIMAL_WIDTH;/*_I*/}/*I_*/ // increase minimum size
            /*_I*/{_prof.prof.cnt[3412]++;/*I_*/if (getNode().isLeft()) { // left node
                /*_I*/{_prof.prof.cnt[3413]++;/*I_*/xExtraWidth = -xExtraWidth;/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[3414]++;/*I_*/textfield.setHorizontalAlignment(JTextField.RIGHT);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*//*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[3415]++;/*I_*/textfield.setSize(xSize, getNode().getHeight() + heightAddition);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3416]++;/*I_*/textfield.setFont(getNode().getFont());/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3417]++;/*I_*/textfield.setForeground(getNode().getForeground());/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3418]++;/*I_*/textfield.setSelectedTextColor(getNode().getForeground());/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3419]++;/*I_*/textfield.setSelectionColor(getModeController().getSelectionColor());/*_I*/}/*I_*/
        // textField.selectAll(); // no selection on edit (PN)

        /*_I*/_prof.prof.cnt[3420]++;/*I_*/final int INIT = 0;
        /*_I*/_prof.prof.cnt[3421]++;/*I_*/final int EDIT = 1;
        /*_I*/_prof.prof.cnt[3422]++;/*I_*/final int CANCEL = 2;
        /*_I*/_prof.prof.cnt[3423]++;/*I_*/final Tools.IntHolder eventSource = new Tools.IntHolder();
        /*_I*/{_prof.prof.cnt[3424]++;/*I_*/eventSource.setValue(INIT);/*_I*/}/*I_*/

        // listener class
        class TextFieldListener
            implements KeyListener, FocusListener, MouseListener {

            public void focusGained(FocusEvent e) {/*_I*/_prof.prof.cnt[3425]++;/*I_*/
                // the first time the edit field gains a focus
                // process the predefined first key (if any)

                /*_I*/{_prof.prof.cnt[3426]++;/*I_*/if (eventSource.getValue() == INIT) {
                    /*_I*/{_prof.prof.cnt[3427]++;/*I_*/eventSource.setValue(EDIT);/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[3428]++;/*I_*/if (firstEvent instanceof KeyEvent) {
                        /*_I*/_prof.prof.cnt[3429]++;/*I_*/KeyEvent firstKeyEvent = (KeyEvent) firstEvent;
                        /*_I*/{_prof.prof.cnt[3430]++;/*I_*/if (firstKeyEvent.getKeyChar()
                            == KeyEvent.CHAR_UNDEFINED) {
                            // for the char_undefined the scenario with dispatching 
                            // doesn't work => hard code dispatching :-(
                            // // dispatch action key events as it came
                            // textField.dispatchEvent(firstKeyEvent);

                            // dispatch 2 known events (+ special for insert:) (hardcoded)
                            /*_I*/{_prof.prof.cnt[3431]++;/*I_*/switch (firstKeyEvent.getKeyCode()) {
                                case KeyEvent.VK_HOME :
                                    /*_I*/{_prof.prof.cnt[3432]++;/*I_*/textfield.setCaretPosition(0);/*_I*/}/*I_*/
                                    /*_I*/{_prof.prof.cnt[3433]++;/*I_*/break;/*_I*/}/*I_*/
                                case KeyEvent.VK_END :
                                    /*_I*/{_prof.prof.cnt[3434]++;/*I_*/textfield.setCaretPosition(
                                        textfield.getText().length());/*_I*/}/*I_*/
                                    /*_I*/{_prof.prof.cnt[3435]++;/*I_*/break;/*_I*/}/*I_*/
                            }/*_I*/}/*I_*/
                        } else {
                            // or create new "key type" event for printable key
                            /*_I*/_prof.prof.cnt[3436]++;/*I_*/KeyEvent keyEv;
                            /*_I*/{_prof.prof.cnt[3437]++;/*I_*/keyEv =
                                new KeyEvent(
                                    textfield,	//firstKeyEvent.getComponent(),
                                    KeyEvent.KEY_TYPED,
                                    firstKeyEvent.getWhen(),
                                    firstKeyEvent.getModifiers(),
                                    KeyEvent.VK_UNDEFINED,
                                    firstKeyEvent.getKeyChar(),
                                    KeyEvent.KEY_LOCATION_UNKNOWN);/*_I*/}/*I_*/
                            /*_I*/{_prof.prof.cnt[3438]++;/*I_*/textfield.selectAll();/*_I*/}/*I_*/ // to enable overwrite
                            /*_I*/{_prof.prof.cnt[3439]++;/*I_*/textfield.dispatchEvent(keyEv);/*_I*/}/*I_*/
                        }/*_I*/}/*I_*/
                    }/*_I*/}/*I_*/ // 1st key event defined
                }/*_I*/}/*I_*/ // first focus
            } // focus gained

            public void focusLost(FocusEvent e) {/*_I*/_prof.prof.cnt[3440]++;/*I_*/

                // %%% open problems:
                // - adding of a child to the rightmost node
                // - scrolling while in editing mode (it can behave just like other viewers)
                // - block selected events while in editing mode

                /*_I*/{_prof.prof.cnt[3441]++;/*I_*/if (e == null) { // can be when called explicitly
                    /*_I*/{_prof.prof.cnt[3442]++;/*I_*/getEditControl().ok(textfield.getText());/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[3443]++;/*I_*/hideMe();/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[3444]++;/*I_*/eventSource.setValue(CANCEL);/*_I*/}/*I_*/ // disallow real focus lost
                } else /*_I*/{_prof.prof.cnt[3445]++;/*I_*/if (eventSource.getValue() != CANCEL) {
                    // always confirm the text if not yet
                    /*_I*/{_prof.prof.cnt[3446]++;/*I_*/getEditControl().ok(textfield.getText());/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[3447]++;/*I_*/hideMe();/*_I*/}/*I_*/
                }/*_I*/}/*I_*//*_I*/}/*I_*/
            }

            public void keyPressed(KeyEvent e) {/*_I*/_prof.prof.cnt[3448]++;/*I_*/

                // add to check meta keydown by koh 2004.04.16
                /*_I*/{_prof.prof.cnt[3449]++;/*I_*/if (e.isAltDown() || e.isControlDown() || e.isMetaDown()) {
                    /*_I*/{_prof.prof.cnt[3450]++;/*I_*/return;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/

                /*_I*/_prof.prof.cnt[3451]++;/*I_*/boolean commit = true;

                /*_I*/{_prof.prof.cnt[3452]++;/*I_*/switch (e.getKeyCode()) {
                    case KeyEvent.VK_ESCAPE :
                        /*_I*/{_prof.prof.cnt[3453]++;/*I_*/commit = false;/*_I*/}/*I_*/
                    case KeyEvent.VK_ENTER :
                        /*_I*/{_prof.prof.cnt[3454]++;/*I_*/e.consume();/*_I*/}/*I_*/

                        /*_I*/{_prof.prof.cnt[3455]++;/*I_*/eventSource.setValue(CANCEL);/*_I*/}/*I_*/
                        // do not process loose of focus
                        /*_I*/{_prof.prof.cnt[3456]++;/*I_*/if (commit) {
                            /*_I*/{_prof.prof.cnt[3457]++;/*I_*/getEditControl().ok(textfield.getText());/*_I*/}/*I_*/
                        } else {
                            /*_I*/{_prof.prof.cnt[3458]++;/*I_*/getEditControl().cancel();/*_I*/}/*I_*/
                        }/*_I*/}/*I_*/
                        /*_I*/{_prof.prof.cnt[3459]++;/*I_*/hideMe();/*_I*/}/*I_*/
                        // hack: to keep the focus
                        /*_I*/{_prof.prof.cnt[3460]++;/*I_*/break;/*_I*/}/*I_*/

                    case KeyEvent.VK_SPACE :
                        /*_I*/{_prof.prof.cnt[3461]++;/*I_*/e.consume();/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }
            public void keyTyped(KeyEvent e) {/*_I*/_prof.prof.cnt[3462]++;/*I_*/
            }
            public void keyReleased(KeyEvent e) {/*_I*/_prof.prof.cnt[3463]++;/*I_*/
            }

            public void mouseClicked(MouseEvent e) {/*_I*/_prof.prof.cnt[3464]++;/*I_*/
            }
            public void mouseEntered(MouseEvent e) {/*_I*/_prof.prof.cnt[3465]++;/*I_*/
            }
            public void mouseExited(MouseEvent e) {/*_I*/_prof.prof.cnt[3466]++;/*I_*/
            }

            public void mousePressed(MouseEvent e) {/*_I*/_prof.prof.cnt[3467]++;/*I_*/
                /*_I*/{_prof.prof.cnt[3468]++;/*I_*/conditionallyShowPopup(e);/*_I*/}/*I_*/
            }

            public void mouseReleased(MouseEvent e) {/*_I*/_prof.prof.cnt[3469]++;/*I_*/
                /*_I*/{_prof.prof.cnt[3470]++;/*I_*/conditionallyShowPopup(e);/*_I*/}/*I_*/
            }

            private void conditionallyShowPopup(MouseEvent e) {/*_I*/_prof.prof.cnt[3471]++;/*I_*/
                /*_I*/{_prof.prof.cnt[3472]++;/*I_*/if (e.isPopupTrigger()) {
                    /*_I*/_prof.prof.cnt[3473]++;/*I_*/JPopupMenu popupMenu = new EditPopupMenu(textfield);
                    /*_I*/{_prof.prof.cnt[3474]++;/*I_*/popupMenu.show(e.getComponent(), e.getX(), e.getY());/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[3475]++;/*I_*/e.consume();/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }

        }

        // create the listener
        /*_I*/_prof.prof.cnt[3476]++;/*I_*/final TextFieldListener textFieldListener = new TextFieldListener();

        // Add listeners
        /*_I*/{_prof.prof.cnt[3477]++;/*I_*/this.textFieldListener = textFieldListener;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3478]++;/*I_*/textfield.addFocusListener(textFieldListener);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3479]++;/*I_*/textfield.addKeyListener(textFieldListener);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3480]++;/*I_*/textfield.addMouseListener(textFieldListener);/*_I*/}/*I_*/

        // screen positionining ---------------------------------------------

        // SCROLL if necessary
        /*_I*/{_prof.prof.cnt[3481]++;/*I_*/getView().scrollNodeToVisible(getNode(), xExtraWidth);/*_I*/}/*I_*/

        // NOTE: this must be calculated after scroll because the pane location changes
        /*_I*/_prof.prof.cnt[3482]++;/*I_*/Point frameScreenLocation =
            getFrame().getLayeredPane().getLocationOnScreen();
        /*_I*/_prof.prof.cnt[3483]++;/*I_*/Point nodeScreenLocation = getNode().getLocationOnScreen();

        /*_I*/_prof.prof.cnt[3484]++;/*I_*/int xLeft =
            (int) (nodeScreenLocation.getX()
                - frameScreenLocation.getX()
                + xOffset);
        /*_I*/{_prof.prof.cnt[3485]++;/*I_*/if (xExtraWidth < 0) {
            /*_I*/{_prof.prof.cnt[3486]++;/*I_*/xLeft += xExtraWidth;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[3487]++;/*I_*/textfield.setLocation(
            xLeft,
            (int) (nodeScreenLocation.getY()
                - frameScreenLocation.getY()
                + yOffset));/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[3488]++;/*I_*/getFrame().getLayeredPane().add(textfield);/*_I*/}/*I_*/ // 2000);
        /*_I*/{_prof.prof.cnt[3489]++;/*I_*/getFrame().repaint();/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[3490]++;/*I_*/SwingUtilities.invokeLater(new Runnable() { // PN 0.6.2
            public void run() {/*_I*/_prof.prof.cnt[3491]++;/*I_*/
                /*_I*/{_prof.prof.cnt[3492]++;/*I_*/textfield.requestFocus();/*_I*/}/*I_*/
            }
        });/*_I*/}/*I_*/
    }

    private void hideMe() {/*_I*/_prof.prof.cnt[3493]++;/*I_*/
        /*_I*/{_prof.prof.cnt[3494]++;/*I_*/getFrame().getLayeredPane().remove(textfield);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3495]++;/*I_*/getFrame().repaint();/*_I*/}/*I_*/ //  getLayeredPane().repaint();
        /*_I*/{_prof.prof.cnt[3496]++;/*I_*/textFieldListener = null;/*_I*/}/*I_*/
    }
    


}
