/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: NodeView.java,v 1.27.14.17 2005/07/03 20:53:50 christianfoltin Exp $*/

package freemind.view.mindmapview;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetListener;
import java.net.MalformedURLException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import freemind.controller.Controller;
import freemind.main.FreeMind;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.modes.MindIcon;
import freemind.modes.MindMapCloud;
import freemind.modes.MindMapNode;
import freemind.modes.NodeAdapter;
import freemind.preferences.FreemindPropertyListener;


/**
 * This class represents a single Node of a MindMap
 * (in analogy to TreeCellRenderer).
 */
public abstract class NodeView extends JLabel {

    protected MindMapNode model;
    protected MapView map;
    protected EdgeView edge;
    /** the Color of the Rectangle of a selected Node */
	protected final static Color selectedColor = new Color(210,210,210); //Color.lightGray; //the Color of the Rectangle of a selected Node
    protected final static Color dragColor = Color.lightGray; //the Color of appearing GradientBox on drag over
	protected int treeWidth = 0;
	protected int treeHeight = 0;
	protected int upperChildShift = 0;
    private boolean left = true; //is the node left of root?
    int relYPos = 0;//the relative Y Position to it's parent
    private boolean isLong = false;
    
    public final static int DRAGGED_OVER_NO = 0;
    public final static int DRAGGED_OVER_SON = 1;
    public final static int DRAGGED_OVER_SIBLING = 2;
    /** For RootNodeView.*/
    public final static int DRAGGED_OVER_SON_LEFT = 3;

    protected int isDraggedOver = DRAGGED_OVER_NO;
    public void setDraggedOver(int draggedOver) {/*_I*/_prof.prof.cnt[2702]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2703]++;/*I_*/isDraggedOver = draggedOver;/*_I*/}/*I_*/ }
    public void setDraggedOver(Point p) {/*_I*/_prof.prof.cnt[2704]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2705]++;/*I_*/setDraggedOver( (dropAsSibling(p.getX())) ? NodeView.DRAGGED_OVER_SIBLING : NodeView.DRAGGED_OVER_SON) ;/*_I*/}/*I_*/ }
    public int getDraggedOver() {/*_I*/_prof.prof.cnt[2706]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2707]++;/*I_*/return isDraggedOver;/*_I*/}/*I_*/ }

	final static int ALIGN_BOTTOM = -1;
	final static int ALIGN_CENTER = 0;
	final static int ALIGN_TOP = 1;
    
    //
    // Constructors
    //
    
    private static Color standardSelectColor;
    private static Color standardNodeColor;
    protected NodeView(MindMapNode model, MapView map) {/*_I*/_prof.prof.cnt[2708]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2709]++;/*I_*/setHorizontalAlignment(CENTER);/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[2710]++;/*I_*/this.model = model;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2711]++;/*I_*/setMap(map);/*_I*/}/*I_*/

	// initialize the standard node color.
	/*_I*/{_prof.prof.cnt[2712]++;/*I_*/if (standardNodeColor == null) {
        /*_I*/{_prof.prof.cnt[2713]++;/*I_*/standardNodeColor =
            Tools.xmlToColor(
                map.getController().getProperty(FreeMind.RESOURCES_NODE_COLOR));/*_I*/}/*I_*/
        // add listener:
        /*_I*/{_prof.prof.cnt[2714]++;/*I_*/Controller
                    .addPropertyChangeListener(new FreemindPropertyListener() {

                        public void propertyChanged(String propertyName,
                                String newValue, String oldValue) {/*_I*/_prof.prof.cnt[2715]++;/*I_*/
                            /*_I*/{_prof.prof.cnt[2716]++;/*I_*/if (propertyName
                                    .equals(FreeMind.RESOURCES_NODE_COLOR)) {
                                /*_I*/{_prof.prof.cnt[2717]++;/*I_*/standardNodeColor = Tools.xmlToColor(newValue);/*_I*/}/*I_*/
                            }/*_I*/}/*I_*/
                            /*_I*/{_prof.prof.cnt[2718]++;/*I_*/if (propertyName
                                    .equals(FreeMind.RESOURCES_SELECTED_NODE_COLOR)) {
                                /*_I*/{_prof.prof.cnt[2719]++;/*I_*/standardSelectColor = Tools.xmlToColor(newValue);/*_I*/}/*I_*/
                            }/*_I*/}/*I_*/
                        }
                    });/*_I*/}/*I_*/

    }/*_I*/}/*I_*/
	// initialize the selectedColor:
	/*_I*/{_prof.prof.cnt[2720]++;/*I_*/if(standardSelectColor== null) {
		/*_I*/_prof.prof.cnt[2721]++;/*I_*/String stdcolor = map.getController().getFrame().getProperty(FreeMind.RESOURCES_SELECTED_NODE_COLOR);
		/*_I*/{_prof.prof.cnt[2722]++;/*I_*/if (stdcolor.length() == 7) {
			/*_I*/{_prof.prof.cnt[2723]++;/*I_*/standardSelectColor = Tools.xmlToColor(stdcolor);/*_I*/}/*I_*/
		} else {
			/*_I*/{_prof.prof.cnt[2724]++;/*I_*/standardSelectColor = new Color(210,210,210);/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}/*_I*/}/*I_*/

	//Root has no edge
	/*_I*/{_prof.prof.cnt[2725]++;/*I_*/if (!isRoot()) {
	    /*_I*/{_prof.prof.cnt[2726]++;/*I_*/if (getModel().getEdge().getStyle().equals("linear")) {
		/*_I*/{_prof.prof.cnt[2727]++;/*I_*/edge = new LinearEdgeView(getParentView(),this);/*_I*/}/*I_*/
	    } else /*_I*/{_prof.prof.cnt[2728]++;/*I_*/if (getModel().getEdge().getStyle().equals("bezier")) {
		/*_I*/{_prof.prof.cnt[2729]++;/*I_*/edge = new BezierEdgeView(getParentView(),this);/*_I*/}/*I_*/
	    } else /*_I*/{_prof.prof.cnt[2730]++;/*I_*/if (getModel().getEdge().getStyle().equals("sharp_linear")) {
		/*_I*/{_prof.prof.cnt[2731]++;/*I_*/edge = new SharpLinearEdgeView(getParentView(),this);/*_I*/}/*I_*/
	    } else /*_I*/{_prof.prof.cnt[2732]++;/*I_*/if (getModel().getEdge().getStyle().equals("sharp_bezier")) {
		/*_I*/{_prof.prof.cnt[2733]++;/*I_*/edge = new SharpBezierEdgeView(getParentView(),this);/*_I*/}/*I_*/
	    } else {
		/*_I*/{_prof.prof.cnt[2734]++;/*I_*/System.err.println("Unknown Edge Type.");/*_I*/}/*I_*/
	    }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
	}/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[2735]++;/*I_*/addMouseListener( map.getNodeMouseMotionListener() );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2736]++;/*I_*/addMouseMotionListener( map.getNodeMouseMotionListener() );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2737]++;/*I_*/addKeyListener( map.getNodeKeyListener() );/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[2738]++;/*I_*/addDragListener( map.getNodeDragListener() );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2739]++;/*I_*/addDropListener( map.getNodeDropListener() );/*_I*/}/*I_*/
    }
    
    protected void addToMap(){/*_I*/_prof.prof.cnt[2740]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[2741]++;/*I_*/map.add(this);/*_I*/}/*I_*/
    }
    
    protected void removeFromMap(){/*_I*/_prof.prof.cnt[2742]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[2743]++;/*I_*/map.remove(this);/*_I*/}/*I_*/
    }

    void addDragListener(DragGestureListener dgl) {/*_I*/_prof.prof.cnt[2744]++;/*I_*/
	/*_I*/_prof.prof.cnt[2745]++;/*I_*/DragSource dragSource = DragSource.getDefaultDragSource();
	/*_I*/{_prof.prof.cnt[2746]++;/*I_*/dragSource.createDefaultDragGestureRecognizer 
           (this, DnDConstants.ACTION_COPY | DnDConstants.ACTION_MOVE | DnDConstants.ACTION_LINK ,dgl);/*_I*/}/*I_*/
    }

    void addDropListener(DropTargetListener dtl) {/*_I*/_prof.prof.cnt[2747]++;/*I_*/
	/*_I*/_prof.prof.cnt[2748]++;/*I_*/DropTarget dropTarget = new DropTarget(this,dtl);
	/*_I*/{_prof.prof.cnt[2749]++;/*I_*/dropTarget.setActive(true);/*_I*/}/*I_*/
    }

    /**
     * Factory method which creates the right NodeView for the model.
     */ 
    protected static NodeView newNodeView(MindMapNode model, MapView map) {/*_I*/_prof.prof.cnt[2750]++;/*I_*/
	/*_I*/_prof.prof.cnt[2751]++;/*I_*/NodeView newView;
	/*_I*/{_prof.prof.cnt[2752]++;/*I_*/if (model.isRoot()) {
	    /*_I*/{_prof.prof.cnt[2753]++;/*I_*/newView = new RootNodeView( model, map );/*_I*/}/*I_*/
	} else /*_I*/{_prof.prof.cnt[2754]++;/*I_*/if (model.getStyle().equals(MindMapNode.STYLE_FORK) ) {
	    /*_I*/{_prof.prof.cnt[2755]++;/*I_*/newView = new ForkNodeView( model, map );/*_I*/}/*I_*/
//		newView = new BubbleNodeView( model, map );
	} else /*_I*/{_prof.prof.cnt[2756]++;/*I_*/if (model.getStyle().equals(MindMapNode.STYLE_BUBBLE) ) {
//		newView = new ForkNodeView( model, map );
		/*_I*/{_prof.prof.cnt[2757]++;/*I_*/newView = new BubbleNodeView( model, map );/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2758]++;/*I_*/System.err.println("Tried to create a NodeView of unknown Style.");/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[2759]++;/*I_*/newView = new ForkNodeView(model, map);/*_I*/}/*I_*/
	}/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2760]++;/*I_*/model.setViewer(newView);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2761]++;/*I_*/newView.addToMap();/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2762]++;/*I_*/newView.update();/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2763]++;/*I_*/return newView;/*_I*/}/*I_*/
    }

    //
    // public methods
    //

    public boolean dropAsSibling(double xCoord) {/*_I*/_prof.prof.cnt[2764]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2765]++;/*I_*/return isInVerticalRegion(xCoord, 1./3);/*_I*/}/*I_*/
     }

    /** Determines whether or not the xCoord is in the part p of the node:
     *  if node is on the left: part [1-p,1] 
     *  if node is on the right: part[  0,p] of the total width.
     */
    public boolean isInVerticalRegion(double xCoord, double p) {/*_I*/_prof.prof.cnt[2766]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2767]++;/*I_*/return isLeft() ?
           xCoord > getSize().width*(1.0-p) :
           xCoord < getSize().width*p;/*_I*/}/*I_*/ 
     }

    /** @return true if should be on the left, false otherwise.*/
    public boolean dropPosition (double xCoord) {/*_I*/_prof.prof.cnt[2768]++;/*I_*/
        /* here it is the same as me. */
       /*_I*/{_prof.prof.cnt[2769]++;/*I_*/return isLeft();/*_I*/}/*I_*/ 
    }

    public boolean isInFollowLinkRegion(double xCoord) {/*_I*/_prof.prof.cnt[2770]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2771]++;/*I_*/return getModel().getLink() != null &&
          (getModel().isRoot() || !getModel().hasChildren() || isInVerticalRegion(xCoord, 1./2));/*_I*/}/*I_*/ 
    }

    /**
     * @param xCoord
     * @return true if a link is to be displayed and the curser is the hand now.
     */
    public boolean updateCursor(double xCoord) {/*_I*/_prof.prof.cnt[2772]++;/*I_*/
      /*_I*/_prof.prof.cnt[2773]++;/*I_*/boolean followLink = isInFollowLinkRegion(xCoord);
    /*_I*/_prof.prof.cnt[2774]++;/*I_*/int requiredCursor = followLink ? Cursor.HAND_CURSOR : Cursor.DEFAULT_CURSOR;
      /*_I*/{_prof.prof.cnt[2775]++;/*I_*/if (getCursor().getType() != requiredCursor) {
        /*_I*/{_prof.prof.cnt[2776]++;/*I_*/setCursor(new Cursor(requiredCursor));/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2777]++;/*I_*/return followLink;/*_I*/}/*I_*/
    }

    public boolean isRoot() {/*_I*/_prof.prof.cnt[2778]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2779]++;/*I_*/return model.isRoot();/*_I*/}/*I_*/
    }

    public boolean getIsLong() {/*_I*/_prof.prof.cnt[2780]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2781]++;/*I_*/return isLong;/*_I*/}/*I_*/ }

    /* fc, 25.1.2004: Refactoring necessary: should call the model. */
    public boolean isSiblingOf(NodeView myNodeView) {/*_I*/_prof.prof.cnt[2782]++;/*I_*/ 
       /*_I*/{_prof.prof.cnt[2783]++;/*I_*/return getParentView() == myNodeView.getParentView();/*_I*/}/*I_*/ }

    /* fc, 25.1.2004: Refactoring necessary: should call the model. */
    public boolean isChildOf(NodeView myNodeView) {/*_I*/_prof.prof.cnt[2784]++;/*I_*/ 
       /*_I*/{_prof.prof.cnt[2785]++;/*I_*/return getParentView() == myNodeView;/*_I*/}/*I_*/ }

    /* fc, 25.1.2004: Refactoring necessary: should call the model. */
    public boolean isParentOf(NodeView myNodeView) {/*_I*/_prof.prof.cnt[2786]++;/*I_*/ 
       /*_I*/{_prof.prof.cnt[2787]++;/*I_*/return (this == myNodeView.getParentView());/*_I*/}/*I_*/ }

    public MindMapNode getModel() {/*_I*/_prof.prof.cnt[2788]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2789]++;/*I_*/return model;/*_I*/}/*I_*/
    }

    /** Returns the coordinates occupied by the node and its children as a vector of four point per node.*/
	public void getCoordinates(LinkedList inList) {/*_I*/_prof.prof.cnt[2790]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2791]++;/*I_*/getCoordinates(inList, 0, false);/*_I*/}/*I_*/
	}
	private void getCoordinates(LinkedList inList, int additionalDistanceForConvexHull, boolean byChildren) {/*_I*/_prof.prof.cnt[2792]++;/*I_*/
		/*_I*/_prof.prof.cnt[2793]++;/*I_*/MindMapCloud cloud = getModel().getCloud();

		// consider existing clouds of children
		/*_I*/{_prof.prof.cnt[2794]++;/*I_*/if (byChildren && cloud != null){
			/*_I*/{_prof.prof.cnt[2795]++;/*I_*/additionalDistanceForConvexHull  += CloudView.getAdditionalHeigth(cloud, this) / 2;/*_I*/}/*I_*/ 
		}/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2796]++;/*I_*/inList.addLast(new Point( -additionalDistanceForConvexHull + getExtendedX()             ,  -additionalDistanceForConvexHull + getExtendedY()              ));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2797]++;/*I_*/inList.addLast(new Point( -additionalDistanceForConvexHull + getExtendedX()             ,   additionalDistanceForConvexHull + getExtendedY() + getExtendedHeight()));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2798]++;/*I_*/inList.addLast(new Point(  additionalDistanceForConvexHull + getExtendedX() + getExtendedWidth(),   additionalDistanceForConvexHull + getExtendedY() + getExtendedHeight()));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2799]++;/*I_*/inList.addLast(new Point(  additionalDistanceForConvexHull + getExtendedX() + getExtendedWidth(),  -additionalDistanceForConvexHull + getExtendedY()              ));/*_I*/}/*I_*/
		
        /*_I*/_prof.prof.cnt[2800]++;/*I_*/LinkedList childrenViews = getChildrenViews();
        /*_I*/_prof.prof.cnt[2801]++;/*I_*/ListIterator children_it = childrenViews.listIterator();
        /*_I*/{_prof.prof.cnt[2802]++;/*I_*/while(children_it.hasNext()) {
            /*_I*/_prof.prof.cnt[2803]++;/*I_*/NodeView child = (NodeView)children_it.next();
	        /*_I*/{_prof.prof.cnt[2804]++;/*I_*/child.getCoordinates(inList, additionalDistanceForConvexHull, true);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }   
	private static boolean NEED_PREF_SIZE_BUG_FIX = Controller.JAVA_VERSION.compareTo("1.5.0") < 0;
	private static final int MIN_HOR_NODE_SIZE = 10;
    public Dimension getPreferredSize() {/*_I*/_prof.prof.cnt[2805]++;/*I_*/
        /*_I*/_prof.prof.cnt[2806]++;/*I_*/boolean isEmpty = getText().length() == 0;
        /*_I*/{_prof.prof.cnt[2807]++;/*I_*/if(isEmpty){
            /*_I*/{_prof.prof.cnt[2808]++;/*I_*/setText("!");/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    	/*_I*/_prof.prof.cnt[2809]++;/*I_*/Dimension prefSize = super.getPreferredSize();
        /*_I*/{_prof.prof.cnt[2810]++;/*I_*/if(map.isCurrentlyPrinting() && NEED_PREF_SIZE_BUG_FIX) {
        	/*_I*/{_prof.prof.cnt[2811]++;/*I_*/prefSize.width += (int)(10f*map.getZoom());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/ 
        /*_I*/{_prof.prof.cnt[2812]++;/*I_*/prefSize.width = Math.max(map.getZoomed(MIN_HOR_NODE_SIZE), prefSize.width);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2813]++;/*I_*/if (isEmpty){
            /*_I*/{_prof.prof.cnt[2814]++;/*I_*/setText("");/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[2815]++;/*I_*/prefSize.width += 4;/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[2816]++;/*I_*/prefSize.height += 4;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2817]++;/*I_*/return prefSize;/*_I*/}/*I_*/
    }
    
    /** get width including folding symbol*/	
	public int getExtendedWidth()
	{/*_I*/_prof.prof.cnt[2818]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2819]++;/*I_*/return getExtendedWidth(getWidth());/*_I*/}/*I_*/
	}
  
	/** get height including folding symbol*/	
	public int getExtendedHeight()
	{/*_I*/_prof.prof.cnt[2820]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2821]++;/*I_*/return getExtendedHeight(getHeight());/*_I*/}/*I_*/
	}
  
	protected int getExtendedWidth(int w)
	{/*_I*/_prof.prof.cnt[2822]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2823]++;/*I_*/return w;/*_I*/}/*I_*/
	}
  
	/** get height including folding symbol*/	
	protected int getExtendedHeight(int h)
	{/*_I*/_prof.prof.cnt[2824]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2825]++;/*I_*/return h;/*_I*/}/*I_*/
	}
  
	/** get x coordinate including folding symbol*/	
	public int getExtendedX()
	{/*_I*/_prof.prof.cnt[2826]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2827]++;/*I_*/return getX();/*_I*/}/*I_*/
	}
  
	/** get y coordinate including folding symbol*/	
	public int getExtendedY()
	{/*_I*/_prof.prof.cnt[2828]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2829]++;/*I_*/return getY();/*_I*/}/*I_*/
	}

  
	public void setBounds(int x,	int y){/*_I*/_prof.prof.cnt[2830]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2831]++;/*I_*/setLocation(x, y);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[2832]++;/*I_*/setSize(getPreferredSize());/*_I*/}/*I_*/
	}

   public void requestFocus(){/*_I*/_prof.prof.cnt[2833]++;/*I_*/
      /*_I*/{_prof.prof.cnt[2834]++;/*I_*/map.getController().getMode().getModeController().anotherNodeSelected(getModel());/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2835]++;/*I_*/super. requestFocus();/*_I*/}/*I_*/
   }

   /** draw folding symbol*/	
	public void paintFoldingMark(Graphics2D g){/*_I*/_prof.prof.cnt[2836]++;/*I_*/ 
	}

    public void paint(Graphics graphics) {/*_I*/_prof.prof.cnt[2837]++;/*I_*/
        // background color starts here, fc. 9.11.2003: todo
//           graphics.setColor(Color.yellow);
//           graphics.fillRect(0,0,getWidth(), getHeight());
          // background color ends here, fc. 9.11.2003: todo
	/*_I*/{_prof.prof.cnt[2838]++;/*I_*/super.paint(graphics);/*_I*/}/*I_*/
    }
    public void paintSelected(Graphics2D graphics, Dimension size) {/*_I*/_prof.prof.cnt[2839]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2840]++;/*I_*/if (this.isSelected()) {
			/*_I*/{_prof.prof.cnt[2841]++;/*I_*/paintBackground(graphics, size, getSelectedColor());/*_I*/}/*I_*/
			//g.drawRect(0,0,size.width-1, size.height-2);
		} else /*_I*/{_prof.prof.cnt[2842]++;/*I_*/if (getModel().getBackgroundColor() != null) {
			/*_I*/{_prof.prof.cnt[2843]++;/*I_*/paintBackground(graphics, size, getModel().getBackgroundColor());/*_I*/}/*I_*/
		}/*_I*/}/*I_*//*_I*/}/*I_*/
//		if (this.isSelected()) {
//			paintBackground(graphics, size, getSelectedColor());
//			//g.drawRect(0,0,size.width-1, size.height-2);
//		} /*else*/
//		if  (true){
//			Dimension newSize = size;
//			newSize.height -= 5;
//			newSize.width -= 5;
//			paintBackground(graphics, newSize, (getModel().getBackgroundColor() != null)?getModel().getBackgroundColor():Color.WHITE);
//		}
    }

	protected void paintBackground(Graphics2D graphics, Dimension size, Color color) {/*_I*/_prof.prof.cnt[2844]++;/*I_*/
		/*_I*/{_prof.prof.cnt[2845]++;/*I_*/graphics.setColor(color);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[2846]++;/*I_*/graphics.fillRect(0,0,size.width, size.height);/*_I*/}/*I_*/		
	}


   public void paintDragOver(Graphics2D graphics, Dimension size) {/*_I*/_prof.prof.cnt[2847]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2848]++;/*I_*/if (isDraggedOver == DRAGGED_OVER_SON) {
           /*_I*/{_prof.prof.cnt[2849]++;/*I_*/if (isLeft()) {
              /*_I*/{_prof.prof.cnt[2850]++;/*I_*/graphics.setPaint( new GradientPaint(size.width*3/4,0,map.getBackground(), size.width/4, 0, dragColor));/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[2851]++;/*I_*/graphics.fillRect(0, 0, size.width*3/4, size.height-1);/*_I*/}/*I_*/ }
           else {
              /*_I*/{_prof.prof.cnt[2852]++;/*I_*/graphics.setPaint( new GradientPaint(size.width/4,0,map.getBackground(), size.width*3/4, 0, dragColor));/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[2853]++;/*I_*/graphics.fillRect(size.width/4, 0, size.width-1, size.height-1);/*_I*/}/*I_*/ }/*_I*/}/*I_*/       
	}/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[2854]++;/*I_*/if (isDraggedOver == DRAGGED_OVER_SIBLING) {
            /*_I*/{_prof.prof.cnt[2855]++;/*I_*/graphics.setPaint( new GradientPaint(0,size.height*3/5,map.getBackground(), 0, size.height/5, dragColor));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[2856]++;/*I_*/graphics.fillRect(0, 0, size.width-1, size.height-1);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

     //
    // get/set methods
    //

    /**
    * Calculates the tree height increment because of the clouds.
    */
	public int getAdditionalCloudHeigth() {/*_I*/_prof.prof.cnt[2857]++;/*I_*/
		/*_I*/_prof.prof.cnt[2858]++;/*I_*/MindMapCloud cloud = getModel().getCloud();
		/*_I*/{_prof.prof.cnt[2859]++;/*I_*/if( cloud!= null) { 
			/*_I*/{_prof.prof.cnt[2860]++;/*I_*/return CloudView.getAdditionalHeigth(cloud, this);/*_I*/}/*I_*/
		} else {           
			/*_I*/{_prof.prof.cnt[2861]++;/*I_*/return 0;/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
	}

 
    protected boolean isSelected() {/*_I*/_prof.prof.cnt[2862]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2863]++;/*I_*/return (getMap().isSelected(this));/*_I*/}/*I_*/
    }

    /**Is the node left of root?*/
    public boolean isLeft() {/*_I*/_prof.prof.cnt[2864]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2865]++;/*I_*/if(getModel().isLeft() == null)
            /*_I*/{_prof.prof.cnt[2866]++;/*I_*/return true;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2867]++;/*I_*/return getModel().isLeft().getValue();/*_I*/}/*I_*/
    }

    protected void setLeft(boolean left) {/*_I*/_prof.prof.cnt[2868]++;/*I_*/
        //this.left = left;
        /*_I*/{_prof.prof.cnt[2869]++;/*I_*/getModel().setLeft(left);/*_I*/}/*I_*/
    }
	
//     public boolean isLeftDefault() {
//         return getModel().isLeft() == null;
//     }
    
    protected void setModel( MindMapNode model ) {/*_I*/_prof.prof.cnt[2870]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2871]++;/*I_*/this.model = model;/*_I*/}/*I_*/
    }

    MapView getMap() {/*_I*/_prof.prof.cnt[2872]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2873]++;/*I_*/return map;/*_I*/}/*I_*/
    }

    protected void setMap( MapView map ) {/*_I*/_prof.prof.cnt[2874]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2875]++;/*I_*/this.map = map;/*_I*/}/*I_*/
    }

    EdgeView getEdge() {/*_I*/_prof.prof.cnt[2876]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2877]++;/*I_*/return edge;/*_I*/}/*I_*/
    }

    void setEdge(EdgeView edge) {/*_I*/_prof.prof.cnt[2878]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2879]++;/*I_*/this.edge = edge;/*_I*/}/*I_*/
    }

    protected NodeView getParentView() {/*_I*/_prof.prof.cnt[2880]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2881]++;/*I_*/return getModel().getParentNode().getViewer();/*_I*/}/*I_*/
    }

    /**
     * This method returns the NodeViews that are children of this node.
     */
    public LinkedList getChildrenViews() {/*_I*/_prof.prof.cnt[2882]++;/*I_*/
	/*_I*/_prof.prof.cnt[2883]++;/*I_*/LinkedList childrenViews = new LinkedList();
	/*_I*/_prof.prof.cnt[2884]++;/*I_*/ListIterator it = getModel().childrenUnfolded();
	/*_I*/{_prof.prof.cnt[2885]++;/*I_*/if (it != null) {
           /*_I*/{_prof.prof.cnt[2886]++;/*I_*/while(it.hasNext()) {
              /*_I*/_prof.prof.cnt[2887]++;/*I_*/NodeView view = ((MindMapNode)it.next()).getViewer();
              /*_I*/{_prof.prof.cnt[2888]++;/*I_*/if (view != null) { // Visible view
                 /*_I*/{_prof.prof.cnt[2889]++;/*I_*/childrenViews.add(view);/*_I*/}/*I_*/ // child.getViewer() );
              }/*_I*/}/*I_*/
           }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2890]++;/*I_*/return childrenViews;/*_I*/}/*I_*/
    }
    
    protected LinkedList getSiblingViews() {/*_I*/_prof.prof.cnt[2891]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2892]++;/*I_*/return getParentView().getChildrenViews();/*_I*/}/*I_*/
    }

    /**
     * Returns the Point where the OutEdge
     * should leave the Node.
     * THIS SHOULD BE DECLARED ABSTRACT AND BE DONE IN BUBBLENODEVIEW ETC.
     */
    Point getOutPoint() {/*_I*/_prof.prof.cnt[2893]++;/*I_*/
	/*_I*/_prof.prof.cnt[2894]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2895]++;/*I_*/if( isRoot() ) {
	    /*_I*/{_prof.prof.cnt[2896]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	} else /*_I*/{_prof.prof.cnt[2897]++;/*I_*/if( isLeft() ) {
	    /*_I*/{_prof.prof.cnt[2898]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height - 2);/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2899]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height - 2);/*_I*/}/*I_*/
	}/*_I*/}/*I_*//*_I*/}/*I_*/ 
    }

    
    /* fc, 26.06.2005 */
    /** Returns the point the edge should start given the point of the child node 
     * that should be connected.
     * @param destinationPoint the outpoint should point in the direction of destinationPoint 
     * @param isLeft TODO
     * @return
     */
    Point getOutPoint(Point destinationPoint, boolean isLeft) {/*_I*/_prof.prof.cnt[2900]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2901]++;/*I_*/return getOutPoint();/*_I*/}/*I_*/
    }
    /* end fc, 26.06.2005 */
    
    /**
     * Returns the Point where the InEdge
     * should arrive the Node.
     * THIS SHOULD BE DECLARED ABSTRACT AND BE DONE IN BUBBLENODEVIEW ETC.
     */
    Point getInPoint() {/*_I*/_prof.prof.cnt[2902]++;/*I_*/
	/*_I*/_prof.prof.cnt[2903]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2904]++;/*I_*/if( isRoot() ) {
	    /*_I*/{_prof.prof.cnt[2905]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	} else /*_I*/{_prof.prof.cnt[2906]++;/*I_*/if( isLeft() ) {
	    /*_I*/{_prof.prof.cnt[2907]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height - 2);/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2908]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height - 2);/*_I*/}/*I_*/
	}/*_I*/}/*I_*//*_I*/}/*I_*/ 
    }


    /**
     * Returns the Point where the Links 
     * should arrive the Node.
     * THIS SHOULD BE DECLARED ABSTRACT AND BE DONE IN BUBBLENODEVIEW ETC.
     */
    public Point getLinkPoint(Point declination) {/*_I*/_prof.prof.cnt[2909]++;/*I_*/
		/*_I*/_prof.prof.cnt[2910]++;/*I_*/Dimension size = getSize();
		/*_I*/_prof.prof.cnt[2911]++;/*I_*/int x, y;
		/*_I*/{_prof.prof.cnt[2912]++;/*I_*/if(declination != null){
			/*_I*/{_prof.prof.cnt[2913]++;/*I_*/x = getMap().getZoomed(declination.x);/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[2914]++;/*I_*/y = getMap().getZoomed(declination.y);/*_I*/}/*I_*/
		}
		else{
			/*_I*/{_prof.prof.cnt[2915]++;/*I_*/x = 1;/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[2916]++;/*I_*/y = 0;/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[2917]++;/*I_*/if( isRoot() || isLeft()) {
			/*_I*/{_prof.prof.cnt[2918]++;/*I_*/x = -x;/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[2919]++;/*I_*/if(y != 0){
			/*_I*/_prof.prof.cnt[2920]++;/*I_*/double ctgRect = Math.abs((double)size.width / size.height);
			/*_I*/_prof.prof.cnt[2921]++;/*I_*/double ctgLine = Math.abs((double)x / y);
			/*_I*/_prof.prof.cnt[2922]++;/*I_*/int absLinkX, absLinkY;
			/*_I*/{_prof.prof.cnt[2923]++;/*I_*/if(ctgRect > ctgLine){
				/*_I*/{_prof.prof.cnt[2924]++;/*I_*/absLinkX = Math.abs(x*size.height / (2 * y));/*_I*/}/*I_*/
				/*_I*/{_prof.prof.cnt[2925]++;/*I_*/absLinkY = size.height / 2;/*_I*/}/*I_*/
			}
			else{
				/*_I*/{_prof.prof.cnt[2926]++;/*I_*/absLinkX = size.width / 2;/*_I*/}/*I_*/
				/*_I*/{_prof.prof.cnt[2927]++;/*I_*/absLinkY = Math.abs(y*size.width / (2 * x));/*_I*/}/*I_*/
			}/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[2928]++;/*I_*/return new Point(getLocation().x + size.width / 2 + (x>0 ? absLinkX : -absLinkX), 
							getLocation().y + size.height / 2 + (y>0 ? absLinkY : -absLinkY));/*_I*/}/*I_*/	
		}
		else{
			/*_I*/{_prof.prof.cnt[2929]++;/*I_*/return new Point(getLocation().x + (x>0 ? size.width:0), 
			                 getLocation().y + (size.height / 2));/*_I*/}/*I_*/	
		}/*_I*/}/*I_*/
    }

    /**
     * Returns the relative position of the Edge.
	 * This is used by bold edge to know how to shift the line.
     */
    int getAlignment() {/*_I*/_prof.prof.cnt[2930]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2931]++;/*I_*/if( isRoot() )
	    /*_I*/{_prof.prof.cnt[2932]++;/*I_*/return ALIGN_CENTER;/*_I*/}/*I_*//*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2933]++;/*I_*/return ALIGN_BOTTOM;/*_I*/}/*I_*/
    }
	
    //
    // Navigation
    //
    protected NodeView getNextPage() {/*_I*/_prof.prof.cnt[2934]++;/*I_*/
      /*_I*/{_prof.prof.cnt[2935]++;/*I_*/if (isRoot()) {
        /*_I*/{_prof.prof.cnt[2936]++;/*I_*/return this;/*_I*/}/*I_*/ // I'm root
      }/*_I*/}/*I_*/
      /*_I*/_prof.prof.cnt[2937]++;/*I_*/NodeView sibling = getNextSibling();
      /*_I*/{_prof.prof.cnt[2938]++;/*I_*/if (sibling == this) {
        /*_I*/{_prof.prof.cnt[2939]++;/*I_*/return this;/*_I*/}/*I_*/ // at the end
      }/*_I*/}/*I_*/
//      if (sibling.getParentView() != this.getParentView()) {
//        return sibling; // sibling on another page (has different parent)
//      }
      /*_I*/_prof.prof.cnt[2940]++;/*I_*/NodeView nextSibling = sibling.getNextSibling();
      /*_I*/{_prof.prof.cnt[2941]++;/*I_*/while (nextSibling != sibling 
              && sibling.getParentView() == nextSibling.getParentView()) {
        /*_I*/{_prof.prof.cnt[2942]++;/*I_*/sibling = nextSibling;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2943]++;/*I_*/nextSibling = nextSibling.getNextSibling();/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2944]++;/*I_*/return sibling;/*_I*/}/*I_*/ // last on the page
    }
      
    protected NodeView getPreviousPage() {/*_I*/_prof.prof.cnt[2945]++;/*I_*/
      /*_I*/{_prof.prof.cnt[2946]++;/*I_*/if (isRoot()) {
        /*_I*/{_prof.prof.cnt[2947]++;/*I_*/return this;/*_I*/}/*I_*/ // I'm root
      }/*_I*/}/*I_*/
      /*_I*/_prof.prof.cnt[2948]++;/*I_*/NodeView sibling = getPreviousSibling();
      /*_I*/{_prof.prof.cnt[2949]++;/*I_*/if (sibling == this) {
        /*_I*/{_prof.prof.cnt[2950]++;/*I_*/return this;/*_I*/}/*I_*/ // at the end
      }/*_I*/}/*I_*/
//      if (sibling.getParentView() != this.getParentView()) {
//        return sibling; // sibling on another page (has different parent)
//      }
      /*_I*/_prof.prof.cnt[2951]++;/*I_*/NodeView previousSibling = sibling.getPreviousSibling();
      /*_I*/{_prof.prof.cnt[2952]++;/*I_*/while (previousSibling != sibling 
              && sibling.getParentView() == previousSibling.getParentView()) {
        /*_I*/{_prof.prof.cnt[2953]++;/*I_*/sibling = previousSibling;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2954]++;/*I_*/previousSibling = previousSibling.getPreviousSibling();/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2955]++;/*I_*/return sibling;/*_I*/}/*I_*/ // last on the page
    }
    
    protected NodeView getNextSibling() {/*_I*/_prof.prof.cnt[2956]++;/*I_*/
      /*_I*/_prof.prof.cnt[2957]++;/*I_*/NodeView sibling;
      /*_I*/_prof.prof.cnt[2958]++;/*I_*/NodeView nextSibling = this;
      
      // get next sibling even in higher levels
      /*_I*/{_prof.prof.cnt[2959]++;/*I_*/for (sibling = this; !sibling.isRoot(); sibling = sibling.getParentView()) { 
        /*_I*/{_prof.prof.cnt[2960]++;/*I_*/nextSibling = sibling.getNextSiblingSingle();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2961]++;/*I_*/if (sibling != nextSibling) {
          /*_I*/{_prof.prof.cnt[2962]++;/*I_*/break;/*_I*/}/*I_*/ // found sibling
        }/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      
      /*_I*/{_prof.prof.cnt[2963]++;/*I_*/if (sibling.isRoot()) {
        /*_I*/{_prof.prof.cnt[2964]++;/*I_*/return this;/*_I*/}/*I_*/  // didn't find (we are at the end)
      }/*_I*/}/*I_*/
      
      // we have the nextSibling, search in childs
      // untill: leaf, closed node, max level
      /*_I*/{_prof.prof.cnt[2965]++;/*I_*/sibling = nextSibling;/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2966]++;/*I_*/while (sibling.getModel().getNodeLevel() < getMap().getSiblingMaxLevel()) {
        // can we drill down?
        /*_I*/{_prof.prof.cnt[2967]++;/*I_*/if (sibling.getChildrenViews().size() <= 0) {
          /*_I*/{_prof.prof.cnt[2968]++;/*I_*/break;/*_I*/}/*I_*/ // no
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2969]++;/*I_*/sibling = (NodeView)(sibling.getChildrenViews().getFirst());/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2970]++;/*I_*/return sibling;/*_I*/}/*I_*/
    }

    protected NodeView getPreviousSibling() {/*_I*/_prof.prof.cnt[2971]++;/*I_*/
      /*_I*/_prof.prof.cnt[2972]++;/*I_*/NodeView sibling;
      /*_I*/_prof.prof.cnt[2973]++;/*I_*/NodeView previousSibling = this;
      
      // get Previous sibling even in higher levels
      /*_I*/{_prof.prof.cnt[2974]++;/*I_*/for (sibling = this; !sibling.isRoot(); sibling = sibling.getParentView()) { 
        /*_I*/{_prof.prof.cnt[2975]++;/*I_*/previousSibling = sibling.getPreviousSiblingSingle();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2976]++;/*I_*/if (sibling != previousSibling) {
          /*_I*/{_prof.prof.cnt[2977]++;/*I_*/break;/*_I*/}/*I_*/ // found sibling
        }/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      
      /*_I*/{_prof.prof.cnt[2978]++;/*I_*/if (sibling.isRoot()) {
        /*_I*/{_prof.prof.cnt[2979]++;/*I_*/return this;/*_I*/}/*I_*/  // didn't find (we are at the end)
      }/*_I*/}/*I_*/
      
      // we have the PreviousSibling, search in childs
      // untill: leaf, closed node, max level
      /*_I*/{_prof.prof.cnt[2980]++;/*I_*/sibling = previousSibling;/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2981]++;/*I_*/while (sibling.getModel().getNodeLevel() < getMap().getSiblingMaxLevel()) {
        // can we drill down?
        /*_I*/{_prof.prof.cnt[2982]++;/*I_*/if (sibling.getChildrenViews().size() <= 0) {
          /*_I*/{_prof.prof.cnt[2983]++;/*I_*/break;/*_I*/}/*I_*/ // no
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2984]++;/*I_*/sibling = (NodeView)(sibling.getChildrenViews().getLast());/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2985]++;/*I_*/return sibling;/*_I*/}/*I_*/
    }

    protected NodeView getNextSiblingSingle() {/*_I*/_prof.prof.cnt[2986]++;/*I_*/
	/*_I*/_prof.prof.cnt[2987]++;/*I_*/LinkedList v = null;
	/*_I*/{_prof.prof.cnt[2988]++;/*I_*/if (getParentView().isRoot()) {
	    /*_I*/{_prof.prof.cnt[2989]++;/*I_*/if (this.isLeft()) {
		/*_I*/{_prof.prof.cnt[2990]++;/*I_*/v = ((RootNodeView)getParentView()).getLeft();/*_I*/}/*I_*/
	    } else {
		/*_I*/{_prof.prof.cnt[2991]++;/*I_*/v = ((RootNodeView)getParentView()).getRight();/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2992]++;/*I_*/v = getParentView().getChildrenViews();/*_I*/}/*I_*/
	}/*_I*/}/*I_*/	
	/*_I*/_prof.prof.cnt[2993]++;/*I_*/NodeView sibling;
	/*_I*/{_prof.prof.cnt[2994]++;/*I_*/if (v.size()-1 == v.indexOf(this)) { //this is last, return first
//	    sibling = (NodeView)v.getFirst(); // loop
            /*_I*/{_prof.prof.cnt[2995]++;/*I_*/sibling = this;/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2996]++;/*I_*/sibling = (NodeView)v.get(v.indexOf(this)+1);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2997]++;/*I_*/return sibling;/*_I*/}/*I_*/
    }

    protected NodeView getPreviousSiblingSingle() {/*_I*/_prof.prof.cnt[2998]++;/*I_*/
	/*_I*/_prof.prof.cnt[2999]++;/*I_*/LinkedList v = null;
	/*_I*/{_prof.prof.cnt[3000]++;/*I_*/if (getParentView().isRoot()) {
	    /*_I*/{_prof.prof.cnt[3001]++;/*I_*/if (this.isLeft()) {
		/*_I*/{_prof.prof.cnt[3002]++;/*I_*/v = ((RootNodeView)getParentView()).getLeft();/*_I*/}/*I_*/
	    } else {
		/*_I*/{_prof.prof.cnt[3003]++;/*I_*/v = ((RootNodeView)getParentView()).getRight();/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[3004]++;/*I_*/v = getParentView().getChildrenViews();/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/_prof.prof.cnt[3005]++;/*I_*/NodeView sibling;
	/*_I*/{_prof.prof.cnt[3006]++;/*I_*/if (v.indexOf(this) <= 0) {//this is first, return last
//	    sibling = (NodeView)v.getLast(); // loop
          /*_I*/{_prof.prof.cnt[3007]++;/*I_*/sibling = this;/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[3008]++;/*I_*/sibling = (NodeView)v.get(v.indexOf(this)-1);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3009]++;/*I_*/return sibling;/*_I*/}/*I_*/
    }

    //
    // Update from Model
    //

    void insert() {/*_I*/_prof.prof.cnt[3010]++;/*I_*/
       /*_I*/_prof.prof.cnt[3011]++;/*I_*/ListIterator it = getModel().childrenFolded();
       /*_I*/{_prof.prof.cnt[3012]++;/*I_*/while(it.hasNext()) {               
          /*_I*/{_prof.prof.cnt[3013]++;/*I_*/insert((MindMapNode)it.next());/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
    }

    /**
     * Create views for the newNode and all his descendants, set their
     * isLeft attribute according to this view. Observe that views know
     * about their parents only through their models.
     */

    void insert(MindMapNode newNode) {/*_I*/_prof.prof.cnt[3014]++;/*I_*/
       /*_I*/_prof.prof.cnt[3015]++;/*I_*/NodeView newView = NodeView.newNodeView(newNode,getMap());
       /*_I*/{_prof.prof.cnt[3016]++;/*I_*/newView.setLeft(this.isLeft());/*_I*/}/*I_*/

       /*_I*/_prof.prof.cnt[3017]++;/*I_*/ListIterator it = newNode.childrenFolded();
       /*_I*/{_prof.prof.cnt[3018]++;/*I_*/while (it.hasNext()) {
          /*_I*/_prof.prof.cnt[3019]++;/*I_*/MindMapNode child = (MindMapNode)it.next();
          /*_I*/{_prof.prof.cnt[3020]++;/*I_*/newView.insert(child);/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
    }
    
    /**
     * This is a bit problematic, because getChildrenViews() only works if 
     * model is not yet removed. (So do not _really_ delete the model before the view 
     * removed (it needs to stay in memory)
     */
    void remove() {/*_I*/_prof.prof.cnt[3021]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3022]++;/*I_*/removeFromMap();/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3023]++;/*I_*/if (getEdge()!=null) {
           /*_I*/{_prof.prof.cnt[3024]++;/*I_*/getEdge().remove();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3025]++;/*I_*/getModel().setViewer(null);/*_I*/}/*I_*/ // Let the model know he is invisible
	/*_I*/{_prof.prof.cnt[3026]++;/*I_*/for(ListIterator e = getChildrenViews().listIterator();e.hasNext();) {
           /*_I*/{_prof.prof.cnt[3027]++;/*I_*/((NodeView)e.next()).remove();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

     void update() {/*_I*/_prof.prof.cnt[3028]++;/*I_*/
        //System.err.println("update");
        // 1) Set color
        /*_I*/_prof.prof.cnt[3029]++;/*I_*/Color color = getModel().getColor();
        /*_I*/{_prof.prof.cnt[3030]++;/*I_*/if (color==null) {
        	/*_I*/{_prof.prof.cnt[3031]++;/*I_*/color = standardNodeColor;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3032]++;/*I_*/setForeground(color);/*_I*/}/*I_*/

        // 2) icons left or right? 
        /*_I*/{_prof.prof.cnt[3033]++;/*I_*/setHorizontalTextPosition((getModel().isOneLeftSideOfRoot())?SwingConstants.LEADING:SwingConstants.TRAILING);/*_I*/}/*I_*/
        // 3) Create the icons:
        /*_I*/_prof.prof.cnt[3034]++;/*I_*/MultipleImage iconImages = new MultipleImage(map.getZoom());
        /*_I*/_prof.prof.cnt[3035]++;/*I_*/boolean iconPresent = false;
        /* fc, 06.10.2003: images?*/
        
        /*_I*/_prof.prof.cnt[3036]++;/*I_*/FreeMindMain frame = map.getController().getFrame();
        /*_I*/_prof.prof.cnt[3037]++;/*I_*/Map stateIcons = (getModel()).getStateIcons();
        /*_I*/{_prof.prof.cnt[3038]++;/*I_*/for (Iterator i = stateIcons.keySet().iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[3039]++;/*I_*/String key = (String) i.next();
            /*_I*/{_prof.prof.cnt[3040]++;/*I_*/iconPresent = true;/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[3041]++;/*I_*/ImageIcon myIcon = (ImageIcon) stateIcons.get(key);
            /*_I*/{_prof.prof.cnt[3042]++;/*I_*/iconImages.addImage(myIcon);/*_I*/}/*I_*/  
            
        }/*_I*/}/*I_*/

        /*_I*/_prof.prof.cnt[3043]++;/*I_*/List icons = (getModel()).getIcons();
        	/*_I*/{_prof.prof.cnt[3044]++;/*I_*/for (Iterator i = icons.iterator(); i.hasNext();) {
			/*_I*/_prof.prof.cnt[3045]++;/*I_*/MindIcon myIcon = (MindIcon) i.next();
            /*_I*/{_prof.prof.cnt[3046]++;/*I_*/iconPresent = true;/*_I*/}/*I_*/
            //System.out.println("print the icon " + myicon.toString());
            /*_I*/{_prof.prof.cnt[3047]++;/*I_*/iconImages.addImage(myIcon.getIcon(frame));/*_I*/}/*I_*/  
        }/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[3048]++;/*I_*/String link = ((NodeAdapter)getModel()).getLink();
        /*_I*/{_prof.prof.cnt[3049]++;/*I_*/if ( link != null ) 
            {
                /*_I*/{_prof.prof.cnt[3050]++;/*I_*/iconPresent = true;/*_I*/}/*I_*/
                /*_I*/_prof.prof.cnt[3051]++;/*I_*/ImageIcon icon = new ImageIcon(frame.getResource
                                          (link.startsWith("mailto:") ? "images/Mail.png" :
                                           (Tools.executableByExtension(link) ? "images/Executable.png" :
                                            "images/Link.png")));
                /*_I*/{_prof.prof.cnt[3052]++;/*I_*/iconImages.addImage(icon);/*_I*/}/*I_*/ 
            }/*_I*/}/*I_*/
//         /* Folded icon by Matthias Schade (mascha2), fc, 20.12.2003*/
//         if (((NodeAdapter)getModel()).isFolded()) {
//             iconPresent = true;
//             ImageIcon icon = new ImageIcon(((NodeAdapter)getModel()).getFrame().getResource("images/Folded.png"));
//             iconImages.addImage(icon); 
//         }
        // DanielPolansky: set icon only if icon is present, because
        // we don't want to insert any additional white space.        
        /*_I*/{_prof.prof.cnt[3053]++;/*I_*/setIcon(iconPresent?iconImages:null);/*_I*/}/*I_*/

        // 4) Determine font
        /*_I*/_prof.prof.cnt[3054]++;/*I_*/Font font = getModel().getFont();
        /*_I*/{_prof.prof.cnt[3055]++;/*I_*/font = font == null ? map.getController().getDefaultFont() : font;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3056]++;/*I_*/if (font != null) {
           /*_I*/{_prof.prof.cnt[3057]++;/*I_*/if (map.getZoom() != 1F) {
              /*_I*/{_prof.prof.cnt[3058]++;/*I_*/font = font.deriveFont(font.getSize()*map.getZoom());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[3059]++;/*I_*/setFont(font);/*_I*/}/*I_*/ }
        else {
           // We can survive this trouble.
           /*_I*/{_prof.prof.cnt[3060]++;/*I_*/System.err.println("NodeView.update(): default font is null.");/*_I*/}/*I_*/ }/*_I*/}/*I_*/

        // 5) Set the text
        // Right now, this implementation is quite logical, although it allows
        // for nonconvex feature of nodes starting with <html>.

//        String nodeText = getModel().toString();
        /*_I*/_prof.prof.cnt[3061]++;/*I_*/String nodeText = getModel().toString();
 
        // Tell if node is long and its width has to be restricted
        // boolean isMultiline = nodeText.indexOf("\n") >= 0;
        /*_I*/_prof.prof.cnt[3062]++;/*I_*/String[] lines = nodeText.split("\n");
        /*_I*/_prof.prof.cnt[3063]++;/*I_*/boolean widthMustBeRestricted = false;

        /*_I*/{_prof.prof.cnt[3064]++;/*I_*/lines = nodeText.split("\n");/*_I*/}/*I_*/           
        /*_I*/{_prof.prof.cnt[3065]++;/*I_*/for (int line = 0; line < lines.length; line++)
        {
            // Compute the width the node would spontaneously take,
            // by preliminarily setting the text.
            /*_I*/{_prof.prof.cnt[3066]++;/*I_*/setText(lines[line]);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3067]++;/*I_*/widthMustBeRestricted = getPreferredSize().width > map
                    .getZoomed(map.getMaxNodeWidth());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3068]++;/*I_*/if (widthMustBeRestricted)
            {
                /*_I*/{_prof.prof.cnt[3069]++;/*I_*/break;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[3070]++;/*I_*/isLong = widthMustBeRestricted || lines.length > 1;/*_I*/}/*I_*/
   
        /*_I*/{_prof.prof.cnt[3071]++;/*I_*/if (nodeText.startsWith("<html>")) {
           // Make it possible to use relative img references in HTML using tag <base>.
           /*_I*/{_prof.prof.cnt[3072]++;/*I_*/if (nodeText.indexOf("<img")>=0 && nodeText.indexOf("<base ") < 0 ) {
              /*_I*/{_prof.prof.cnt[3073]++;/*I_*/try {
                 /*_I*/{_prof.prof.cnt[3074]++;/*I_*/nodeText = "<html><base href=\""+
                    map.getModel().getURL()+"\">"+nodeText.substring(6);/*_I*/}/*I_*/ }
              catch (MalformedURLException e) {}/*_I*/}/*I_*/ }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[3075]++;/*I_*/setText(nodeText);/*_I*/}/*I_*/ }
        else /*_I*/{_prof.prof.cnt[3076]++;/*I_*/if (nodeText.startsWith("<table>")) {           	             	  
           /*_I*/{_prof.prof.cnt[3077]++;/*I_*/lines[0] = lines[0].substring(7);/*_I*/}/*I_*/ // remove <table> tag
           /*_I*/_prof.prof.cnt[3078]++;/*I_*/int startingLine = lines[0].matches("\\s*") ? 1 : 0;
           // ^ If the remaining first line is empty, do not draw it
           
           /*_I*/_prof.prof.cnt[3079]++;/*I_*/String text = "<html><table border=1 style=\"border-color: white\">";
           //String[] lines = nodeText.split("\n");
           /*_I*/{_prof.prof.cnt[3080]++;/*I_*/for (int line = startingLine; line < lines.length; line++) {
              /*_I*/{_prof.prof.cnt[3081]++;/*I_*/text += "<tr><td style=\"border-color: white;\">"+
                 Tools.toXMLEscapedText(lines[line]).replaceAll("\t","<td style=\"border-color: white\">");/*_I*/}/*I_*/ }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[3082]++;/*I_*/setText(text);/*_I*/}/*I_*/ }
        else /*_I*/{_prof.prof.cnt[3083]++;/*I_*/if (isLong) {
           /*_I*/_prof.prof.cnt[3084]++;/*I_*/String text = "<tr><td>";              
           /*_I*/_prof.prof.cnt[3085]++;/*I_*/int maximumLineLength = 0;
           /*_I*/{_prof.prof.cnt[3086]++;/*I_*/for (int line = 0; line < lines.length; line++) {
              /*_I*/{_prof.prof.cnt[3087]++;/*I_*/text += Tools.toXMLEscapedTextWithNBSPizedSpaces(lines[line]) + "<p>";/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[3088]++;/*I_*/if (lines[line].length() > maximumLineLength) {
                 /*_I*/{_prof.prof.cnt[3089]++;/*I_*/maximumLineLength = lines[line].length();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/
           
			/*_I*/{_prof.prof.cnt[3090]++;/*I_*/text += "</td></tr>";/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[3091]++;/*I_*/setText("<html><table"+
                   (!widthMustBeRestricted?">":" width=\""+map.getZoomed(map.getMaxNodeWidth())+"\">")+
                   text+"</table></html>");/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
   		// 6) ToolTips:
        /*_I*/{_prof.prof.cnt[3092]++;/*I_*/updateToolTip();/*_I*/}/*I_*/
        // 7) Complete
        /*_I*/{_prof.prof.cnt[3093]++;/*I_*/repaint();/*_I*/}/*I_*/ // Because of zoom?
    }
    /**
     * Updates the tool tip of the node.
     */
    public void updateToolTip() {/*_I*/_prof.prof.cnt[3094]++;/*I_*/
        /*_I*/_prof.prof.cnt[3095]++;/*I_*/Map tooltips = getModel().getToolTip();
        /*_I*/{_prof.prof.cnt[3096]++;/*I_*/if(tooltips.size() == 1) {
            /*_I*/{_prof.prof.cnt[3097]++;/*I_*/setToolTipText((String) tooltips.values().iterator().next());/*_I*/}/*I_*/
        } else /*_I*/{_prof.prof.cnt[3098]++;/*I_*/if (tooltips.size()==0) {
																/*_I*/{_prof.prof.cnt[3099]++;/*I_*/setToolTipText(null);/*_I*/}/*I_*/
								} else {
            // html table
            /*_I*/_prof.prof.cnt[3100]++;/*I_*/StringBuffer text = new StringBuffer("<html><table>");
            /*_I*/_prof.prof.cnt[3101]++;/*I_*/Set keySet = tooltips.keySet();
            /*_I*/_prof.prof.cnt[3102]++;/*I_*/TreeSet sortedKeySet = new TreeSet();
            /*_I*/{_prof.prof.cnt[3103]++;/*I_*/sortedKeySet.addAll(keySet);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3104]++;/*I_*/for (Iterator i = sortedKeySet.iterator(); i.hasNext();) {
                /*_I*/_prof.prof.cnt[3105]++;/*I_*/String key = (String) i.next();
                /*_I*/_prof.prof.cnt[3106]++;/*I_*/String value = (String) tooltips.get(key);
                /*_I*/{_prof.prof.cnt[3107]++;/*I_*/text.append("<tr><td>");/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[3108]++;/*I_*/text.append(value);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[3109]++;/*I_*/text.append("</td></tr>");/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3110]++;/*I_*/text.append("</table></html>");/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3111]++;/*I_*/setToolTipText(text.toString());/*_I*/}/*I_*/
        }/*_I*/}/*I_*//*_I*/}/*I_*/
    }

    void updateAll() {/*_I*/_prof.prof.cnt[3112]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3113]++;/*I_*/update();/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3114]++;/*I_*/for(ListIterator e = getChildrenViews().listIterator();e.hasNext();) {
	    /*_I*/_prof.prof.cnt[3115]++;/*I_*/NodeView child = (NodeView)e.next();
	    /*_I*/{_prof.prof.cnt[3116]++;/*I_*/child.updateAll();/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

   protected void setRendering(Graphics2D g) {/*_I*/_prof.prof.cnt[3117]++;/*I_*/
      /*_I*/{_prof.prof.cnt[3118]++;/*I_*/if (map.getController().getAntialiasAll()) {
         /*_I*/{_prof.prof.cnt[3119]++;/*I_*/g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
//       else
//          g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF); 

   }

   abstract String getStyle() ;
   
    /**
     * @return the shift of the tree root node
     * relative to the middle of the tree
     * because of the light shift of the children nodes
     */
    public int getTreeHeight() {/*_I*/_prof.prof.cnt[3120]++;/*I_*/
        /*_I*/{_prof.prof.cnt[3121]++;/*I_*/return treeHeight;/*_I*/}/*I_*/
    }

	/**
	 * sets the shift of the tree root node
	 * relative to the middle of the tree
	 * because of the light shift of the children nodes.
	 */
    public void setTreeHeight(int i) {/*_I*/_prof.prof.cnt[3122]++;/*I_*/
        /*_I*/{_prof.prof.cnt[3123]++;/*I_*/treeHeight = i;/*_I*/}/*I_*/
    }

    public int getTreeWidth() {/*_I*/_prof.prof.cnt[3124]++;/*I_*/
        /*_I*/{_prof.prof.cnt[3125]++;/*I_*/return treeWidth;/*_I*/}/*I_*/
    }

    public void setTreeWidth(int i) {/*_I*/_prof.prof.cnt[3126]++;/*I_*/
        /*_I*/{_prof.prof.cnt[3127]++;/*I_*/treeWidth = i;/*_I*/}/*I_*/
    }

    public int getZoomedFoldingSymbolHalfWidth() {/*_I*/_prof.prof.cnt[3128]++;/*I_*/
    	/*_I*/_prof.prof.cnt[3129]++;/*I_*/int preferredFoldingSymbolHalfWidth = map.getZoomedFoldingSymbolHalfWidth();
        /*_I*/{_prof.prof.cnt[3130]++;/*I_*/return Math.min(preferredFoldingSymbolHalfWidth, super.getPreferredSize().height / 2);/*_I*/}/*I_*/
    }

    /**
     * @return returns the color that should used to select the node.
     */
    protected Color getSelectedColor() {/*_I*/_prof.prof.cnt[3131]++;/*I_*/
//		Color backgroundColor = getModel().getBackgroundColor();
////        if(backgroundColor != null) {
////			Color backBrighter = backgroundColor.brighter();
////			// white?
////			if(backBrighter.getRGB() == Color.WHITE.getRGB()) {
////				return standardSelectColor;
////			}
////			// == standard??
////            if (backBrighter.equals (standardSelectColor) ) {
////                return backgroundColor.darker();
////            }
////            return backBrighter;
////		}
//		// == standard??
//		  if (backgroundColor != null /*&& backgroundColor.equals(standardSelectColor)*/ ) {
//		  	// bad hack:
//		  	return getAntiColor1(backgroundColor);
////			  return new Color(0xFFFFFF - backgroundColor.getRGB());
//		  }
        /*_I*/{_prof.prof.cnt[3132]++;/*I_*/return standardSelectColor;/*_I*/}/*I_*/
    }

/* http://groups.google.de/groups?hl=de&lr=&ie=UTF-8&threadm=9i5bbo%24h1kmi%243%40ID-77081.news.dfncis.de&rnum=1&prev=/groups%3Fq%3Djava%2520komplement%25C3%25A4rfarbe%2520helligkeit%26hl%3Dde%26lr%3D%26ie%3DUTF-8%26sa%3DN%26as_qdr%3Dall%26tab%3Dwg */
	/**
	 * Ermittelt zu einer Farbe eine andere Farbe, die sich möglichst gut von
	 * dieser abhebt.
	 * Diese Farbe unterscheidet sich auch von {@link #getAntiColor2}.
	 * @since PPS 1.1.1
	 */
   protected static Color getAntiColor1(Color c)
   {/*_I*/_prof.prof.cnt[3133]++;/*I_*/
	/*_I*/_prof.prof.cnt[3134]++;/*I_*/float[] hsb = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
	/*_I*/{_prof.prof.cnt[3135]++;/*I_*/hsb[0] += 0.40;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3136]++;/*I_*/if (hsb[0] > 1)
	 /*_I*/{_prof.prof.cnt[3137]++;/*I_*/hsb[0]--;/*_I*/}/*I_*//*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3138]++;/*I_*/hsb[1] = 1;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3139]++;/*I_*/hsb[2] = 0.7f;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3140]++;/*I_*/return Color.getHSBColor(hsb[0], hsb[1], hsb[2]);/*_I*/}/*I_*/
   }

   /**
	* Ermittelt zu einer Farbe eine andere Farbe, die sich möglichst gut von
	* dieser abhebt.
	* Diese Farbe unterscheidet sich von {@link #getAntiColor1}.
	* @since PPS 1.1.1
	*/
  protected static Color getAntiColor2(Color c)
  {/*_I*/_prof.prof.cnt[3141]++;/*I_*/
   /*_I*/_prof.prof.cnt[3142]++;/*I_*/float[] hsb = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
   /*_I*/{_prof.prof.cnt[3143]++;/*I_*/hsb[0] -= 0.40;/*_I*/}/*I_*/
   /*_I*/{_prof.prof.cnt[3144]++;/*I_*/if (hsb[0] < 0)
	/*_I*/{_prof.prof.cnt[3145]++;/*I_*/hsb[0]++;/*_I*/}/*I_*//*_I*/}/*I_*/
   /*_I*/{_prof.prof.cnt[3146]++;/*I_*/hsb[1] = 1;/*_I*/}/*I_*/
   /*_I*/{_prof.prof.cnt[3147]++;/*I_*/hsb[2] = (float)0.8;/*_I*/}/*I_*/
   /*_I*/{_prof.prof.cnt[3148]++;/*I_*/return Color.getHSBColor(hsb[0], hsb[1], hsb[2]);/*_I*/}/*I_*/
  }

	/**
	 * @return Returns the sHIFT.
	 */

  public int getShift() {/*_I*/_prof.prof.cnt[3149]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3150]++;/*I_*/return map.getZoomed(model.calcShiftY());/*_I*/}/*I_*/
}
  

    /**
	 * @return Returns the VGAP.
	 */
	public int getVGap() {/*_I*/_prof.prof.cnt[3151]++;/*I_*/		
        /*_I*/{_prof.prof.cnt[3152]++;/*I_*/return  map.getZoomed(model.calcVGap());/*_I*/}/*I_*/
        // TODO 
	}

	public int getHGap() {/*_I*/_prof.prof.cnt[3153]++;/*I_*/
		/*_I*/{_prof.prof.cnt[3154]++;/*I_*/return  map.getZoomed(model.getHGap());/*_I*/}/*I_*/
	}


	public int getUpperChildShift() {/*_I*/_prof.prof.cnt[3155]++;/*I_*/
		/*_I*/{_prof.prof.cnt[3156]++;/*I_*/return upperChildShift;/*_I*/}/*I_*/
	}
	public void setUpperChildShift(int treeShift) {/*_I*/_prof.prof.cnt[3157]++;/*I_*/
		/*_I*/{_prof.prof.cnt[3158]++;/*I_*/this.upperChildShift = treeShift;/*_I*/}/*I_*/
	}
	public NodeMotionListenerView getMotionListenerView() {/*_I*/_prof.prof.cnt[3159]++;/*I_*/
		/*_I*/{_prof.prof.cnt[3160]++;/*I_*/return null;/*_I*/}/*I_*/
	}
}
